/*!
 * jQuery UI Bootstrap (0.22)
 * http://addyosmani.github.com/jquery-ui-bootstrap
 *
 * Copyright 2012, Addy Osmani
 * Dual licensed under the MIT or GPL Version 2 licenses.
 *
 * Portions copyright jQuery UI & Twitter Bootstrap
 */


/* Layout helpers
----------------------------------*/
.ui-helper-hidden { display: none; }
.ui-helper-hidden-accessible { position: absolute !important; clip: rect(1px 1px 1px 1px); clip: rect(1px,1px,1px,1px); }
.ui-helper-reset { margin: 0; padding: 0; border: 0; outline: 0; line-height: 1.3; text-decoration: none; font-size: 100%; list-style: none; }
.ui-helper-clearfix:after { content: "."; display: block; height: 0; clear: both; visibility: hidden; }
.ui-helper-clearfix { display: inline-block; }
/* required comment for clearfix to work in Opera \*/
* html .ui-helper-clearfix { height:1%; }
.ui-helper-clearfix { display:block; }
/* end clearfix */
.ui-helper-zfix { width: 100%; height: 100%; top: 0; left: 0; position: absolute; opacity: 0; filter:Alpha(Opacity=0); }


/* Interaction Cues
----------------------------------*/
.ui-state-disabled { cursor: default !important; }


/* Icons
----------------------------------*/

/* states and images */
.ui-icon { display: block; text-indent: -99999px; overflow: hidden; background-repeat: no-repeat; }


/* Misc visuals
----------------------------------*/

/* Overlays */
.ui-widget-overlay { position: absolute; top: 0; left: 0; width: 100%; height: 100%; }


/*
 * jQuery UI CSS Framework 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Theming/API
 *
 * To view and modify this theme, visit http://jqueryui.com/themeroller/?ctl=themeroller
 */


/* Component containers
----------------------------------*/
.ui-widget {   font-family: "Helvetica Neue", Helvetica, Arial, sans-serif; font-size:13px; }
.ui-widget .ui-widget { font-size: 1em; }
.ui-widget input, .ui-widget select, .ui-widget textarea, .ui-widget button { font-family: "Helvetica Neue", Helvetica, Arial, sans-serif; font-size: 1em; }
.ui-widget-content { border: 1px solid #aaaaaa; background: #ffffff url(images/ui-bg_glass_75_ffffff_1x400.png) 50% 50% repeat-x; color: #404040; }
.ui-widget-content a { color: #404040; }
.ui-widget-header {
  font-weight:bold;
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  border:1px solid #666;

 }
.ui-widget-header a { color: #222222; }

/* Interaction states
----------------------------------*/
.ui-state-default, .ui-widget-content .ui-state-default, .ui-widget-header .ui-state-default {

  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);

  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);

  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear background-image;
  -moz-transition: 0.1s linear background-image;
  -ms-transition: 0.1s linear background-image;
  -o-transition: 0.1s linear background-image;
  transition: 0.1s linear background-image;
   overflow: visible;

 }


.ui-state-default a, .ui-state-default a:link, .ui-state-default a:visited { color: #555555; text-decoration: none; }
.ui-state-hover, .ui-widget-content .ui-state-hover, .ui-widget-header .ui-state-hover, .ui-state-focus, .ui-widget-content .ui-state-focus, .ui-widget-header .ui-state-focus {
  background-position: 0 -15px;
  color: #333;
  text-decoration: none;


 }
.ui-state-hover a, .ui-state-hover a:hover { color: #212121; text-decoration: none; }
.ui-state-active, .ui-widget-content .ui-state-active, .ui-widget-header .ui-state-active { border: 1px solid #aaaaaa;  font-weight: normal; color: #212121; }
.ui-state-active a, .ui-state-active a:link, .ui-state-active a:visited { color: #212121; text-decoration: none; }
.ui-widget :active { outline: none; }

/* Interaction Cues
----------------------------------*/


.ui-state-highlight p, .ui-state-error p, .ui-state-default p{
  font-size: 13px;
  font-weight: normal;
  line-height: 18px;
  margin:7px 15px;
}
.ui-state-highlight, .ui-widget-content .ui-state-highlight, .ui-widget-header .ui-state-highlight  {


  position: relative;
  margin-bottom: 18px;
  color: #404040;
  background-color: #eedc94;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#fceec1), to(#eedc94));
  background-image: -moz-linear-gradient(top, #fceec1, #eedc94);
  background-image: -ms-linear-gradient(top, #fceec1, #eedc94);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #fceec1), color-stop(100%, #eedc94));
  background-image: -webkit-linear-gradient(top, #fceec1, #eedc94);
  background-image: -o-linear-gradient(top, #fceec1, #eedc94);
  background-image: linear-gradient(top, #fceec1, #eedc94);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fceec1', endColorstr='#eedc94', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #eedc94 #eedc94 #e4c652;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
  border-width: 1px;
  border-style: solid;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);


}
.ui-state-highlight a, .ui-widget-content .ui-state-highlight a,.ui-widget-header .ui-state-highlight a { color: #363636; }
.ui-state-error, .ui-widget-content .ui-state-error, .ui-widget-header .ui-state-error {


  position: relative;
  margin-bottom: 18px;
  color: #ffffff;
  border-width: 1px;
  border-style: solid;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25);
 background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee5f5b), to(#c43c35));
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee5f5b), color-stop(100%, #c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);


}
.ui-state-error a, .ui-widget-content .ui-state-error a, .ui-widget-header .ui-state-error a { color: #cd0a0a; }
.ui-state-error-text, .ui-widget-content .ui-state-error-text, .ui-widget-header .ui-state-error-text { color: #cd0a0a; }
.ui-priority-primary, .ui-widget-content .ui-priority-primary, .ui-widget-header .ui-priority-primary { font-weight: bold; }
.ui-priority-secondary, .ui-widget-content .ui-priority-secondary,  .ui-widget-header .ui-priority-secondary { opacity: .7; filter:Alpha(Opacity=70); font-weight: normal; }
.ui-state-disabled, .ui-widget-content .ui-state-disabled, .ui-widget-header .ui-state-disabled { opacity: .35; filter:Alpha(Opacity=35); background-image: none;  }



/* Icons
----------------------------------*/

/* states and images */
.ui-icon { width: 16px; height: 16px; background-image: url(images/ui-icons_222222_256x240.png); }
.ui-widget-content .ui-icon {background-image: url(images/ui-icons_222222_256x240.png); }
.ui-widget-header .ui-icon {background-image: url(images/ui-icons_222222_256x240.png); }
.ui-state-default .ui-icon { background-image: url(images/ui-icons_888888_256x240.png); }
.ui-state-hover .ui-icon, .ui-state-focus .ui-icon {background-image: url(images/ui-icons_454545_256x240.png); }
.ui-state-active .ui-icon {background-image: url(images/ui-icons_454545_256x240.png); }
.ui-state-highlight .ui-icon {background-image: url(images/ui-icons_2e83ff_256x240.png); }
.ui-state-error .ui-icon, .ui-state-error-text .ui-icon {background-image: url(images/ui-icons_f6cf3b_256x240.png); }

/* positioning */
.ui-icon-carat-1-n { background-position: 0 0; }
.ui-icon-carat-1-ne { background-position: -16px 0; }
.ui-icon-carat-1-e { background-position: -32px 0; }
.ui-icon-carat-1-se { background-position: -48px 0; }
.ui-icon-carat-1-s { background-position: -64px 0; }
.ui-icon-carat-1-sw { background-position: -80px 0; }
.ui-icon-carat-1-w { background-position: -96px 0; }
.ui-icon-carat-1-nw { background-position: -112px 0; }
.ui-icon-carat-2-n-s { background-position: -128px 0; }
.ui-icon-carat-2-e-w { background-position: -144px 0; }
.ui-icon-triangle-1-n { background-position: 0 -16px; }
.ui-icon-triangle-1-ne { background-position: -16px -16px; }
.ui-icon-triangle-1-e { background-position: -32px -16px; }
.ui-icon-triangle-1-se { background-position: -48px -16px; }
.ui-icon-triangle-1-s { background-position: -64px -16px; }
.ui-icon-triangle-1-sw { background-position: -80px -16px; }
.ui-icon-triangle-1-w { background-position: -96px -16px; }
.ui-icon-triangle-1-nw { background-position: -112px -16px; }
.ui-icon-triangle-2-n-s { background-position: -128px -16px; }
.ui-icon-triangle-2-e-w { background-position: -144px -16px; }
.ui-icon-arrow-1-n { background-position: 0 -32px; }
.ui-icon-arrow-1-ne { background-position: -16px -32px; }
.ui-icon-arrow-1-e { background-position: -32px -32px; }
.ui-icon-arrow-1-se { background-position: -48px -32px; }
.ui-icon-arrow-1-s { background-position: -64px -32px; }
.ui-icon-arrow-1-sw { background-position: -80px -32px; }
.ui-icon-arrow-1-w { background-position: -96px -32px; }
.ui-icon-arrow-1-nw { background-position: -112px -32px; }
.ui-icon-arrow-2-n-s { background-position: -128px -32px; }
.ui-icon-arrow-2-ne-sw { background-position: -144px -32px; }
.ui-icon-arrow-2-e-w { background-position: -160px -32px; }
.ui-icon-arrow-2-se-nw { background-position: -176px -32px; }
.ui-icon-arrowstop-1-n { background-position: -192px -32px; }
.ui-icon-arrowstop-1-e { background-position: -208px -32px; }
.ui-icon-arrowstop-1-s { background-position: -224px -32px; }
.ui-icon-arrowstop-1-w { background-position: -240px -32px; }
.ui-icon-arrowthick-1-n { background-position: 0 -48px; }
.ui-icon-arrowthick-1-ne { background-position: -16px -48px; }
.ui-icon-arrowthick-1-e { background-position: -32px -48px; }
.ui-icon-arrowthick-1-se { background-position: -48px -48px; }
.ui-icon-arrowthick-1-s { background-position: -64px -48px; }
.ui-icon-arrowthick-1-sw { background-position: -80px -48px; }
.ui-icon-arrowthick-1-w { background-position: -96px -48px; }
.ui-icon-arrowthick-1-nw { background-position: -112px -48px; }
.ui-icon-arrowthick-2-n-s { background-position: -128px -48px; }
.ui-icon-arrowthick-2-ne-sw { background-position: -144px -48px; }
.ui-icon-arrowthick-2-e-w { background-position: -160px -48px; }
.ui-icon-arrowthick-2-se-nw { background-position: -176px -48px; }
.ui-icon-arrowthickstop-1-n { background-position: -192px -48px; }
.ui-icon-arrowthickstop-1-e { background-position: -208px -48px; }
.ui-icon-arrowthickstop-1-s { background-position: -224px -48px; }
.ui-icon-arrowthickstop-1-w { background-position: -240px -48px; }
.ui-icon-arrowreturnthick-1-w { background-position: 0 -64px; }
.ui-icon-arrowreturnthick-1-n { background-position: -16px -64px; }
.ui-icon-arrowreturnthick-1-e { background-position: -32px -64px; }
.ui-icon-arrowreturnthick-1-s { background-position: -48px -64px; }
.ui-icon-arrowreturn-1-w { background-position: -64px -64px; }
.ui-icon-arrowreturn-1-n { background-position: -80px -64px; }
.ui-icon-arrowreturn-1-e { background-position: -96px -64px; }
.ui-icon-arrowreturn-1-s { background-position: -112px -64px; }
.ui-icon-arrowrefresh-1-w { background-position: -128px -64px; }
.ui-icon-arrowrefresh-1-n { background-position: -144px -64px; }
.ui-icon-arrowrefresh-1-e { background-position: -160px -64px; }
.ui-icon-arrowrefresh-1-s { background-position: -176px -64px; }
.ui-icon-arrow-4 { background-position: 0 -80px; }
.ui-icon-arrow-4-diag { background-position: -16px -80px; }
.ui-icon-extlink { background-position: -32px -80px; }
.ui-icon-newwin { background-position: -48px -80px; }
.ui-icon-refresh { background-position: -64px -80px; }
.ui-icon-shuffle { background-position: -80px -80px; }
.ui-icon-transfer-e-w { background-position: -96px -80px; }
.ui-icon-transferthick-e-w { background-position: -112px -80px; }
.ui-icon-folder-collapsed { background-position: 0 -96px; }
.ui-icon-folder-open { background-position: -16px -96px; }
.ui-icon-document { background-position: -32px -96px; }
.ui-icon-document-b { background-position: -48px -96px; }
.ui-icon-note { background-position: -64px -96px; }
.ui-icon-mail-closed { background-position: -80px -96px; }
.ui-icon-mail-open { background-position: -96px -96px; }
.ui-icon-suitcase { background-position: -112px -96px; }
.ui-icon-comment { background-position: -128px -96px; }
.ui-icon-person { background-position: -144px -96px; }
.ui-icon-print { background-position: -160px -96px; }
.ui-icon-trash { background-position: -176px -96px; }
.ui-icon-locked { background-position: -192px -96px; }
.ui-icon-unlocked { background-position: -208px -96px; }
.ui-icon-bookmark { background-position: -224px -96px; }
.ui-icon-tag { background-position: -240px -96px; }
.ui-icon-home { background-position: 0 -112px; }
.ui-icon-flag { background-position: -16px -112px; }
.ui-icon-calendar { background-position: -32px -112px; }
.ui-icon-cart { background-position: -48px -112px; }
.ui-icon-pencil { background-position: -64px -112px; }
.ui-icon-clock { background-position: -80px -112px; }
.ui-icon-disk { background-position: -96px -112px; }
.ui-icon-calculator { background-position: -112px -112px; }
.ui-icon-zoomin { background-position: -128px -112px; }
.ui-icon-zoomout { background-position: -144px -112px; }
.ui-icon-search { background-position: -160px -112px; }
.ui-icon-wrench { background-position: -176px -112px; }
.ui-icon-gear { background-position: -192px -112px; }
.ui-icon-heart { background-position: -208px -112px; }
.ui-icon-star { background-position: -224px -112px; }
.ui-icon-link { background-position: -240px -112px; }
.ui-icon-cancel { background-position: 0 -128px; }
.ui-icon-plus { background-position: -16px -128px; }
.ui-icon-plusthick { background-position: -32px -128px; }
.ui-icon-minus { background-position: -48px -128px; }
.ui-icon-minusthick { background-position: -64px -128px; }
.ui-icon-close { background-position: -80px -128px; }
.ui-icon-closethick { background-position: -96px -128px; }
.ui-icon-key { background-position: -112px -128px; }
.ui-icon-lightbulb { background-position: -128px -128px; }
.ui-icon-scissors { background-position: -144px -128px; }
.ui-icon-clipboard { background-position: -160px -128px; }
.ui-icon-copy { background-position: -176px -128px; }
.ui-icon-contact { background-position: -192px -128px; }
.ui-icon-image { background-position: -208px -128px; }
.ui-icon-video { background-position: -224px -128px; }
.ui-icon-script { background-position: -240px -128px; }
.ui-icon-alert { background-position: 0 -144px; }
.ui-icon-info { background-position: -16px -144px; }
.ui-icon-notice { background-position: -32px -144px; }
.ui-icon-help { background-position: -48px -144px; }
.ui-icon-check { background-position: -64px -144px; }
.ui-icon-bullet { background-position: -80px -144px; }
.ui-icon-radio-off { background-position: -96px -144px; }
.ui-icon-radio-on { background-position: -112px -144px; }
.ui-icon-pin-w { background-position: -128px -144px; }
.ui-icon-pin-s { background-position: -144px -144px; }
.ui-icon-play { background-position: 0 -160px; }
.ui-icon-pause { background-position: -16px -160px; }
.ui-icon-seek-next { background-position: -32px -160px; }
.ui-icon-seek-prev { background-position: -48px -160px; }
.ui-icon-seek-end { background-position: -64px -160px; }
.ui-icon-seek-start { background-position: -80px -160px; }
/* ui-icon-seek-first is deprecated, use ui-icon-seek-start instead */
.ui-icon-seek-first { background-position: -80px -160px; }
.ui-icon-stop { background-position: -96px -160px; }
.ui-icon-eject { background-position: -112px -160px; }
.ui-icon-volume-off { background-position: -128px -160px; }
.ui-icon-volume-on { background-position: -144px -160px; }
.ui-icon-power { background-position: 0 -176px; }
.ui-icon-signal-diag { background-position: -16px -176px; }
.ui-icon-signal { background-position: -32px -176px; }
.ui-icon-battery-0 { background-position: -48px -176px; }
.ui-icon-battery-1 { background-position: -64px -176px; }
.ui-icon-battery-2 { background-position: -80px -176px; }
.ui-icon-battery-3 { background-position: -96px -176px; }
.ui-icon-circle-plus { background-position: 0 -192px; }
.ui-icon-circle-minus { background-position: -16px -192px; }
.ui-icon-circle-close { background-position: -32px -192px; }
.ui-icon-circle-triangle-e { background-position: -48px -192px; }
.ui-icon-circle-triangle-s { background-position: -64px -192px; }
.ui-icon-circle-triangle-w { background-position: -80px -192px; }
.ui-icon-circle-triangle-n { background-position: -96px -192px; }
.ui-icon-circle-arrow-e { background-position: -112px -192px; }
.ui-icon-circle-arrow-s { background-position: -128px -192px; }
.ui-icon-circle-arrow-w { background-position: -144px -192px; }
.ui-icon-circle-arrow-n { background-position: -160px -192px; }
.ui-icon-circle-zoomin { background-position: -176px -192px; }
.ui-icon-circle-zoomout { background-position: -192px -192px; }
.ui-icon-circle-check { background-position: -208px -192px; }
.ui-icon-circlesmall-plus { background-position: 0 -208px; }
.ui-icon-circlesmall-minus { background-position: -16px -208px; }
.ui-icon-circlesmall-close { background-position: -32px -208px; }
.ui-icon-squaresmall-plus { background-position: -48px -208px; }
.ui-icon-squaresmall-minus { background-position: -64px -208px; }
.ui-icon-squaresmall-close { background-position: -80px -208px; }
.ui-icon-grip-dotted-vertical { background-position: 0 -224px; }
.ui-icon-grip-dotted-horizontal { background-position: -16px -224px; }
.ui-icon-grip-solid-vertical { background-position: -32px -224px; }
.ui-icon-grip-solid-horizontal { background-position: -48px -224px; }
.ui-icon-gripsmall-diagonal-se { background-position: -64px -224px; }
.ui-icon-grip-diagonal-se { background-position: -80px -224px; }


/* Misc visuals
----------------------------------*/

/* Corner radius */
.ui-corner-all, .ui-corner-top, .ui-corner-left, .ui-corner-tl { -moz-border-radius-topleft: 4px; -webkit-border-top-left-radius: 4px; -khtml-border-top-left-radius: 4px; border-top-left-radius: 4px; }
.ui-corner-all, .ui-corner-top, .ui-corner-right, .ui-corner-tr { -moz-border-radius-topright: 4px; -webkit-border-top-right-radius: 4px; -khtml-border-top-right-radius: 4px; border-top-right-radius: 4px; }
.ui-corner-all, .ui-corner-bottom, .ui-corner-left, .ui-corner-bl { -moz-border-radius-bottomleft: 4px; -webkit-border-bottom-left-radius: 4px; -khtml-border-bottom-left-radius: 4px; border-bottom-left-radius: 4px; }
.ui-corner-all, .ui-corner-bottom, .ui-corner-right, .ui-corner-br { -moz-border-radius-bottomright: 4px; -webkit-border-bottom-right-radius: 4px; -khtml-border-bottom-right-radius: 4px; border-bottom-right-radius: 4px; }



/* Overlays */
.ui-widget-overlay { background: #aaaaaa url(images/ui-bg_flat_0_aaaaaa_40x100.png) 50% 50% repeat-x; opacity: .30;filter:Alpha(Opacity=30); }
.ui-widget-shadow { margin: -8px 0 0 -8px; padding: 8px; background: #aaaaaa url(images/ui-bg_flat_0_aaaaaa_40x100.png) 50% 50% repeat-x; opacity: .30;filter:Alpha(Opacity=30); -moz-border-radius: 8px; -khtml-border-radius: 8px; -webkit-border-radius: 8px; border-radius: 8px; }/*
 * jQuery UI Resizable 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Resizable#theming
 */

 

/*
 * jQuery UI Autocomplete 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Autocomplete#theming
 */
.ui-autocomplete { position: absolute; cursor: default; }

/* workarounds */
* html .ui-autocomplete { width:1px; } /* without this, the menu expands to 100% in IE6 */

/*
 * jQuery UI Menu 1.8.16
 *
 * Copyright 2010, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Menu#theming
 */
.ui-menu {
  list-style:none;
  padding: 2px;
  margin: 0;
  display:block;
  float: left;
}
.ui-menu .ui-menu {
  margin-top: -3px;
}
.ui-menu .ui-menu-item {
  margin:0;
  padding: 0;
  zoom: 1;
  float: left;
  clear: left;
  width: 100%;
}
.ui-menu .ui-menu-item a {
  text-decoration:none;
  display:block;
  padding:.2em .4em;
  line-height:1.5;
  zoom:1;
}
.ui-menu .ui-menu-item a.ui-state-hover,
.ui-menu .ui-menu-item a.ui-state-active {
  font-weight: normal;
  background:#0064CD;
  color:#fff
}


/*
 * jQuery UI Button 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Button#theming
 */
.ui-button {

  cursor: pointer;
  display: inline-block;
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  padding: 5px 14px 6px;
  margin: 0;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;

  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear background-image;
  -moz-transition: 0.1s linear background-image;
  -ms-transition: 0.1s linear background-image;
  -o-transition: 0.1s linear background-image;
  transition: 0.1s linear background-image;
   overflow: visible;

} /* the overflow property removes extra width in IE */

.ui-button-primary {
  color: #ffffff;
  background-color: #0064cd;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#049cdb), to(#0064cd));
  background-image: -moz-linear-gradient(top, #049cdb, #0064cd);
  background-image: -ms-linear-gradient(top, #049cdb, #0064cd);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #049cdb), color-stop(100%, #0064cd));
  background-image: -webkit-linear-gradient(top, #049cdb, #0064cd);
  background-image: -o-linear-gradient(top, #049cdb, #0064cd);
  background-image: linear-gradient(top, #049cdb, #0064cd);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#049cdb', endColorstr='#0064cd', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);

}



.ui-button-success{
  color:#ffffff;
  background-color: #57a957;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#62c462), to(#57a957));
  background-image: -moz-linear-gradient(top, #62c462, #57a957);
  background-image: -ms-linear-gradient(top, #62c462, #57a957);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #62c462), color-stop(100%, #57a957));
  background-image: -webkit-linear-gradient(top, #62c462, #57a957);
  background-image: -o-linear-gradient(top, #62c462, #57a957);
  background-image: linear-gradient(top, #62c462, #57a957);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#62c462', endColorstr='#57a957', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #57a957 #57a957 #3d773d;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}

.ui-button-error{
  color:#ffffff;
  background-color: #c43c35;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee5f5b), to(#c43c35));
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee5f5b), color-stop(100%, #c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #c43c35 #c43c35 #882a25;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}

.ui-button-icon-only { width: 2.2em; } /* to make room for the icon, a width needs to be set here */
button.ui-button-icon-only { } /* button elements seem to need a little more width */
.ui-button-icons-only { width: 3.4em; }
button.ui-button-icons-only { width: 3.7em; }

/*button text element */

.ui-button .ui-button-text { display: block;   }
.ui-button-text-only .ui-button-text {  }
.ui-button-icon-only .ui-button-text, .ui-button-icons-only .ui-button-text { padding: .4em; text-indent: -9999999px; /*tempfix*/ display:none;}
.ui-button-text-icon-primary .ui-button-text, .ui-button-text-icons .ui-button-text { padding: .4em 1em .4em 2.1em; }
.ui-button-text-icon-secondary .ui-button-text, .ui-button-text-icons .ui-button-text { padding: .4em 2.1em .4em 1em; }
.ui-button-text-icons .ui-button-text { padding-left: 2.1em; padding-right: 2.1em; }
/* no icon support for input elements, provide padding by default */
/* input.ui-button { padding: .4em 1em; } */

/*button icon element(s) */
.ui-button-icon-only .ui-icon, .ui-button-text-icon-primary .ui-icon, .ui-button-text-icon-secondary .ui-icon, .ui-button-text-icons .ui-icon, .ui-button-icons-only .ui-icon { top: 50%; margin-top:-3px; margin-bottom:3px; }
.ui-button-icon-only .ui-icon { left: 50%; margin-left: -8px; }
.ui-button-text-icon-primary .ui-button-icon-primary, .ui-button-text-icons .ui-button-icon-primary, .ui-button-icons-only .ui-button-icon-primary { left: .5em; }
.ui-button-text-icon-secondary .ui-button-icon-secondary, .ui-button-text-icons .ui-button-icon-secondary, .ui-button-icons-only .ui-button-icon-secondary { right: .5em; }
.ui-button-text-icons .ui-button-icon-secondary, .ui-button-icons-only .ui-button-icon-secondary { right: .5em; }

/*button sets*/


.ui-buttonset { margin-right: 7px; }
.ui-buttonset .ui-state-active {
  color: #ffffff;
  background-color: #0064cd;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#049cdb), to(#0064cd));
  background-image: -moz-linear-gradient(top, #049cdb, #0064cd);
  background-image: -ms-linear-gradient(top, #049cdb, #0064cd);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #049cdb), color-stop(100%, #0064cd));
  background-image: -webkit-linear-gradient(top, #049cdb, #0064cd);
  background-image: -o-linear-gradient(top, #049cdb, #0064cd);
  background-image: linear-gradient(top, #049cdb, #0064cd);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#049cdb', endColorstr='#0064cd', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
.ui-buttonset .ui-button { margin-left: 0; margin-right: -.4em; }

/* workarounds */
button.ui-button::-moz-focus-inner { border: 0; padding: 0; } /* reset extra padding in Firefox */



/*
 * jQuery UI Dialog 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Dialog#theming
 */
.ui-dialog { position: absolute; padding: .2em; width: 300px; overflow: hidden; }
.ui-dialog .ui-dialog-titlebar { /*padding: .4em 1em;*/

  position: relative;
  padding:5px 15px;

  border:0px 0px 0px 1px solid;
  border-color: white;
  padding: 5px 15px;
  font-size: 18px;
  text-decoration:none;
  background:none;
  -moz-border-radius-bottomright: 0px;
  -webkit-border-bottom-right-radius: 0px;
  -khtml-border-bottom-right-radius: 0px;

  -moz-border-radius-bottomleft: 0px;
  -webkit-border-bottom-left-radius: 0px;
  -khtml-border-bottom-left-radius: 0px;
  border-bottom-left-radius: 0px;

  border-bottom:1px solid #ccc;

}
.ui-dialog .ui-dialog-title {
  float: left;
  color:#404040;
  font-weight:bold;
  margin-top:5px;
  margin-bottom:5px;
  padding:5px;

}
.ui-dialog .ui-dialog-titlebar-close {
  position: absolute;
  right: .3em;
  top: 50%;
  width: 19px;
  margin: -10px 0 0 0;
  padding: 1px;
  height: 18px;
  font-size: 20px;
  font-weight: bold;
  line-height: 13.5px;
  text-shadow: 0 1px 0 #ffffff;
  filter: alpha(opacity=25);
  -khtml-opacity: 0.25;
  -moz-opacity: 0.25;
  opacity: 0.25;
}

.ui-dialog .ui-dialog-titlebar-close span { 
  display: block; 
  margin: 1px;
  text-indent: 9999px;
}

.ui-dialog .ui-dialog-titlebar-close:hover, .ui-dialog .ui-dialog-titlebar-close:focus { padding: 0;   filter: alpha(opacity=90);
  -khtml-opacity: 0.90;
  -moz-opacity: 0.90;
  opacity: 0.90;   }

.ui-dialog .ui-dialog-content { position: relative; border: 0; padding: .5em 1em; background: none; overflow: auto; zoom: 1; }

.ui-dialog .ui-dialog-buttonpane {
  text-align: left;
  border-width: 1px 0 0 0;
  background-image: none;
  margin: .5em 0 0 0;
  background-color: #f5f5f5;
  padding: 5px 15px 5px;
  border-top: 1px solid #ddd;
  -webkit-border-radius: 0 0 6px 6px;
  -moz-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
  -webkit-box-shadow: inset 0 1px 0 #ffffff;
  -moz-box-shadow: inset 0 1px 0 #ffffff;
  box-shadow: inset 0 1px 0 #ffffff;
  zoom: 1;
  margin-bottom: 0;

}
.ui-dialog .ui-dialog-buttonpane .ui-dialog-buttonset { float: right; }
.ui-dialog .ui-dialog-buttonpane button { margin: .5em .4em .5em 0; cursor: pointer; }
.ui-dialog .ui-resizable-se { width: 14px; height: 14px; right: 3px; bottom: 3px; }
.ui-draggable .ui-dialog-titlebar { cursor: move; }

.ui-dialog-buttonpane .ui-dialog-buttonset .ui-button{
  color: #ffffff;
  background-color: #0064cd;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#049cdb), to(#0064cd));
  background-image: -moz-linear-gradient(top, #049cdb, #0064cd);
  background-image: -ms-linear-gradient(top, #049cdb, #0064cd);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #049cdb), color-stop(100%, #0064cd));
  background-image: -webkit-linear-gradient(top, #049cdb, #0064cd);
  background-image: -o-linear-gradient(top, #049cdb, #0064cd);
  background-image: linear-gradient(top, #049cdb, #0064cd);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#049cdb', endColorstr='#0064cd', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
}
/*
 * jQuery UI Slider 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Slider#theming
 */
.ui-slider { position: relative; text-align: left; }
.ui-slider .ui-slider-handle { position: absolute; z-index: 2; width: 1.2em; height: 1.2em; cursor: default; }
.ui-slider .ui-slider-range { position: absolute; z-index: 1; font-size: .7em; display: block; border: 0; background-position: 0 0;

  color: #ffffff;
  background-color: #0064cd;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#049cdb), to(#0064cd));
  background-image: -moz-linear-gradient(top, #049cdb, #0064cd);
  background-image: -ms-linear-gradient(top, #049cdb, #0064cd);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #049cdb), color-stop(100%, #0064cd));
  background-image: -webkit-linear-gradient(top, #049cdb, #0064cd);
  background-image: -o-linear-gradient(top, #049cdb, #0064cd);
  background-image: linear-gradient(top, #049cdb, #0064cd);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#049cdb', endColorstr='#0064cd', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);

}

.ui-slider-horizontal { height: .8em; }
.ui-slider-horizontal .ui-slider-handle { top: -.3em; margin-left: -.6em; }
.ui-slider-horizontal .ui-slider-range { top: 0; height: 100%; }
.ui-slider-horizontal .ui-slider-range-min { left: 0; }
.ui-slider-horizontal .ui-slider-range-max { right: 0; }

.ui-slider-vertical { width: .8em; height: 100px; }
.ui-slider-vertical .ui-slider-handle { left: -.3em; margin-left: 0; margin-bottom: -.6em; }
.ui-slider-vertical .ui-slider-range { left: 0; width: 100%; }
.ui-slider-vertical .ui-slider-range-min { bottom: 0; }
.ui-slider-vertical .ui-slider-range-max { top: 0; }/*
 * jQuery UI Tabs 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Tabs#theming
 */
 .ui-tabs .ui-tabs-nav{ background:none; border-color: #ddd;
  border-style: solid;
  border-width: 0 0 1px;}
.ui-tabs { position: relative; padding: .2em; zoom: 1; border:0px;} /* position: relative prevents IE scroll bug (element with position: relative inside container with overflow: auto appear as "fixed") */


.ui-tabs .ui-tabs-nav li:hover, .ui-tabs .ui-tabs-nav li a:hover{
  background:whiteSmoke;
  border-bottom:1px solid #ddd;
  padding-bottom:0px;
   color:#00438A;
}


.ui-tabs .ui-tabs-nav { margin: 0; padding: .2em .2em 0; border-bottom:1px solid #DDD; }
.ui-tabs .ui-tabs-nav li {  text-decoration: none; list-style: none; float: left; position: relative; top: 1px; padding: 0px 0px 1px 0px; white-space: nowrap; background:none; border:0px; 

}

.ui-tabs-nav .ui-state-default{
  -webkit-box-shadow: 0px 0px 0px #ffffff; /* Saf3-4, iOS 4.0.2 - 4.2, Android 2.3+ */
     -moz-box-shadow: 0px 0px 0px #ffffff; /* FF3.5 - 3.6 */
          box-shadow: 0px 0px 0px #ffffff; /* Opera 10.5, IE9, FF4+, Chrome 6+, iOS 5 */
}
.ui-tabs .ui-tabs-nav li a {

  float: left;
  text-decoration: none;
  cursor: text;
  padding: 0 15px;
  margin-right: 2px;
  line-height: 34px;
  border: 1px solid transparent;
  -webkit-border-radius: 4px 4px 0 0;
  -moz-border-radius: 4px 4px 0 0;
  border-radius: 4px 4px 0 0;


 }
.ui-tabs .ui-tabs-nav li.ui-tabs-selected { margin-bottom: 0; padding-bottom: 0px; outline:none;}

.ui-tabs .ui-tabs-nav li.ui-tabs-selected a, .ui-tabs .ui-tabs-nav li.ui-state-disabled a, .ui-tabs .ui-tabs-nav li.ui-state-processing a {

  background-color: #ffffff;
  border: 1px solid #ddd;
  border-bottom-color: #ffffff;
  cursor: default;
  color:gray;
  outline:none;
}


.ui-tabs .ui-tabs-nav li.ui-tabs-selected:hover{
  background:#ffffff;
  outline:none;
}

.ui-tabs .ui-tabs-nav li a, .ui-tabs.ui-tabs-collapsible .ui-tabs-nav li.ui-tabs-selected a { cursor: pointer; color:#0069D6; background:none; font-weight:normal; margin-bottom:-1px;} 
/* first selector in group seems obsolete, but required to overcome bug in Opera applying cursor: text overall if defined elsewhere... */
.ui-tabs .ui-tabs-panel { display: block; border-width: 0; padding: 1em 1.4em; background: none; }
.ui-tabs-panel .ui-button{text-decoration:none;}
.ui-tabs .ui-tabs-hide { display: none !important; }


/* IE fix for background inheritance from ui-widget*/
.ui-tabs .ui-tabs-nav li{
  filter:none;
}


/*
 * jQuery UI Progressbar 1.8.16
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Progressbar#theming
 */
.ui-progressbar { height:2em; text-align: left; }
.ui-progressbar .ui-progressbar-value {margin: -1px; height:100%;

/*this can be removed if ui-widget-header is blue*/
    color: #ffffff;
  background-color: #0064cd;
  background-repeat: repeat-x;
  background-image: -khtml-gradient(linear, left top, left bottom, from(#049cdb), to(#0064cd));
  background-image: -moz-linear-gradient(top, #049cdb, #0064cd);
  background-image: -ms-linear-gradient(top, #049cdb, #0064cd);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #049cdb), color-stop(100%, #0064cd));
  background-image: -webkit-linear-gradient(top, #049cdb, #0064cd);
  background-image: -o-linear-gradient(top, #049cdb, #0064cd);
  background-image: linear-gradient(top, #049cdb, #0064cd);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#049cdb', endColorstr='#0064cd', GradientType=0);
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  border-color: #0064cd #0064cd #003f81;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
 }



/*** Input field styling from Bootstrap **/
 /*TRP 1-21-13*//*
 input, textarea {
  -webkit-transition: border linear 0.2s, box-shadow linear 0.2s;
  -moz-transition: border linear 0.2s, box-shadow linear 0.2s;
  -ms-transition: border linear 0.2s, box-shadow linear 0.2s;
  -o-transition: border linear 0.2s, box-shadow linear 0.2s;
  transition: border linear 0.2s, box-shadow linear 0.2s;
  -webkit-box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1);
  -moz-box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1);
  box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1);
}
input:focus, textarea:focus {
  outline: 0;
  border-color: rgba(82, 168, 236, 0.8);
  -webkit-box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1), 0 0 8px rgba(82, 168, 236, 0.6);
  -moz-box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1), 0 0 8px rgba(82, 168, 236, 0.6);
  box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.1), 0 0 8px rgba(82, 168, 236, 0.6);
}
input[type=file]:focus, input[type=checkbox]:focus, select:focus {
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
  outline: 1px dotted #666;
}
/*TRP 1-21-13*//*
/*TRP 1-21-13*//*
input[type="text"],
input[type="password"],
.ui-autocomplete-input,
textarea,
.uneditable-input {
  display: inline-block;
  padding: 4px;
  font-size: 13px;
  line-height: 18px;
  color: #808080;
  border: 1px solid #ccc;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}*/






/***Dialog fixes**/

.ui-dialog-buttonset .ui-button:nth-child(2){
  cursor: pointer;
  display: inline-block;
  background-color: #e6e6e6;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(25%, #ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -moz-linear-gradient(top, #ffffff, #ffffff 25%, #e6e6e6);
  background-image: -ms-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: -o-linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  background-image: linear-gradient(#ffffff, #ffffff 25%, #e6e6e6);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  padding: 5px 14px 6px;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  color: #333;
  font-size: 13px;
  line-height: normal;
  border: 1px solid #ccc;
  border-bottom-color: #bbb;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -webkit-transition: 0.1s linear all;
  -moz-transition: 0.1s linear all;
  -ms-transition: 0.1s linear all;
  -o-transition: 0.1s linear all;
  transition: 0.1s linear all;
   overflow: visible;
}/*!
 * Bootstrap v2.0.2
 *
 * Copyright 2012 Twitter, Inc
 * Licensed under the Apache License v2.0
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Designed and built with all the love in the world @twitter by @mdo and @fat.
 */
article,
aside,
details,
figcaption,
figure,
footer,
header,
hgroup,
nav,
section {
  display: block;
}
audio,
canvas,
video {
  display: inline-block;
  *display: inline;
  *zoom: 1;
}
audio:not([controls]) {
  display: none;
}
html {
  font-size: 100%;
  -webkit-text-size-adjust: 100%;
  -ms-text-size-adjust: 100%;
}
a:focus {
  outline: thin dotted #333;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px;
}
a:hover,
a:active {
  outline: 0;
}
sub,
sup {
  position: relative;
  font-size: 75%;
  line-height: 0;
  vertical-align: baseline;
}
sup {
  top: -0.5em;
}
sub {
  bottom: -0.25em;
}
img {
  
  border: 0;
  -ms-interpolation-mode: bicubic;
  vertical-align: middle;
}
button,
input,
select,
textarea {
  margin: 0;
  font-size: 100%;
  vertical-align: middle;
}
button,
input {
  *overflow: visible;
  line-height: normal;
}
button::-moz-focus-inner,
input::-moz-focus-inner {
  padding: 0;
  border: 0;
}
button,
input[type="button"],
input[type="reset"],
input[type="submit"] {
  cursor: pointer;
  -webkit-appearance: button;
}
input[type="search"] {
  -webkit-appearance: textfield;
  -webkit-box-sizing: content-box;
  -moz-box-sizing: content-box;
  box-sizing: content-box;
}
input[type="search"]::-webkit-search-decoration,
input[type="search"]::-webkit-search-cancel-button {
  -webkit-appearance: none;
}
textarea {
  overflow: auto;
  vertical-align: top;
}
.clearfix {
  *zoom: 1;
}
.clearfix:before,
.clearfix:after {
  display: table;
  content: "";
}
.clearfix:after {
  clear: both;
}
.hide-text {
  overflow: hidden;
  text-indent: 100%;
  white-space: nowrap;
}
.input-block-level {
  display: block;
  width: 100%;
  min-height: 28px;
  /* Make inputs at least the height of their button counterpart */

  /* Makes inputs behave like true block-level elements */

  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  box-sizing: border-box;
}
body {
  margin: 0;
 /*TRP font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 13px;
  line-height: 18px;
  color: #333333;
 /* background-color: #ffffff;*/
}
/*TRPa {
  color: #0088cc;
  text-decoration: none;
}
a:hover {
  color: #005580;
  text-decoration: underline;
}*/
.row {
  margin-left: -20px;
  *zoom: 1;
}
.row:before,
.row:after {
  display: table;
  content: "";
}
.row:after {
  clear: both;
}
[class*="span"] {
  float: left;
  margin-left: 20px;
}
.container,
.navbar-fixed-top .container,
.navbar-fixed-bottom .container {
  width: 940px;
}
.span12 {
  width: 940px;
}
.span11 {
  width: 860px;
}
.span10 {
  width: 780px;
}
.span9 {
  width: 700px;
}
.span8 {
  width: 620px;
}
.span7 {
  width: 540px;
}
.span6 {
  width: 460px;
}
.span5 {
  width: 380px;
}
.span4 {
  width: 300px;
}
.span3 {
  width: 220px;
}
.span2 {
  width: 140px;
}
.span1 {
  width: 60px;
}
.offset12 {
  margin-left: 980px;
}
.offset11 {
  margin-left: 900px;
}
.offset10 {
  margin-left: 820px;
}
.offset9 {
  margin-left: 740px;
}
.offset8 {
  margin-left: 660px;
}
.offset7 {
  margin-left: 580px;
}
.offset6 {
  margin-left: 500px;
}
.offset5 {
  margin-left: 420px;
}
.offset4 {
  margin-left: 340px;
}
.offset3 {
  margin-left: 260px;
}
.offset2 {
  margin-left: 180px;
}
.offset1 {
  margin-left: 100px;
}
.row-fluid {
  width: 100%;
  *zoom: 1;
}
.row-fluid:before,
.row-fluid:after {
  display: table;
  content: "";
}
.row-fluid:after {
  clear: both;
}
.row-fluid > [class*="span"] {
  float: left;
  margin-left: 2.127659574%;
}
.row-fluid > [class*="span"]:first-child {
  margin-left: 0;
}
.row-fluid > .span12 {
  width: 99.99999998999999%;
}
.row-fluid > .span11 {
  width: 91.489361693%;
}
.row-fluid > .span10 {
  width: 82.97872339599999%;
}
.row-fluid > .span9 {
  width: 74.468085099%;
}
.row-fluid > .span8 {
  width: 65.95744680199999%;
}
.row-fluid > .span7 {
  width: 57.446808505%;
}
.row-fluid > .span6 {
  width: 48.93617020799999%;
}
.row-fluid > .span5 {
  width: 40.425531911%;
}
.row-fluid > .span4 {
  width: 31.914893614%;
}
.row-fluid > .span3 {
  width: 23.404255317%;
}
.row-fluid > .span2 {
  width: 14.89361702%;
}
.row-fluid > .span1 {
  width: 6.382978723%;
}
.container {
  margin-left: auto;
  margin-right: auto;
  *zoom: 1;
}
.container:before,
.container:after {
  display: table;
  content: "";
}
.container:after {
  clear: both;
}
.container-fluid {
  padding-left: 20px;
  padding-right: 20px;
  *zoom: 1;
}
.container-fluid:before,
.container-fluid:after {
  display: table;
  content: "";
}
.container-fluid:after {
  clear: both;
}
p {
  margin: 0 0 9px;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 13px;
  line-height: 18px;
}
p small {
  font-size: 11px;
  color: #999999;
}
.lead {
  margin-bottom: 18px;
  font-size: 20px;
  font-weight: 200;
  line-height: 27px;
}
h1,
h2,
h3,
h4,
h5,
h6 {
  margin: 0;
  font-family: inherit;
  font-weight: bold;
  color: inherit;
  text-rendering: optimizelegibility;
}
h1 small,
h2 small,
h3 small,
h4 small,
h5 small,
h6 small {
  font-weight: normal;
  color: #999999;
}
h1 {
  font-size: 30px;
  line-height: 36px;
}
h1 small {
  font-size: 18px;
}
h2 {
  font-size: 24px;
  line-height: 36px;
}
h2 small {
  font-size: 18px;
}
h3 {
  line-height: 27px;
  font-size: 18px;
}
h3 small {
  font-size: 14px;
}
h4,
h5,
h6 {
  line-height: 18px;
}
h4 {
  font-size: 14px;
}
h4 small {
  font-size: 12px;
}
h5 {
  font-size: 12px;
}
h6 {
  font-size: 11px;
  color: #999999;
  text-transform: uppercase;
}
.page-header {
  padding-bottom: 17px;
  margin: 18px 0;
  border-bottom: 1px solid #eeeeee;
}
.page-header h1 {
  line-height: 1;
}
ul,
ol {
  padding: 0;
  margin: 0 0 9px 25px;
}
ul ul,
ul ol,
ol ol,
ol ul {
  margin-bottom: 0;
}
ul {
  list-style: disc;
}
ol {
  list-style: decimal;
}
li {
  line-height: 18px;
}
ul.unstyled,
ol.unstyled {
  margin-left: 0;
  list-style: none;
}
dl {
  margin-bottom: 18px;
}
dt,
dd {
  line-height: 18px;
}
dt {
  font-weight: bold;
  line-height: 17px;
}
dd {
  margin-left: 9px;
}
.dl-horizontal dt {
  float: left;
  clear: left;
  width: 120px;
  text-align: right;
}
.dl-horizontal dd {
  margin-left: 130px;
}
hr {
  margin: 18px 0;
  border: 0;
  border-top: 1px solid #eeeeee;
  border-bottom: 1px solid #ffffff;
}
strong {
  font-weight: bold;
}
em {
  font-style: italic;
}
.muted {
  color: #999999;
}
abbr[title] {
  border-bottom: 1px dotted #ddd;
  cursor: help;
}
abbr.initialism {
  font-size: 90%;
  text-transform: uppercase;
}
blockquote {
  padding: 0 0 0 15px;
  margin: 0 0 18px;
  border-left: 5px solid #eeeeee;
}
blockquote p {
  margin-bottom: 0;
  font-size: 16px;
  font-weight: 300;
  line-height: 22.5px;
}
blockquote small {
  display: block;
  line-height: 18px;
  color: #999999;
}
blockquote small:before {
  content: '\2014 \00A0';
}
blockquote.pull-right {
  float: right;
  padding-left: 0;
  padding-right: 15px;
  border-left: 0;
  border-right: 5px solid #eeeeee;
}
blockquote.pull-right p,
blockquote.pull-right small {
  text-align: right;
}
q:before,
q:after,
blockquote:before,
blockquote:after {
  content: "";
}
address {
  display: block;
  margin-bottom: 18px;
  line-height: 18px;
  font-style: normal;
}
small {
  font-size: 100%;
}
cite {
  font-style: normal;
}
code,
pre {
  padding: 0 3px 2px;
  font-family: Menlo, Monaco, "Courier New", monospace;
  font-size: 12px;
  color: #333333;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}
code {
  padding: 2px 4px;
  color: #d14;
  background-color: #f7f7f9;
  border: 1px solid #e1e1e8;
}
pre {
  display: block;
  padding: 8.5px;
  margin: 0 0 9px;
  font-size: 12.025px;
  line-height: 18px;
  background-color: #f5f5f5;
  border: 1px solid #ccc;
  border: 1px solid rgba(0, 0, 0, 0.15);
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  white-space: pre;
  white-space: pre-wrap;
  word-break: break-all;
  word-wrap: break-word;
}
pre.prettyprint {
  margin-bottom: 18px;
}
pre code {
  padding: 0;
  color: inherit;
  background-color: transparent;
  border: 0;
}
.pre-scrollable {
  max-height: 340px;
  overflow-y: scroll;
}
form {
  margin: 0 0 18px;
}
fieldset {
  padding: 0;
  margin: 0;
  border: 0;
}
legend {
  display: block;
  width: 100%;
  padding: 0;
  margin-bottom: 27px;
  font-size: 19.5px;
  line-height: 36px;
  color: #333333;
  border: 0;
  border-bottom: 1px solid #eee;
}
legend small {
  font-size: 13.5px;
  color: #999999;
}
label,
input,
button,
select,
textarea {
  font-size: 13px;
  font-weight: normal;
  line-height: 18px;
}
input,
button,
select,
textarea {
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}
label {
  display: block;
  margin-bottom: 5px;
  color: #333333;
}
input,
textarea,
select,
.uneditable-input {
  display: inline-block;
  width: 210px;
  height: 18px;
  padding: 4px;
  /*TRP margin-bottom: 9px;*/
  margin-bottom: 5px;/*TRP */
  font-size: 13px;
  line-height: 18px;
  color: #555555;
  border: 1px solid #cccccc;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}
.uneditable-textarea {
  width: auto;
  height: auto;
}
label input,
label textarea,
label select {
  display: block;
}
input[type="image"],
input[type="checkbox"],
input[type="radio"] {
  width: auto;
  height: auto;
  padding: 0;
  margin: 3px 0;
  *margin-top: 0;
  /* IE7 */

  line-height: normal;
  cursor: pointer;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
  border: 0 \9;
  /* IE9 and down */

}
input[type="image"] {
  border: 0;
}
input[type="file"] {
  width: auto;
  padding: initial;
  line-height: initial;
  border: initial;
  background-color: #ffffff;
  background-color: initial;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}
input[type="button"],
input[type="reset"],
input[type="submit"] {
  width: auto;
  height: auto;
}
select,
input[type="file"] {
  height: 28px;
  /* In IE7, the height of the select element cannot be changed by height, only font-size */

  *margin-top: 4px;
  /* For IE7, add top margin to align select with labels */

  line-height: 28px;
}
input[type="file"] {
  line-height: 18px \9;
}
select {
  width: 220px;
  background-color: #ffffff;
}
select[multiple],
select[size] {
  height: auto;
}
input[type="image"] {
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}
textarea {
  height: auto;
}
input[type="hidden"] {
  display: none;
}
.radio,
.checkbox {
  padding-left: 18px;
}
.radio input[type="radio"],
.checkbox input[type="checkbox"] {
  float: left;
  margin-left: -18px;
}
.controls > .radio:first-child,
.controls > .checkbox:first-child {
  padding-top: 5px;
}
.radio.inline,
.checkbox.inline {
  display: inline-block;
  padding-top: 5px;
  margin-bottom: 0;
  vertical-align: middle;
}
.radio.inline + .radio.inline,
.checkbox.inline + .checkbox.inline {
  margin-left: 10px;
}
input,
textarea {
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  -webkit-transition: border linear 0.2s, box-shadow linear 0.2s;
  -moz-transition: border linear 0.2s, box-shadow linear 0.2s;
  -ms-transition: border linear 0.2s, box-shadow linear 0.2s;
  -o-transition: border linear 0.2s, box-shadow linear 0.2s;
  transition: border linear 0.2s, box-shadow linear 0.2s;
}
input:focus,
textarea:focus {
  /*TRPborder-color: rgba(82, 168, 236, 0.8);
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(82, 168, 236, 0.6);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(82, 168, 236, 0.6);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 8px rgba(82, 168, 236, 0.6);*/
  border-color: rgba(65, 131, 196, 0.5); /*Github .ajaxLink color*/
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
  outline: 0;
  outline: thin dotted \9;
  /* IE6-9 */

}
input[type="file"]:focus,
input[type="radio"]:focus,
input[type="checkbox"]:focus,
select:focus {
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
  outline: thin dotted #333;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px;
}
.input-mini {
  width: 60px;
}
.input-small {
  width: 90px;
}
.input-medium {
  width: 150px;
}
.input-large {
  width: 210px;
}
.input-xlarge {
  width: 270px;
}
.input-xxlarge {
  width: 530px;
}
input[class*="span"],
select[class*="span"],
textarea[class*="span"],
.uneditable-input {
  float: none;
  margin-left: 0;
}
input,
textarea,
.uneditable-input {
  margin-left: 0;
}
input.span12, textarea.span12, .uneditable-input.span12 {
  width: 930px;
}
input.span11, textarea.span11, .uneditable-input.span11 {
  width: 850px;
}
input.span10, textarea.span10, .uneditable-input.span10 {
  width: 770px;
}
input.span9, textarea.span9, .uneditable-input.span9 {
  width: 690px;
}
input.span8, textarea.span8, .uneditable-input.span8 {
  width: 610px;
}
input.span7, textarea.span7, .uneditable-input.span7 {
  width: 530px;
}
input.span6, textarea.span6, .uneditable-input.span6 {
  width: 450px;
}
input.span5, textarea.span5, .uneditable-input.span5 {
  width: 370px;
}
input.span4, textarea.span4, .uneditable-input.span4 {
  width: 290px;
}
input.span3, textarea.span3, .uneditable-input.span3 {
  width: 210px;
}
input.span2, textarea.span2, .uneditable-input.span2 {
  width: 130px;
}
input.span1, textarea.span1, .uneditable-input.span1 {
  width: 50px;
}
input[disabled],
select[disabled],
textarea[disabled],
input[readonly],
select[readonly],
textarea[readonly] {
  background-color: #eeeeee;
  border-color: #ddd;
  cursor: not-allowed;
}
.control-group.warning > label,
.control-group.warning .help-block,
.control-group.warning .help-inline {
  color: #c09853;
}
.control-group.warning input,
.control-group.warning select,
.control-group.warning textarea {
  color: #c09853;
  border-color: #c09853;
}
.control-group.warning input:focus,
.control-group.warning select:focus,
.control-group.warning textarea:focus {
  border-color: #a47e3c;
  -webkit-box-shadow: 0 0 6px #dbc59e;
  -moz-box-shadow: 0 0 6px #dbc59e;
  box-shadow: 0 0 6px #dbc59e;
}
.control-group.warning .input-prepend .add-on,
.control-group.warning .input-append .add-on {
  color: #c09853;
  background-color: #fcf8e3;
  border-color: #c09853;
}
.control-group.error > label,
.control-group.error .help-block,
.control-group.error .help-inline {
  color: #b94a48;
}
.control-group.error input,
.control-group.error select,
.control-group.error textarea {
  color: #b94a48;
  border-color: #b94a48;
}
.control-group.error input:focus,
.control-group.error select:focus,
.control-group.error textarea:focus {
  border-color: #953b39;
  -webkit-box-shadow: 0 0 6px #d59392;
  -moz-box-shadow: 0 0 6px #d59392;
  box-shadow: 0 0 6px #d59392;
}
.control-group.error .input-prepend .add-on,
.control-group.error .input-append .add-on {
  color: #b94a48;
  background-color: #f2dede;
  border-color: #b94a48;
}
.control-group.success > label,
.control-group.success .help-block,
.control-group.success .help-inline {
  color: #468847;
}
.control-group.success input,
.control-group.success select,
.control-group.success textarea {
  color: #468847;
  border-color: #468847;
}
.control-group.success input:focus,
.control-group.success select:focus,
.control-group.success textarea:focus {
  border-color: #356635;
  -webkit-box-shadow: 0 0 6px #7aba7b;
  -moz-box-shadow: 0 0 6px #7aba7b;
  box-shadow: 0 0 6px #7aba7b;
}
.control-group.success .input-prepend .add-on,
.control-group.success .input-append .add-on {
  color: #468847;
  background-color: #dff0d8;
  border-color: #468847;
}
input:focus:required:invalid,
textarea:focus:required:invalid,
select:focus:required:invalid {
  color: #b94a48;
  border-color: #ee5f5b;
}
input:focus:required:invalid:focus,
textarea:focus:required:invalid:focus,
select:focus:required:invalid:focus {
  border-color: #e9322d;
  -webkit-box-shadow: 0 0 6px #f8b9b7;
  -moz-box-shadow: 0 0 6px #f8b9b7;
  box-shadow: 0 0 6px #f8b9b7;
}
.form-actions {
  padding: 17px 20px 18px;
  margin-top: 18px;
  margin-bottom: 18px;
  background-color: #eeeeee;
  border-top: 1px solid #ddd;
  *zoom: 1;
}
.form-actions:before,
.form-actions:after {
  display: table;
  content: "";
}
.form-actions:after {
  clear: both;
}
.uneditable-input {
  display: block;
  background-color: #ffffff;
  border-color: #eee;
  -webkit-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
  -moz-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
  box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.025);
  cursor: not-allowed;
}
:-moz-placeholder {
  color: #999999;
}
::-webkit-input-placeholder {
  color: #999999;
}
.help-block,
.help-inline {
  color: #555555;
}
.help-block {
  display: block;
  margin-bottom: 9px;
}
.help-inline {
  display: inline-block;
  *display: inline;
  /* IE7 inline-block hack */

  *zoom: 1;
  vertical-align: middle;
  padding-left: 5px;
}
.input-prepend,
.input-append {
  margin-bottom: 5px;
}
.input-prepend input,
.input-append input,
.input-prepend select,
.input-append select,
.input-prepend .uneditable-input,
.input-append .uneditable-input {
  *margin-left: 0;
  -webkit-border-radius: 0 3px 3px 0;
  -moz-border-radius: 0 3px 3px 0;
  border-radius: 0 3px 3px 0;
}
.input-prepend input:focus,
.input-append input:focus,
.input-prepend select:focus,
.input-append select:focus,
.input-prepend .uneditable-input:focus,
.input-append .uneditable-input:focus {
  position: relative;
  z-index: 2;
}
.input-prepend .uneditable-input,
.input-append .uneditable-input {
  border-left-color: #ccc;
}
.input-prepend .add-on,
.input-append .add-on {
  display: inline-block;
  width: auto;
  min-width: 16px;
  height: 18px;
  padding: 4px 5px;
  font-weight: normal;
  line-height: 18px;
  text-align: center;
  text-shadow: 0 1px 0 #ffffff;
  vertical-align: middle;
  background-color: #eeeeee;
  border: 1px solid #ccc;
}
.input-prepend .add-on,
.input-append .add-on,
.input-prepend .btn,
.input-append .btn {
  -webkit-border-radius: 3px 0 0 3px;
  -moz-border-radius: 3px 0 0 3px;
  border-radius: 3px 0 0 3px;
}
.input-prepend .active,
.input-append .active {
  background-color: #a9dba9;
  border-color: #46a546;
}
.input-prepend .add-on,
.input-prepend .btn {
  margin-right: -1px;
}
.input-append input,
.input-append select .uneditable-input {
  -webkit-border-radius: 3px 0 0 3px;
  -moz-border-radius: 3px 0 0 3px;
  border-radius: 3px 0 0 3px;
}
.input-append .uneditable-input {
  border-left-color: #eee;
  border-right-color: #ccc;
}
.input-append .add-on,
.input-append .btn {
  margin-left: -1px;
  -webkit-border-radius: 0 3px 3px 0;
  -moz-border-radius: 0 3px 3px 0;
  border-radius: 0 3px 3px 0;
}
.input-prepend.input-append input,
.input-prepend.input-append select,
.input-prepend.input-append .uneditable-input {
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
}
.input-prepend.input-append .add-on:first-child,
.input-prepend.input-append .btn:first-child {
  margin-right: -1px;
  -webkit-border-radius: 3px 0 0 3px;
  -moz-border-radius: 3px 0 0 3px;
  border-radius: 3px 0 0 3px;
}
.input-prepend.input-append .add-on:last-child,
.input-prepend.input-append .btn:last-child {
  margin-left: -1px;
  -webkit-border-radius: 0 3px 3px 0;
  -moz-border-radius: 0 3px 3px 0;
  border-radius: 0 3px 3px 0;
}
.search-query {
  /*TRP padding-left: 14px;
  padding-right: 14px;*/
  margin-bottom: 0;
  -webkit-border-radius: 14px;
  -moz-border-radius: 14px;
  border-radius: 14px;
}
.form-search input,
.form-inline input,
.form-horizontal input,
.form-search textarea,
.form-inline textarea,
.form-horizontal textarea,
.form-search select,
.form-inline select,
.form-horizontal select,
.form-search .help-inline,
.form-inline .help-inline,
.form-horizontal .help-inline,
.form-search .uneditable-input,
.form-inline .uneditable-input,
.form-horizontal .uneditable-input,
.form-search .input-prepend,
.form-inline .input-prepend,
.form-horizontal .input-prepend,
.form-search .input-append,
.form-inline .input-append,
.form-horizontal .input-append {
  display: inline-block;
  margin-bottom: 0;
}
.form-search .hide,
.form-inline .hide,
.form-horizontal .hide {
  display: none;
}
.form-search label,
.form-inline label {
  display: inline-block;
}
.form-search .input-append,
.form-inline .input-append,
.form-search .input-prepend,
.form-inline .input-prepend {
  margin-bottom: 0;
}
.form-search .radio,
.form-search .checkbox,
.form-inline .radio,
.form-inline .checkbox {
  padding-left: 0;
  margin-bottom: 0;
  vertical-align: middle;
}
.form-search .radio input[type="radio"],
.form-search .checkbox input[type="checkbox"],
.form-inline .radio input[type="radio"],
.form-inline .checkbox input[type="checkbox"] {
  float: left;
  margin-left: 0;
  margin-right: 3px;
}
.control-group {
  margin-bottom: 9px;
}
legend + .control-group {
  margin-top: 18px;
  -webkit-margin-top-collapse: separate;
}
.form-horizontal .control-group {
  margin-bottom: 18px;
  *zoom: 1;
}
.form-horizontal .control-group:before,
.form-horizontal .control-group:after {
  display: table;
  content: "";
}
.form-horizontal .control-group:after {
  clear: both;
}
.form-horizontal .control-label {
  float: left;
  width: 140px;
  padding-top: 5px;
  text-align: right;
}
.form-horizontal .controls {
  margin-left: 160px;
  /* Super jank IE7 fix to ensure the inputs in .input-append and input-prepend don't inherit the margin of the parent, in this case .controls */

  *display: inline-block;
  *margin-left: 0;
  *padding-left: 20px;
}
.form-horizontal .help-block {
  margin-top: 9px;
  margin-bottom: 0;
}
.form-horizontal .form-actions {
  padding-left: 160px;
}
table {
  max-width: 100%;
  border-collapse: collapse;
  border-spacing: 0;
  background-color: transparent;
}
.table {
  width: 100%;
  margin-bottom: 18px;
}
.table th,
.table td {
  padding: 8px;
  line-height: 18px;
  text-align: left;
  vertical-align: top;
  border-top: 1px solid #dddddd;
}
.table th {
  font-weight: bold;
}
.table thead th {
  vertical-align: bottom;
}
.table colgroup + thead tr:first-child th,
.table colgroup + thead tr:first-child td,
.table thead:first-child tr:first-child th,
.table thead:first-child tr:first-child td {
  border-top: 0;
}
.table tbody + tbody {
  border-top: 2px solid #dddddd;
}
.table-condensed th,
.table-condensed td {
  padding: 4px 5px;
}
.table-bordered {
  border: 1px solid #dddddd;
  border-left: 0;
  border-collapse: separate;
  *border-collapse: collapsed;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.table-bordered th,
.table-bordered td {
  border-left: 1px solid #dddddd;
}
.table-bordered thead:first-child tr:first-child th,
.table-bordered tbody:first-child tr:first-child th,
.table-bordered tbody:first-child tr:first-child td {
  border-top: 0;
}
.table-bordered thead:first-child tr:first-child th:first-child,
.table-bordered tbody:first-child tr:first-child td:first-child {
  -webkit-border-radius: 4px 0 0 0;
  -moz-border-radius: 4px 0 0 0;
  border-radius: 4px 0 0 0;
}
.table-bordered thead:first-child tr:first-child th:last-child,
.table-bordered tbody:first-child tr:first-child td:last-child {
  -webkit-border-radius: 0 4px 0 0;
  -moz-border-radius: 0 4px 0 0;
  border-radius: 0 4px 0 0;
}
.table-bordered thead:last-child tr:last-child th:first-child,
.table-bordered tbody:last-child tr:last-child td:first-child {
  -webkit-border-radius: 0 0 0 4px;
  -moz-border-radius: 0 0 0 4px;
  border-radius: 0 0 0 4px;
}
.table-bordered thead:last-child tr:last-child th:last-child,
.table-bordered tbody:last-child tr:last-child td:last-child {
  -webkit-border-radius: 0 0 4px 0;
  -moz-border-radius: 0 0 4px 0;
  border-radius: 0 0 4px 0;
}
.table-striped tbody tr:nth-child(odd) td,
.table-striped tbody tr:nth-child(odd) th {
  background-color: #f9f9f9;
}
.table tbody tr:hover td,
.table tbody tr:hover th {
  background-color: #f5f5f5;
}
table .span1 {
  float: none;
  width: 44px;
  margin-left: 0;
}
table .span2 {
  float: none;
  width: 124px;
  margin-left: 0;
}
table .span3 {
  float: none;
  width: 204px;
  margin-left: 0;
}
table .span4 {
  float: none;
  width: 284px;
  margin-left: 0;
}
table .span5 {
  float: none;
  width: 364px;
  margin-left: 0;
}
table .span6 {
  float: none;
  width: 444px;
  margin-left: 0;
}
table .span7 {
  float: none;
  width: 524px;
  margin-left: 0;
}
table .span8 {
  float: none;
  width: 604px;
  margin-left: 0;
}
table .span9 {
  float: none;
  width: 684px;
  margin-left: 0;
}
table .span10 {
  float: none;
  width: 764px;
  margin-left: 0;
}
table .span11 {
  float: none;
  width: 844px;
  margin-left: 0;
}
table .span12 {
  float: none;
  width: 924px;
  margin-left: 0;
}
table .span13 {
  float: none;
  width: 1004px;
  margin-left: 0;
}
table .span14 {
  float: none;
  width: 1084px;
  margin-left: 0;
}
table .span15 {
  float: none;
  width: 1164px;
  margin-left: 0;
}
table .span16 {
  float: none;
  width: 1244px;
  margin-left: 0;
}
table .span17 {
  float: none;
  width: 1324px;
  margin-left: 0;
}
table .span18 {
  float: none;
  width: 1404px;
  margin-left: 0;
}
table .span19 {
  float: none;
  width: 1484px;
  margin-left: 0;
}
table .span20 {
  float: none;
  width: 1564px;
  margin-left: 0;
}
table .span21 {
  float: none;
  width: 1644px;
  margin-left: 0;
}
table .span22 {
  float: none;
  width: 1724px;
  margin-left: 0;
}
table .span23 {
  float: none;
  width: 1804px;
  margin-left: 0;
}
table .span24 {
  float: none;
  width: 1884px;
  margin-left: 0;
}
/*TRP[class^="icon-"],
[class*=" icon-"] {
  display: inline-block;
  width: 14px;
  height: 14px;
  line-height: 14px;
  vertical-align: text-top;
  background-image: url("../img/glyphicons-halflingsBlue.png");
  background-position: 14px 14px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}*/
[class^="icon-"]:last-child,
[class*=" icon-"]:last-child {
  *margin-left: 0;
}
.icon-white {
  background-image: url("../img/glyphicons-halflings-white.png");
}
.icon-glass {
  background-position: 0      0;
}
.icon-music {
  background-position: -24px 0;
}
.icon-search {
  background-position: -48px 0;
}
.icon-envelope {
  background-position: -72px 0;
}
.icon-heart {
  background-position: -96px 0;
}
.icon-star {
  background-position: -120px 0;
}
.icon-star-empty {
  background-position: -144px 0;
}
.icon-user {
  background-position: -168px 0;
}
.icon-film {
  background-position: -192px 0;
}
.icon-th-large {
  background-position: -216px 0;
}
.icon-th {
  background-position: -240px 0;
}
.icon-th-list {
  background-position: -264px 0;
}
.icon-ok {
  background-position: -288px 0;
}
.icon-remove {
  background-position: -312px 0;
}
.icon-zoom-in {
  background-position: -336px 0;
}
.icon-zoom-out {
  background-position: -360px 0;
}
.icon-off {
  background-position: -384px 0;
}
.icon-signal {
  background-position: -408px 0;
}
.icon-cog {
  background-position: -432px 0;
}
.icon-trash {
  background-position: -456px 0;
}
.icon-home {
  background-position: 0 -24px;
}
.icon-file {
  background-position: -24px -24px;
}
.icon-time {
  background-position: -48px -24px;
}
.icon-road {
  background-position: -72px -24px;
}
.icon-download-alt {
  background-position: -96px -24px;
}
.icon-download {
  background-position: -120px -24px;
}
.icon-upload {
  background-position: -144px -24px;
}
.icon-inbox {
  background-position: -168px -24px;
}
.icon-play-circle {
  background-position: -192px -24px;
}
.icon-repeat {
  background-position: -216px -24px;
}
.icon-refresh {
  background-position: -240px -24px;
}
.icon-list-alt {
  background-position: -264px -24px;
}
.icon-lock {
  background-position: -287px -24px;
}
.icon-flag {
  background-position: -312px -24px;
}
.icon-headphones {
  background-position: -336px -24px;
}
.icon-volume-off {
  background-position: -360px -24px;
}
.icon-volume-down {
  background-position: -384px -24px;
}
.icon-volume-up {
  background-position: -408px -24px;
}
.icon-qrcode {
  background-position: -432px -24px;
}
.icon-barcode {
  background-position: -456px -24px;
}
.icon-tag {
  background-position: 0 -48px;
}
.icon-tags {
  background-position: -25px -48px;
}
.icon-book {
  background-position: -48px -48px;
}
.icon-bookmark {
  background-position: -72px -48px;
}
.icon-print {
  background-position: -96px -48px;
}
.icon-camera {
  background-position: -120px -48px;
}
.icon-font {
  background-position: -144px -48px;
}
.icon-bold {
  background-position: -167px -48px;
}
.icon-italic {
  background-position: -192px -48px;
}
.icon-text-height {
  background-position: -216px -48px;
}
.icon-text-width {
  background-position: -240px -48px;
}
.icon-align-left {
  background-position: -264px -48px;
}
.icon-align-center {
  background-position: -288px -48px;
}
.icon-align-right {
  background-position: -312px -48px;
}
.icon-align-justify {
  background-position: -336px -48px;
}
.icon-list {
  background-position: -360px -48px;
}
.icon-indent-left {
  background-position: -384px -48px;
}
.icon-indent-right {
  background-position: -408px -48px;
}
.icon-facetime-video {
  background-position: -432px -48px;
}
.icon-picture {
  background-position: -456px -48px;
}
.icon-pencil {
  background-position: 0 -72px;
}
.icon-map-marker {
  background-position: -24px -72px;
}
.icon-adjust {
  background-position: -48px -72px;
}
.icon-tint {
  background-position: -72px -72px;
}
.icon-edit {
  background-position: -96px -72px;
}
.icon-share {
  background-position: -120px -72px;
}
.icon-check {
  background-position: -144px -72px;
}
.icon-move {
  background-position: -168px -72px;
}
.icon-step-backward {
  background-position: -192px -72px;
}
.icon-fast-backward {
  background-position: -216px -72px;
}
.icon-backward {
  background-position: -240px -72px;
}
.icon-play {
  background-position: -264px -72px;
}
.icon-pause {
  background-position: -288px -72px;
}
.icon-stop {
  background-position: -312px -72px;
}
.icon-forward {
  background-position: -336px -72px;
}
.icon-fast-forward {
  background-position: -360px -72px;
}
.icon-step-forward {
  background-position: -384px -72px;
}
.icon-eject {
  background-position: -408px -72px;
}
.icon-chevron-left {
  background-position: -432px -72px;
}
.icon-chevron-right {
  background-position: -456px -72px;
}
.icon-plus-sign {
  background-position: 0 -96px;
}
.icon-minus-sign {
  background-position: -24px -96px;
}
.icon-remove-sign {
  background-position: -48px -96px;
}
.icon-ok-sign {
  background-position: -72px -96px;
}
.icon-question-sign {
  background-position: -96px -96px;
}
.icon-info-sign {
  background-position: -120px -96px;
}
.icon-screenshot {
  background-position: -144px -96px;
}
.icon-remove-circle {
  background-position: -168px -96px;
}
.icon-ok-circle {
  background-position: -192px -96px;
}
.icon-ban-circle {
  background-position: -216px -96px;
}
.icon-arrow-left {
  background-position: -240px -96px;
}
.icon-arrow-right {
  background-position: -264px -96px;
}
.icon-arrow-up {
  background-position: -289px -96px;
}
.icon-arrow-down {
  background-position: -312px -96px;
}
.icon-share-alt {
  background-position: -336px -96px;
}
.icon-resize-full {
  background-position: -360px -96px;
}
.icon-resize-small {
  background-position: -384px -96px;
}
.icon-plus {
  background-position: -408px -96px;
}
.icon-minus {
  background-position: -433px -96px;
}
.icon-asterisk {
  background-position: -456px -96px;
}
.icon-exclamation-sign {
  background-position: 0 -120px;
}
.icon-gift {
  background-position: -24px -120px;
}
.icon-leaf {
  background-position: -48px -120px;
}
.icon-fire {
  background-position: -72px -120px;
}
.icon-eye-open {
  background-position: -96px -120px;
}
.icon-eye-close {
  background-position: -120px -120px;
}
.icon-warning-sign {
  background-position: -144px -120px;
}
.icon-plane {
  background-position: -168px -120px;
}
.icon-calendar {
  background-position: -192px -120px;
}
.icon-random {
  background-position: -216px -120px;
}
.icon-comment {
  background-position: -240px -120px;
}
.icon-magnet {
  background-position: -264px -120px;
}
.icon-chevron-up {
  background-position: -288px -120px;
}
.icon-chevron-down {
  background-position: -313px -119px;
}
.icon-retweet {
  background-position: -336px -120px;
}
.icon-shopping-cart {
  background-position: -360px -120px;
}
.icon-folder-close {
  background-position: -384px -120px;
}
.icon-folder-open {
  background-position: -408px -120px;
}
.icon-resize-vertical {
  background-position: -432px -119px;
}
.icon-resize-horizontal {
  background-position: -456px -118px;
}
.dropdown {
  position: relative;
}
.dropdown-toggle {
  *margin-bottom: -3px;
}
.dropdown-toggle:active,
.open .dropdown-toggle {
  outline: 0;
}
.caret {
  display: inline-block;
  width: 0;
  height: 0;
  vertical-align: top;
  border-left: 4px solid transparent;
  border-right: 4px solid transparent;
  border-top: 4px solid #000000;
  opacity: 0.3;
  filter: alpha(opacity=30);
  content: "";
}
.dropdown .caret {
  margin-top: 8px;
  margin-left: 2px;
}
.dropdown:hover .caret,
.open.dropdown .caret {
  opacity: 1;
  filter: alpha(opacity=100);
}
.dropdown-menu {
  position: absolute;
  top: 100%;
  left: 0;
  z-index: 1000;
  float: left;
  display: none;
  min-width: 160px;
  padding: 4px 0;
  margin: 0;
  list-style: none;
  background-color: #ffffff;
  border-color: #ccc;
  border-color: rgba(0, 0, 0, 0.2);
  border-style: solid;
  border-width: 1px;
  -webkit-border-radius: 0 0 5px 5px;
  -moz-border-radius: 0 0 5px 5px;
  border-radius: 0 0 5px 5px;
  -webkit-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
  -moz-box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
  box-shadow: 0 5px 10px rgba(0, 0, 0, 0.2);
  -webkit-background-clip: padding-box;
  -moz-background-clip: padding;
  background-clip: padding-box;
  *border-right-width: 2px;
  *border-bottom-width: 2px;
}
.dropdown-menu.pull-right {
  right: 0;
  left: auto;
}
.dropdown-menu .divider {
  height: 1px;
  margin: 8px 1px;
  overflow: hidden;
  background-color: #e5e5e5;
  border-bottom: 1px solid #ffffff;
  *width: 100%;
  *margin: -5px 0 5px;
}
.dropdown-menu a {
  display: block;
  padding: 3px 15px;
  clear: both;
  font-weight: normal;
  line-height: 18px;
  color: #333333;
  white-space: nowrap;
}
.dropdown-menu li > a:hover,
.dropdown-menu .active > a,
.dropdown-menu .active > a:hover {
  color: #ffffff;
  text-decoration: none;
  background-color: #0088cc;
}
.dropdown.open {
  *z-index: 1000;
}
.dropdown.open .dropdown-toggle {
  color: #ffffff;
  background: #ccc;
  background: rgba(0, 0, 0, 0.3);
}
.dropdown.open .dropdown-menu {
  display: block;
}
.pull-right .dropdown-menu {
  left: auto;
  right: 0;
}
.dropup .caret,
.navbar-fixed-bottom .dropdown .caret {
  border-top: 0;
  border-bottom: 4px solid #000000;
  content: "\2191";
}
.dropup .dropdown-menu,
.navbar-fixed-bottom .dropdown .dropdown-menu {
  top: auto;
  bottom: 100%;
  margin-bottom: 1px;
}
.typeahead {
  margin-top: 2px;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.well {
  min-height: 20px;
  padding: 19px;
  margin-bottom: 20px;
  background-color: #f5f5f5;
  border: 1px solid #eee;
  border: 1px solid rgba(0, 0, 0, 0.05);
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.05);
}
.well blockquote {
  border-color: #ddd;
  border-color: rgba(0, 0, 0, 0.15);
}
.well-large {
  padding: 24px;
  -webkit-border-radius: 6px;
  -moz-border-radius: 6px;
  border-radius: 6px;
}
.well-small {
  padding: 9px;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}
.fade {
  -webkit-transition: opacity 0.15s linear;
  -moz-transition: opacity 0.15s linear;
  -ms-transition: opacity 0.15s linear;
  -o-transition: opacity 0.15s linear;
  transition: opacity 0.15s linear;
  opacity: 0;
}
.fade.in {
  opacity: 1;
}
.collapse {
  -webkit-transition: height 0.35s ease;
  -moz-transition: height 0.35s ease;
  -ms-transition: height 0.35s ease;
  -o-transition: height 0.35s ease;
  transition: height 0.35s ease;
  position: relative;
  overflow: hidden;
  height: 0;
}
.collapse.in {
  height: auto;
}
.close {
  float: right;
  font-size: 20px;
  font-weight: bold;
  line-height: 18px;
  color: #000000;
  text-shadow: 0 1px 0 #ffffff;
  opacity: 0.2;
  filter: alpha(opacity=20);
}
.close:hover {
  color: #000000;
  text-decoration: none;
  opacity: 0.4;
  filter: alpha(opacity=40);
  cursor: pointer;
}
.btn {
  display: inline-block;
  *display: inline;
  /* IE7 inline-block hack */

  *zoom: 1;
  padding: 4px 10px 4px;
  margin-bottom: 0;
  font-size: 13px;
  line-height: 18px;
  color: #333333;
  text-align: center;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
  vertical-align: middle;
  background-color: #f5f5f5;
  background-image: -moz-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -ms-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), to(#e6e6e6));
  background-image: -webkit-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: -o-linear-gradient(top, #ffffff, #e6e6e6);
  background-image: linear-gradient(top, #ffffff, #e6e6e6);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#e6e6e6', GradientType=0);
  border-color: #e6e6e6 #e6e6e6 #bfbfbf;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
  border: 1px solid #cccccc;
  border-bottom-color: #b3b3b3;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  cursor: pointer;
  *margin-left: .3em;
}
.btn:hover,
.btn:active,
.btn.active,
.btn.disabled,
.btn[disabled] {
  background-color: #e6e6e6;
}
.btn:active,
.btn.active {
  background-color: #cccccc \9;
}
.btn:first-child {
  *margin-left: 0;
}
.btn:hover {
  color: #666;
 /*TRP color: #333333;*/
  text-decoration: none;
  background-color: #e6e6e6;
  background-position: 0 -15px;
  -webkit-transition: background-position 0.1s linear;
  -moz-transition: background-position 0.1s linear;
  -ms-transition: background-position 0.1s linear;
  -o-transition: background-position 0.1s linear;
  transition: background-position 0.1s linear;
}
.btn:focus {
  outline: thin dotted #333;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px;
}
.btn.active,
.btn:active {
  background-image: none;
  -webkit-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 2px 4px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  background-color: #e6e6e6;
  background-color: #d9d9d9 \9;
  outline: 0;
}
.btn.disabled,
.btn[disabled] {
  cursor: default;
  background-image: none;
  background-color: #e6e6e6;
  opacity: 0.65;
  filter: alpha(opacity=65);
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}
.btn-large {
  padding: 9px 14px;
  font-size: 15px;
  line-height: normal;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
}
.btn-large [class^="icon-"] {
  margin-top: 1px;
}
.btn-small {
  padding: 5px 9px;
  font-size: 11px;
  line-height: 16px;
}
.btn-small [class^="icon-"] {
  margin-top: -1px;
}
.btn-mini {
  padding: 2px 6px;
  font-size: 11px;
  line-height: 14px;
}
.btn-primary,
.btn-primary:hover,
.btn-warning,
.btn-warning:hover,
.btn-danger,
.btn-danger:hover,
.btn-success,
.btn-success:hover,
.btn-info,
.btn-info:hover,
.btn-inverse,
.btn-inverse:hover {
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  color: #ffffff;
}
.btn-primary.active,
.btn-warning.active,
.btn-danger.active,
.btn-success.active,
.btn-info.active,
.btn-inverse.active {
  color: rgba(255, 255, 255, 0.75);
}
/*TRP.btn-primary {
  background-color: #0074cc;
  background-image: -moz-linear-gradient(top, #0088cc, #0055cc);
  background-image: -ms-linear-gradient(top, #0088cc, #0055cc);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#0088cc), to(#0055cc));
  background-image: -webkit-linear-gradient(top, #0088cc, #0055cc);
  background-image: -o-linear-gradient(top, #0088cc, #0055cc);
  background-image: linear-gradient(top, #0088cc, #0055cc);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#0088cc', endColorstr='#0055cc', GradientType=0);
  border-color: #0055cc #0055cc #003580;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}*/
.btn-primary:hover,
.btn-primary:active,
.btn-primary.active,
.btn-primary.disabled,
.btn-primary[disabled] {
  background-color: #0055cc;
}
.btn-primary:active,
.btn-primary.active {
  background-color: #004099 \9;
}
.btn-warning {
  background-color: #faa732;
  background-image: -moz-linear-gradient(top, #fbb450, #f89406);
  background-image: -ms-linear-gradient(top, #fbb450, #f89406);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fbb450), to(#f89406));
  background-image: -webkit-linear-gradient(top, #fbb450, #f89406);
  background-image: -o-linear-gradient(top, #fbb450, #f89406);
  background-image: linear-gradient(top, #fbb450, #f89406);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fbb450', endColorstr='#f89406', GradientType=0);
  border-color: #f89406 #f89406 #ad6704;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}
.btn-warning:hover,
.btn-warning:active,
.btn-warning.active,
.btn-warning.disabled,
.btn-warning[disabled] {
  background-color: #f89406;
}
.btn-warning:active,
.btn-warning.active {
  background-color: #c67605 \9;
}
.btn-danger {
  background-color: #da4f49;
  background-image: -moz-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -ms-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ee5f5b), to(#bd362f));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: -o-linear-gradient(top, #ee5f5b, #bd362f);
  background-image: linear-gradient(top, #ee5f5b, #bd362f);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#bd362f', GradientType=0);
  border-color: #bd362f #bd362f #802420;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}
.btn-danger:hover,
.btn-danger:active,
.btn-danger.active,
.btn-danger.disabled,
.btn-danger[disabled] {
  background-color: #bd362f;
}
.btn-danger:active,
.btn-danger.active {
  background-color: #942a25 \9;
}
.btn-success {
  background-color: #5bb75b;
  background-image: -moz-linear-gradient(top, #62c462, #51a351);
  background-image: -ms-linear-gradient(top, #62c462, #51a351);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#62c462), to(#51a351));
  background-image: -webkit-linear-gradient(top, #62c462, #51a351);
  background-image: -o-linear-gradient(top, #62c462, #51a351);
  background-image: linear-gradient(top, #62c462, #51a351);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#62c462', endColorstr='#51a351', GradientType=0);
  border-color: #51a351 #51a351 #387038;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}
.btn-success:hover,
.btn-success:active,
.btn-success.active,
.btn-success.disabled,
.btn-success[disabled] {
  background-color: #51a351;
}
.btn-success:active,
.btn-success.active {
  background-color: #408140 \9;
}
.btn-info {
  background-color: #49afcd;
  background-image: -moz-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -ms-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#5bc0de), to(#2f96b4));
  background-image: -webkit-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: -o-linear-gradient(top, #5bc0de, #2f96b4);
  background-image: linear-gradient(top, #5bc0de, #2f96b4);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#5bc0de', endColorstr='#2f96b4', GradientType=0);
  border-color: #2f96b4 #2f96b4 #1f6377;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}
.btn-info:hover,
.btn-info:active,
.btn-info.active,
.btn-info.disabled,
.btn-info[disabled] {
  background-color: #2f96b4;
}
.btn-info:active,
.btn-info.active {
  background-color: #24748c \9;
}
.btn-inverse {
  background-color: #414141;
  background-image: -moz-linear-gradient(top, #555555, #222222);
  background-image: -ms-linear-gradient(top, #555555, #222222);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#555555), to(#222222));
  background-image: -webkit-linear-gradient(top, #555555, #222222);
  background-image: -o-linear-gradient(top, #555555, #222222);
  background-image: linear-gradient(top, #555555, #222222);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#555555', endColorstr='#222222', GradientType=0);
  border-color: #222222 #222222 #000000;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
}
.btn-inverse:hover,
.btn-inverse:active,
.btn-inverse.active,
.btn-inverse.disabled,
.btn-inverse[disabled] {
  background-color: #222222;
}
.btn-inverse:active,
.btn-inverse.active {
  background-color: #080808 \9;
}
button.btn,
input[type="submit"].btn {
  *padding-top: 2px;
  *padding-bottom: 2px;
}
button.btn::-moz-focus-inner,
input[type="submit"].btn::-moz-focus-inner {
  padding: 0;
  border: 0;
}
button.btn.btn-large,
input[type="submit"].btn.btn-large {
  *padding-top: 7px;
  *padding-bottom: 7px;
}
button.btn.btn-small,
input[type="submit"].btn.btn-small {
  *padding-top: 3px;
  *padding-bottom: 3px;
}
button.btn.btn-mini,
input[type="submit"].btn.btn-mini {
  *padding-top: 1px;
  *padding-bottom: 1px;
}
.btn-group {
  position: relative;
  *zoom: 1;
  *margin-left: .3em;
}
.btn-group:before,
.btn-group:after {
  display: table;
  content: "";
}
.btn-group:after {
  clear: both;
}
.btn-group:first-child {
  *margin-left: 0;
}
.btn-group + .btn-group {
  margin-left: 5px;
}
.btn-toolbar {
  margin-top: 9px;
  margin-bottom: 9px;
}
.btn-toolbar .btn-group {
  display: inline-block;
  *display: inline;
  /* IE7 inline-block hack */

  *zoom: 1;
}
.btn-group .btn {
  position: relative;
  float: left;
  margin-left: -1px;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
}
.btn-group .btn:first-child {
  margin-left: 0;
  -webkit-border-top-left-radius: 4px;
  -moz-border-radius-topleft: 4px;
  border-top-left-radius: 4px;
  -webkit-border-bottom-left-radius: 4px;
  -moz-border-radius-bottomleft: 4px;
  border-bottom-left-radius: 4px;
}
.btn-group .btn:last-child,
.btn-group .dropdown-toggle {
  -webkit-border-top-right-radius: 4px;
  -moz-border-radius-topright: 4px;
  border-top-right-radius: 4px;
  -webkit-border-bottom-right-radius: 4px;
  -moz-border-radius-bottomright: 4px;
  border-bottom-right-radius: 4px;
}
.btn-group .btn.large:first-child {
  margin-left: 0;
  -webkit-border-top-left-radius: 6px;
  -moz-border-radius-topleft: 6px;
  border-top-left-radius: 6px;
  -webkit-border-bottom-left-radius: 6px;
  -moz-border-radius-bottomleft: 6px;
  border-bottom-left-radius: 6px;
}
.btn-group .btn.large:last-child,
.btn-group .large.dropdown-toggle {
  -webkit-border-top-right-radius: 6px;
  -moz-border-radius-topright: 6px;
  border-top-right-radius: 6px;
  -webkit-border-bottom-right-radius: 6px;
  -moz-border-radius-bottomright: 6px;
  border-bottom-right-radius: 6px;
}
.btn-group .btn:hover,
.btn-group .btn:focus,
.btn-group .btn:active,
.btn-group .btn.active {
  z-index: 2;
}
.btn-group .dropdown-toggle:active,
.btn-group.open .dropdown-toggle {
  outline: 0;
}
.btn-group .dropdown-toggle {
  padding-left: 8px;
  padding-right: 8px;
  -webkit-box-shadow: inset 1px 0 0 rgba(255, 255, 255, 0.125), inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 1px 0 0 rgba(255, 255, 255, 0.125), inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 1px 0 0 rgba(255, 255, 255, 0.125), inset 0 1px 0 rgba(255, 255, 255, 0.2), 0 1px 2px rgba(0, 0, 0, 0.05);
  *padding-top: 3px;
  *padding-bottom: 3px;
}
.btn-group .btn-mini.dropdown-toggle {
  padding-left: 5px;
  padding-right: 5px;
  *padding-top: 1px;
  *padding-bottom: 1px;
}
.btn-group .btn-small.dropdown-toggle {
  *padding-top: 4px;
  *padding-bottom: 4px;
}
.btn-group .btn-large.dropdown-toggle {
  padding-left: 12px;
  padding-right: 12px;
}
.btn-group.open {
  *z-index: 1000;
}
.btn-group.open .dropdown-menu {
  display: block;
  margin-top: 1px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
}
.btn-group.open .dropdown-toggle {
  background-image: none;
  -webkit-box-shadow: inset 0 1px 6px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: inset 0 1px 6px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: inset 0 1px 6px rgba(0, 0, 0, 0.15), 0 1px 2px rgba(0, 0, 0, 0.05);
}
.btn .caret {
  margin-top: 7px;
  margin-left: 0;
}
.btn:hover .caret,
.open.btn-group .caret {
  opacity: 1;
  filter: alpha(opacity=100);
}
.btn-mini .caret {
  margin-top: 5px;
}
.btn-small .caret {
  margin-top: 6px;
}
.btn-large .caret {
  margin-top: 6px;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  border-top: 5px solid #000000;
}
.btn-primary .caret,
.btn-warning .caret,
.btn-danger .caret,
.btn-info .caret,
.btn-success .caret,
.btn-inverse .caret {
  border-top-color: #ffffff;
  border-bottom-color: #ffffff;
  opacity: 0.75;
  filter: alpha(opacity=75);
}
.alert {
  padding: 8px 35px 8px 14px;
  margin-bottom: 18px;
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
  background-color: #fcf8e3;
  border: 1px solid #fbeed5;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  color: #c09853;
}
.alert-heading {
  color: inherit;
}
.alert .close {
  position: relative;
  top: -2px;
  right: -21px;
  line-height: 18px;
}
.alert-success {
  background-color: #dff0d8;
  border-color: #d6e9c6;
  color: #468847;
}
.alert-danger,
.alert-error {
  background-color: #f2dede;
  border-color: #eed3d7;
  color: #b94a48;
}
.alert-info {
  background-color: #d9edf7;
  border-color: #bce8f1;
  color: #3a87ad;
}
.alert-block {
  padding-top: 14px;
  padding-bottom: 14px;
}
.alert-block > p,
.alert-block > ul {
  margin-bottom: 0;
}
.alert-block p + p {
  margin-top: 5px;
}
.nav {
  margin-left: 0;
  margin-bottom: 18px;
  list-style: none;
}
.nav > li > a {
  display: block;
}
.nav > li > a:hover {
  text-decoration: none;
  background-color: #eeeeee;
}
.nav .nav-header {
  display: block;
  padding: 3px 15px;
  font-size: 11px;
  font-weight: bold;
  line-height: 18px;
  color: #999999;
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
  text-transform: uppercase;
}
.nav li + .nav-header {
  margin-top: 9px;
}
.nav-list {
  padding-left: 15px;
  padding-right: 15px;
  margin-bottom: 0;
}
.nav-list > li > a,
.nav-list .nav-header {
  margin-left: -15px;
  margin-right: -15px;
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
}
.nav-list > li > a {
  padding: 3px 15px;
}
.nav-list > .active > a,
.nav-list > .active > a:hover {
  color: #ffffff;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.2);
  background-color: #0088cc;
}
.nav-list [class^="icon-"] {
  margin-right: 2px;
}
.nav-list .divider {
  height: 1px;
  margin: 8px 1px;
  overflow: hidden;
  background-color: #e5e5e5;
  border-bottom: 1px solid #ffffff;
  *width: 100%;
  *margin: -5px 0 5px;
}
.nav-tabs,
.nav-pills {
  *zoom: 1;
}
.nav-tabs:before,
.nav-pills:before,
.nav-tabs:after,
.nav-pills:after {
  display: table;
  content: "";
}
.nav-tabs:after,
.nav-pills:after {
  clear: both;
}
.nav-tabs > li,
.nav-pills > li {
  float: left;
}
.nav-tabs > li > a,
.nav-pills > li > a {
  padding-right: 12px;
  padding-left: 12px;
  margin-right: 2px;
  line-height: 14px;
}
.nav-tabs {
  border-bottom: 1px solid #ddd;
}
.nav-tabs > li {
  margin-bottom: -1px;
}
.nav-tabs > li > a {
  padding-top: 8px;
  padding-bottom: 8px;
  line-height: 18px;
  border: 1px solid transparent;
  -webkit-border-radius: 4px 4px 0 0;
  -moz-border-radius: 4px 4px 0 0;
  border-radius: 4px 4px 0 0;
}
.nav-tabs > li > a:hover {
  border-color: #eeeeee #eeeeee #dddddd;
}
.nav-tabs > .active > a,
.nav-tabs > .active > a:hover {
  color: #555555;
  background-color: #ffffff;
  border: 1px solid #ddd;
  border-bottom-color: transparent;
  cursor: default;
}
.nav-pills > li > a {
  padding-top: 8px;
  padding-bottom: 8px;
  margin-top: 2px;
  margin-bottom: 2px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
}
.nav-pills > .active > a,
.nav-pills > .active > a:hover {
  color: #ffffff;
  background-color: #0088cc;
}
.nav-stacked > li {
  float: none;
}
.nav-stacked > li > a {
  margin-right: 0;
}
.nav-tabs.nav-stacked {
  border-bottom: 0;
}
.nav-tabs.nav-stacked > li > a {
  border: 1px solid #ddd;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
}
.nav-tabs.nav-stacked > li:first-child > a {
  -webkit-border-radius: 4px 4px 0 0;
  -moz-border-radius: 4px 4px 0 0;
  border-radius: 4px 4px 0 0;
}
.nav-tabs.nav-stacked > li:last-child > a {
  -webkit-border-radius: 0 0 4px 4px;
  -moz-border-radius: 0 0 4px 4px;
  border-radius: 0 0 4px 4px;
}
.nav-tabs.nav-stacked > li > a:hover {
  border-color: #ddd;
  z-index: 2;
}
.nav-pills.nav-stacked > li > a {
  margin-bottom: 3px;
}
.nav-pills.nav-stacked > li:last-child > a {
  margin-bottom: 1px;
}
.nav-tabs .dropdown-menu,
.nav-pills .dropdown-menu {
  margin-top: 1px;
  border-width: 1px;
}
.nav-pills .dropdown-menu {
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.nav-tabs .dropdown-toggle .caret,
.nav-pills .dropdown-toggle .caret {
  border-top-color: #0088cc;
  border-bottom-color: #0088cc;
  margin-top: 6px;
}
.nav-tabs .dropdown-toggle:hover .caret,
.nav-pills .dropdown-toggle:hover .caret {
  border-top-color: #005580;
  border-bottom-color: #005580;
}
.nav-tabs .active .dropdown-toggle .caret,
.nav-pills .active .dropdown-toggle .caret {
  border-top-color: #333333;
  border-bottom-color: #333333;
}
.nav > .dropdown.active > a:hover {
  color: #000000;
  cursor: pointer;
}
.nav-tabs .open .dropdown-toggle,
.nav-pills .open .dropdown-toggle,
.nav > .open.active > a:hover {
  color: #ffffff;
  background-color: #999999;
  border-color: #999999;
}
.nav .open .caret,
.nav .open.active .caret,
.nav .open a:hover .caret {
  border-top-color: #ffffff;
  border-bottom-color: #ffffff;
  opacity: 1;
  filter: alpha(opacity=100);
}
.tabs-stacked .open > a:hover {
  border-color: #999999;
}
.tabbable {
  *zoom: 1;
}
.tabbable:before,
.tabbable:after {
  display: table;
  content: "";
}
.tabbable:after {
  clear: both;
}
.tab-content {
  display: table;
  width: 100%;
}
.tabs-below .nav-tabs,
.tabs-right .nav-tabs,
.tabs-left .nav-tabs {
  border-bottom: 0;
}
.tab-content > .tab-pane,
.pill-content > .pill-pane {
  display: none;
}
.tab-content > .active,
.pill-content > .active {
  display: block;
}
.tabs-below .nav-tabs {
  border-top: 1px solid #ddd;
}
.tabs-below .nav-tabs > li {
  margin-top: -1px;
  margin-bottom: 0;
}
.tabs-below .nav-tabs > li > a {
  -webkit-border-radius: 0 0 4px 4px;
  -moz-border-radius: 0 0 4px 4px;
  border-radius: 0 0 4px 4px;
}
.tabs-below .nav-tabs > li > a:hover {
  border-bottom-color: transparent;
  border-top-color: #ddd;
}
.tabs-below .nav-tabs .active > a,
.tabs-below .nav-tabs .active > a:hover {
  border-color: transparent #ddd #ddd #ddd;
}
.tabs-left .nav-tabs > li,
.tabs-right .nav-tabs > li {
  float: none;
}
.tabs-left .nav-tabs > li > a,
.tabs-right .nav-tabs > li > a {
  min-width: 74px;
  margin-right: 0;
  margin-bottom: 3px;
}
.tabs-left .nav-tabs {
  float: left;
  margin-right: 19px;
  border-right: 1px solid #ddd;
}
.tabs-left .nav-tabs > li > a {
  margin-right: -1px;
  -webkit-border-radius: 4px 0 0 4px;
  -moz-border-radius: 4px 0 0 4px;
  border-radius: 4px 0 0 4px;
}
.tabs-left .nav-tabs > li > a:hover {
  border-color: #eeeeee #dddddd #eeeeee #eeeeee;
}
.tabs-left .nav-tabs .active > a,
.tabs-left .nav-tabs .active > a:hover {
  border-color: #ddd transparent #ddd #ddd;
  *border-right-color: #ffffff;
}
.tabs-right .nav-tabs {
  float: right;
  margin-left: 19px;
  border-left: 1px solid #ddd;
}
.tabs-right .nav-tabs > li > a {
  margin-left: -1px;
  -webkit-border-radius: 0 4px 4px 0;
  -moz-border-radius: 0 4px 4px 0;
  border-radius: 0 4px 4px 0;
}
.tabs-right .nav-tabs > li > a:hover {
  border-color: #eeeeee #eeeeee #eeeeee #dddddd;
}
.tabs-right .nav-tabs .active > a,
.tabs-right .nav-tabs .active > a:hover {
  border-color: #ddd #ddd #ddd transparent;
  *border-left-color: #ffffff;
}
.navbar {
  *position: relative;
  *z-index: 2;
  overflow: visible;
  margin-bottom: 18px;
}
.navbar-inner {
  padding-left: 20px;
  padding-right: 20px;
  background-color: #2c2c2c;
  background-image: -moz-linear-gradient(top, #333333, #222222);
  background-image: -ms-linear-gradient(top, #333333, #222222);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#333333), to(#222222));
  background-image: -webkit-linear-gradient(top, #333333, #222222);
  background-image: -o-linear-gradient(top, #333333, #222222);
  background-image: linear-gradient(top, #333333, #222222);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#333333', endColorstr='#222222', GradientType=0);
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25), inset 0 -1px 0 rgba(0, 0, 0, 0.1);
  -moz-box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25), inset 0 -1px 0 rgba(0, 0, 0, 0.1);
  box-shadow: 0 1px 3px rgba(0, 0, 0, 0.25), inset 0 -1px 0 rgba(0, 0, 0, 0.1);
}
.navbar .container {
  width: auto;
}
.btn-navbar {
  display: none;
  float: right;
  padding: 7px 10px;
  margin-left: 5px;
  margin-right: 5px;
  background-color: #2c2c2c;
  background-image: -moz-linear-gradient(top, #333333, #222222);
  background-image: -ms-linear-gradient(top, #333333, #222222);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#333333), to(#222222));
  background-image: -webkit-linear-gradient(top, #333333, #222222);
  background-image: -o-linear-gradient(top, #333333, #222222);
  background-image: linear-gradient(top, #333333, #222222);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#333333', endColorstr='#222222', GradientType=0);
  border-color: #222222 #222222 #000000;
  border-color: rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.1) rgba(0, 0, 0, 0.25);
  filter: progid:dximagetransform.microsoft.gradient(enabled=false);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.075);
  -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.075);
  box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.075);
}
.btn-navbar:hover,
.btn-navbar:active,
.btn-navbar.active,
.btn-navbar.disabled,
.btn-navbar[disabled] {
  background-color: #222222;
}
.btn-navbar:active,
.btn-navbar.active {
  background-color: #080808 \9;
}
.btn-navbar .icon-bar {
  display: block;
  width: 18px;
  height: 2px;
  background-color: #f5f5f5;
  -webkit-border-radius: 1px;
  -moz-border-radius: 1px;
  border-radius: 1px;
  -webkit-box-shadow: 0 1px 0 rgba(0, 0, 0, 0.25);
  -moz-box-shadow: 0 1px 0 rgba(0, 0, 0, 0.25);
  box-shadow: 0 1px 0 rgba(0, 0, 0, 0.25);
}
.btn-navbar .icon-bar + .icon-bar {
  margin-top: 3px;
}
.nav-collapse.collapse {
  height: auto;
}
.navbar {
  color: #999999;
}
.navbar .brand:hover {
  text-decoration: none;
}
.navbar .brand {
  float: left;
  display: block;
  padding: 8px 20px 12px;
  margin-left: -20px;
  font-size: 20px;
  font-weight: 200;
  line-height: 1;
  color: #ffffff;
}
.navbar .navbar-text {
  margin-bottom: 0;
  line-height: 40px;
}
.navbar .btn,
.navbar .btn-group {
  margin-top: 5px;
}
.navbar .btn-group .btn {
  margin-top: 0;
}
.navbar-form {
  margin-bottom: 0;
  *zoom: 1;
}
.navbar-form:before,
.navbar-form:after {
  display: table;
  content: "";
}
.navbar-form:after {
  clear: both;
}
.navbar-form input,
.navbar-form select,
.navbar-form .radio,
.navbar-form .checkbox {
  margin-top: 5px;
}
.navbar-form input,
.navbar-form select {
  display: inline-block;
  margin-bottom: 0;
}
.navbar-form input[type="image"],
.navbar-form input[type="checkbox"],
.navbar-form input[type="radio"] {
  margin-top: 3px;
}
.navbar-form .input-append,
.navbar-form .input-prepend {
  margin-top: 6px;
  white-space: nowrap;
}
.navbar-form .input-append input,
.navbar-form .input-prepend input {
  margin-top: 0;
}
.navbar-search {
  position: relative;
  float: left;
  margin-top: 6px;
  margin-bottom: 0;
}
.navbar-search .search-query {
  padding: 4px 9px;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
  font-size: 13px;
  font-weight: normal;
  line-height: 1;
  color: #ffffff;
  background-color: #626262;
  border: 1px solid #151515;
  -webkit-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1), 0 1px 0px rgba(255, 255, 255, 0.15);
  -moz-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1), 0 1px 0px rgba(255, 255, 255, 0.15);
  box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1), 0 1px 0px rgba(255, 255, 255, 0.15);
  -webkit-transition: none;
  -moz-transition: none;
  -ms-transition: none;
  -o-transition: none;
  transition: none;
}
.navbar-search .search-query:-moz-placeholder {
  color: #cccccc;
}
.navbar-search .search-query::-webkit-input-placeholder {
  color: #cccccc;
}
.navbar-search .search-query:focus,
.navbar-search .search-query.focused {
  padding: 5px 10px;
  color: #333333;
  text-shadow: 0 1px 0 #ffffff;
  background-color: #ffffff;
  border: 0;
  -webkit-box-shadow: 0 0 3px rgba(0, 0, 0, 0.15);
  -moz-box-shadow: 0 0 3px rgba(0, 0, 0, 0.15);
  box-shadow: 0 0 3px rgba(0, 0, 0, 0.15);
  outline: 0;
}
.navbar-fixed-top,
.navbar-fixed-bottom {
  position: fixed;
  right: 0;
  left: 0;
  z-index: 1030;
  margin-bottom: 0;
}
.navbar-fixed-top .navbar-inner,
.navbar-fixed-bottom .navbar-inner {
  padding-left: 0;
  padding-right: 0;
  -webkit-border-radius: 0;
  -moz-border-radius: 0;
  border-radius: 0;
}
.navbar-fixed-top .container,
.navbar-fixed-bottom .container {
  width: 940px;
}
.navbar-fixed-top {
  top: 0;
}
.navbar-fixed-bottom {
  bottom: 0;
}
.navbar .nav {
  position: relative;
  left: 0;
  display: block;
  float: left;
  margin: 0 10px 0 0;
}
.navbar .nav.pull-right {
  float: right;
}
.navbar .nav > li {
  display: block;
  float: left;
}
.navbar .nav > li > a {
  float: none;
  padding: 10px 10px 11px;
  line-height: 19px;
  color: #999999;
  text-decoration: none;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
}
.navbar .nav > li > a:hover {
  background-color: transparent;
  color: #ffffff;
  text-decoration: none;
}
.navbar .nav .active > a,
.navbar .nav .active > a:hover {
  color: #ffffff;
  text-decoration: none;
  background-color: #222222;
}
.navbar .divider-vertical {
  height: 40px;
  width: 1px;
  margin: 0 9px;
  overflow: hidden;
  background-color: #222222;
  border-right: 1px solid #333333;
}
.navbar .nav.pull-right {
  margin-left: 10px;
  margin-right: 0;
}
.navbar .dropdown-menu {
  margin-top: 1px;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.navbar .dropdown-menu:before {
  content: '';
  display: inline-block;
  border-left: 7px solid transparent;
  border-right: 7px solid transparent;
  border-bottom: 7px solid #ccc;
  border-bottom-color: rgba(0, 0, 0, 0.2);
  position: absolute;
  top: -7px;
  left: 9px;
}
.navbar .dropdown-menu:after {
  content: '';
  display: inline-block;
  border-left: 6px solid transparent;
  border-right: 6px solid transparent;
  border-bottom: 6px solid #ffffff;
  position: absolute;
  top: -6px;
  left: 10px;
}
.navbar-fixed-bottom .dropdown-menu:before {
  border-top: 7px solid #ccc;
  border-top-color: rgba(0, 0, 0, 0.2);
  border-bottom: 0;
  bottom: -7px;
  top: auto;
}
.navbar-fixed-bottom .dropdown-menu:after {
  border-top: 6px solid #ffffff;
  border-bottom: 0;
  bottom: -6px;
  top: auto;
}
.navbar .nav .dropdown-toggle .caret,
.navbar .nav .open.dropdown .caret {
  border-top-color: #ffffff;
  border-bottom-color: #ffffff;
}
.navbar .nav .active .caret {
  opacity: 1;
  filter: alpha(opacity=100);
}
.navbar .nav .open > .dropdown-toggle,
.navbar .nav .active > .dropdown-toggle,
.navbar .nav .open.active > .dropdown-toggle {
  background-color: transparent;
}
.navbar .nav .active > .dropdown-toggle:hover {
  color: #ffffff;
}
.navbar .nav.pull-right .dropdown-menu,
.navbar .nav .dropdown-menu.pull-right {
  left: auto;
  right: 0;
}
.navbar .nav.pull-right .dropdown-menu:before,
.navbar .nav .dropdown-menu.pull-right:before {
  left: auto;
  right: 12px;
}
.navbar .nav.pull-right .dropdown-menu:after,
.navbar .nav .dropdown-menu.pull-right:after {
  left: auto;
  right: 13px;
}
.breadcrumb {
  padding: 7px 14px;
  margin: 0 0 18px;
  list-style: none;
  background-color: #fbfbfb;
  background-image: -moz-linear-gradient(top, #ffffff, #f5f5f5);
  background-image: -ms-linear-gradient(top, #ffffff, #f5f5f5);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), to(#f5f5f5));
  background-image: -webkit-linear-gradient(top, #ffffff, #f5f5f5);
  background-image: -o-linear-gradient(top, #ffffff, #f5f5f5);
  background-image: linear-gradient(top, #ffffff, #f5f5f5);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#f5f5f5', GradientType=0);
  border: 1px solid #ddd;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
  -webkit-box-shadow: inset 0 1px 0 #ffffff;
  -moz-box-shadow: inset 0 1px 0 #ffffff;
  box-shadow: inset 0 1px 0 #ffffff;
}
.breadcrumb li {
  display: inline-block;
  *display: inline;
  /* IE7 inline-block hack */

  *zoom: 1;
  text-shadow: 0 1px 0 #ffffff;
}
.breadcrumb .divider {
  padding: 0 5px;
  color: #999999;
}
.breadcrumb .active a {
  color: #333333;
}
.pagination {
  height: 36px;
  margin: 18px 0;
}
.pagination ul {
  display: inline-block;
  *display: inline;
  /* IE7 inline-block hack */

  *zoom: 1;
  margin-left: 0;
  margin-bottom: 0;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
  -webkit-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
  -moz-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
  box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
}
.pagination li {
  display: inline;
}
.pagination a {
  float: left;
  padding: 0 14px;
  line-height: 34px;
  text-decoration: none;
  border: 1px solid #ddd;
  border-left-width: 0;
}
.pagination a:hover,
.pagination .active a {
  background-color: #f5f5f5;
}
.pagination .active a {
  color: #999999;
  cursor: default;
}
.pagination .disabled span,
.pagination .disabled a,
.pagination .disabled a:hover {
  color: #999999;
  background-color: transparent;
  cursor: default;
}
.pagination li:first-child a {
  border-left-width: 1px;
  -webkit-border-radius: 3px 0 0 3px;
  -moz-border-radius: 3px 0 0 3px;
  border-radius: 3px 0 0 3px;
}
.pagination li:last-child a {
  -webkit-border-radius: 0 3px 3px 0;
  -moz-border-radius: 0 3px 3px 0;
  border-radius: 0 3px 3px 0;
}
.pagination-centered {
  text-align: center;
}
.pagination-right {
  text-align: right;
}
.pager {
  margin-left: 0;
  margin-bottom: 18px;
  list-style: none;
  text-align: center;
  *zoom: 1;
}
.pager:before,
.pager:after {
  display: table;
  content: "";
}
.pager:after {
  clear: both;
}
.pager li {
  display: inline;
}
.pager a {
  display: inline-block;
  padding: 5px 14px;
  background-color: #fff;
  border: 1px solid #ddd;
  -webkit-border-radius: 15px;
  -moz-border-radius: 15px;
  border-radius: 15px;
}
.pager a:hover {
  text-decoration: none;
  background-color: #f5f5f5;
}
.pager .next a {
  float: right;
}
.pager .previous a {
  float: left;
}
.pager .disabled a,
.pager .disabled a:hover {
  color: #999999;
  background-color: #fff;
  cursor: default;
}
.modal-open .dropdown-menu {
  z-index: 2050;
}
.modal-open .dropdown.open {
  *z-index: 2050;
}
.modal-open .popover {
  z-index: 2060;
}
.modal-open .tooltip {
  z-index: 2070;
}
.modal-backdrop {
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1040;
  /*TRPbackground-color: #000000;*/
}
.modal-backdrop.fade {
  opacity: 0;
}
.modal-backdrop,
.modal-backdrop.fade.in {
  opacity:0.5;
filter: alpha(opacity=50);
  /*TRP opacity: 0.8;
  filter: alpha(opacity=80);*/
}
.modal {
  position: fixed;
  top: 50%;
  left: 50%;
  z-index: 1050;
  overflow: auto;
  width: 560px;
  margin: -250px 0 0 -280px;
  background-color: #ffffff;
  border: 1px solid #999;
  border: 1px solid rgba(0, 0, 0, 0.3);
  *border: 1px solid #999;
  /* IE6-7 */

  -webkit-border-radius: 6px;
  -moz-border-radius: 6px;
  border-radius: 6px;
  -webkit-box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
  -moz-box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
  box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
  -webkit-background-clip: padding-box;
  -moz-background-clip: padding-box;
  background-clip: padding-box;
}
.modal.fade {
  -webkit-transition: opacity .3s linear, top .3s ease-out;
  -moz-transition: opacity .3s linear, top .3s ease-out;
  -ms-transition: opacity .3s linear, top .3s ease-out;
  -o-transition: opacity .3s linear, top .3s ease-out;
  transition: opacity .3s linear, top .3s ease-out;
  top: -25%;
}
.modal.fade.in {
  top: 50%;
}
.modal-header {
  padding: 9px 15px;
  border-bottom: 1px solid #eee;
}
.modal-header .close {
  margin-top: 2px;
}
.modal-body {
  overflow-y: auto;
  max-height: 400px;
  padding: 15px;
}
.modal-form {
  margin-bottom: 0;
}
.modal-footer {
  padding: 14px 15px 15px;
  margin-bottom: 0;
  text-align: right;
  background-color: #f5f5f5;
  border-top: 1px solid #ddd;
  -webkit-border-radius: 0 0 6px 6px;
  -moz-border-radius: 0 0 6px 6px;
  border-radius: 0 0 6px 6px;
  -webkit-box-shadow: inset 0 1px 0 #ffffff;
  -moz-box-shadow: inset 0 1px 0 #ffffff;
  box-shadow: inset 0 1px 0 #ffffff;
  *zoom: 1;
}
.modal-footer:before,
.modal-footer:after {
  display: table;
  content: "";
}
.modal-footer:after {
  clear: both;
}
.modal-footer .btn + .btn {
  margin-left: 5px;
  margin-bottom: 0;
}
.modal-footer .btn-group .btn + .btn {
  margin-left: -1px;
}
.tooltip {
  position: absolute;
  z-index: 1020;
  display: block;
  visibility: visible;
  padding: 5px;
  font-size: 11px;
  opacity: 0;
  filter: alpha(opacity=0);
}
.tooltip.in {
  opacity: 0.8;
  filter: alpha(opacity=80);
}
.tooltip.top {
  margin-top: -2px;
}
.tooltip.right {
  margin-left: 2px;
}
.tooltip.bottom {
  margin-top: 2px;
}
.tooltip.left {
  margin-left: -2px;
}
.tooltip.top .tooltip-arrow {
  bottom: 0;
  left: 50%;
  margin-left: -5px;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  border-top: 5px solid #000000;
}
.tooltip.left .tooltip-arrow {
  top: 50%;
  right: 0;
  margin-top: -5px;
  border-top: 5px solid transparent;
  border-bottom: 5px solid transparent;
  border-left: 5px solid #000000;
}
.tooltip.bottom .tooltip-arrow {
  top: 0;
  left: 50%;
  margin-left: -5px;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  border-bottom: 5px solid #000000;
}
.tooltip.right .tooltip-arrow {
  top: 50%;
  left: 0;
  margin-top: -5px;
  border-top: 5px solid transparent;
  border-bottom: 5px solid transparent;
  border-right: 5px solid #000000;
}
.tooltip-inner {
  max-width: 200px;
  padding: 3px 8px;
  color: #ffffff;
  text-align: center;
  text-decoration: none;
  background-color: #000000;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.tooltip-arrow {
  position: absolute;
  width: 0;
  height: 0;
}
.popover {
  position: absolute;
  top: 0;
  left: 0;
  z-index: 1010;
  display: none;
  padding: 5px;
}
.popover.top {
  margin-top: -5px;
}
.popover.right {
  margin-left: 5px;
}
.popover.bottom {
  margin-top: 5px;
}
.popover.left {
  margin-left: -5px;
}
.popover.top .arrow {
  bottom: 0;
  left: 50%;
  margin-left: -5px;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  border-top: 5px solid #000000;
}
.popover.right .arrow {
  top: 50%;
  left: 0;
  margin-top: -5px;
  border-top: 5px solid transparent;
  border-bottom: 5px solid transparent;
  border-right: 5px solid #000000;
}
.popover.bottom .arrow {
  top: 0;
  left: 50%;
  margin-left: -5px;
  border-left: 5px solid transparent;
  border-right: 5px solid transparent;
  border-bottom: 5px solid #000000;
}
.popover.left .arrow {
  top: 50%;
  right: 0;
  margin-top: -5px;
  border-top: 5px solid transparent;
  border-bottom: 5px solid transparent;
  border-left: 5px solid #000000;
}
.popover .arrow {
  position: absolute;
  width: 0;
  height: 0;
}
.popover-inner {
  padding: 3px;
  width: 280px;
  overflow: hidden;
  background: #000000;
  background: rgba(0, 0, 0, 0.8);
  -webkit-border-radius: 6px;
  -moz-border-radius: 6px;
  border-radius: 6px;
  -webkit-box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
  -moz-box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
  box-shadow: 0 3px 7px rgba(0, 0, 0, 0.3);
}
.popover-title {
  padding: 9px 15px;
  line-height: 1;
  background-color: #f5f5f5;
  border-bottom: 1px solid #eee;
  -webkit-border-radius: 3px 3px 0 0;
  -moz-border-radius: 3px 3px 0 0;
  border-radius: 3px 3px 0 0;
}
.popover-content {
  padding: 14px;
  background-color: #ffffff;
  -webkit-border-radius: 0 0 3px 3px;
  -moz-border-radius: 0 0 3px 3px;
  border-radius: 0 0 3px 3px;
  -webkit-background-clip: padding-box;
  -moz-background-clip: padding-box;
  background-clip: padding-box;
}
.popover-content p,
.popover-content ul,
.popover-content ol {
  margin-bottom: 0;
}
.thumbnails {
  margin-left: -20px;
  list-style: none;
  *zoom: 1;
}
.thumbnails:before,
.thumbnails:after {
  display: table;
  content: "";
}
.thumbnails:after {
  clear: both;
}
.thumbnails > li {
  float: left;
  margin: 0 0 18px 20px;
}
.thumbnail {
  display: block;
  padding: 4px;
  line-height: 1;
  border: 1px solid #ddd;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.075);
  -moz-box-shadow: 0 1px 1px rgba(0, 0, 0, 0.075);
  box-shadow: 0 1px 1px rgba(0, 0, 0, 0.075);
}
a.thumbnail:hover {
  border-color: #0088cc;
  -webkit-box-shadow: 0 1px 4px rgba(0, 105, 214, 0.25);
  -moz-box-shadow: 0 1px 4px rgba(0, 105, 214, 0.25);
  box-shadow: 0 1px 4px rgba(0, 105, 214, 0.25);
}
.thumbnail > img {
  display: block;
  max-width: 100%;
  margin-left: auto;
  margin-right: auto;
}
.thumbnail .caption {
  padding: 9px;
}
.label {
  padding: 1px 4px 2px;
  font-size: 10.998px;
  font-weight: bold;
  line-height: 13px;
  color: #ffffff;
  vertical-align: middle;
  white-space: nowrap;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  background-color: #999999;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}
.label:hover {
  color: #ffffff;
  text-decoration: none;
}
.label-important {
  background-color: #b94a48;
}
.label-important:hover {
  background-color: #953b39;
}
.label-warning {
  background-color: #f89406;
}
.label-warning:hover {
  background-color: #c67605;
}
.label-success {
  background-color: #468847;
}
.label-success:hover {
  background-color: #356635;
}
.label-info {
  background-color: #3a87ad;
}
.label-info:hover {
  background-color: #2d6987;
}
.label-inverse {
  background-color: #333333;
}
.label-inverse:hover {
  background-color: #1a1a1a;
}
.badge {
  padding: 1px 9px 2px;
  font-size: 12.025px;
  font-weight: bold;
  white-space: nowrap;
  color: #ffffff;
  background-color: #999999;
  -webkit-border-radius: 9px;
  -moz-border-radius: 9px;
  border-radius: 9px;
}
.badge:hover {
  color: #ffffff;
  text-decoration: none;
  cursor: pointer;
}
.badge-error {
  background-color: #b94a48;
}
.badge-error:hover {
  background-color: #953b39;
}
.badge-warning {
  background-color: #f89406;
}
.badge-warning:hover {
  background-color: #c67605;
}
.badge-success {
  background-color: #468847;
}
.badge-success:hover {
  background-color: #356635;
}
.badge-info {
  background-color: #3a87ad;
}
.badge-info:hover {
  background-color: #2d6987;
}
.badge-inverse {
  background-color: #333333;
}
.badge-inverse:hover {
  background-color: #1a1a1a;
}
@-webkit-keyframes progress-bar-stripes {
  from {
    background-position: 0 0;
  }
  to {
    background-position: 40px 0;
  }
}
@-moz-keyframes progress-bar-stripes {
  from {
    background-position: 0 0;
  }
  to {
    background-position: 40px 0;
  }
}
@-ms-keyframes progress-bar-stripes {
  from {
    background-position: 0 0;
  }
  to {
    background-position: 40px 0;
  }
}
@keyframes progress-bar-stripes {
  from {
    background-position: 0 0;
  }
  to {
    background-position: 40px 0;
  }
}
.progress {
  overflow: hidden;
  height: 18px;
  margin-bottom: 18px;
  background-color: #f7f7f7;
  background-image: -moz-linear-gradient(top, #f5f5f5, #f9f9f9);
  background-image: -ms-linear-gradient(top, #f5f5f5, #f9f9f9);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#f5f5f5), to(#f9f9f9));
  background-image: -webkit-linear-gradient(top, #f5f5f5, #f9f9f9);
  background-image: -o-linear-gradient(top, #f5f5f5, #f9f9f9);
  background-image: linear-gradient(top, #f5f5f5, #f9f9f9);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#f5f5f5', endColorstr='#f9f9f9', GradientType=0);
  -webkit-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1);
  -moz-box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1);
  box-shadow: inset 0 1px 2px rgba(0, 0, 0, 0.1);
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.progress .bar {
  width: 0%;
  height: 18px;
  color: #ffffff;
  font-size: 12px;
  text-align: center;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
  background-color: #0e90d2;
  background-image: -moz-linear-gradient(top, #149bdf, #0480be);
  background-image: -ms-linear-gradient(top, #149bdf, #0480be);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#149bdf), to(#0480be));
  background-image: -webkit-linear-gradient(top, #149bdf, #0480be);
  background-image: -o-linear-gradient(top, #149bdf, #0480be);
  background-image: linear-gradient(top, #149bdf, #0480be);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#149bdf', endColorstr='#0480be', GradientType=0);
  -webkit-box-shadow: inset 0 -1px 0 rgba(0, 0, 0, 0.15);
  -moz-box-shadow: inset 0 -1px 0 rgba(0, 0, 0, 0.15);
  box-shadow: inset 0 -1px 0 rgba(0, 0, 0, 0.15);
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  box-sizing: border-box;
  -webkit-transition: width 0.6s ease;
  -moz-transition: width 0.6s ease;
  -ms-transition: width 0.6s ease;
  -o-transition: width 0.6s ease;
  transition: width 0.6s ease;
}
.progress-striped .bar {
  background-color: #149bdf;
  background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.15)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.15)), color-stop(0.75, rgba(255, 255, 255, 0.15)), color-stop(0.75, transparent), to(transparent));
  background-image: -webkit-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -ms-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -o-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  -webkit-background-size: 40px 40px;
  -moz-background-size: 40px 40px;
  -o-background-size: 40px 40px;
  background-size: 40px 40px;
}
.progress.active .bar {
  -webkit-animation: progress-bar-stripes 2s linear infinite;
  -moz-animation: progress-bar-stripes 2s linear infinite;
  animation: progress-bar-stripes 2s linear infinite;
}
.progress-danger .bar {
  background-color: #dd514c;
  background-image: -moz-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -ms-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ee5f5b), to(#c43c35));
  background-image: -webkit-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: -o-linear-gradient(top, #ee5f5b, #c43c35);
  background-image: linear-gradient(top, #ee5f5b, #c43c35);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ee5f5b', endColorstr='#c43c35', GradientType=0);
}
.progress-danger.progress-striped .bar {
  background-color: #ee5f5b;
  background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.15)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.15)), color-stop(0.75, rgba(255, 255, 255, 0.15)), color-stop(0.75, transparent), to(transparent));
  background-image: -webkit-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -ms-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -o-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
}
.progress-success .bar {
  background-color: #5eb95e;
  background-image: -moz-linear-gradient(top, #62c462, #57a957);
  background-image: -ms-linear-gradient(top, #62c462, #57a957);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#62c462), to(#57a957));
  background-image: -webkit-linear-gradient(top, #62c462, #57a957);
  background-image: -o-linear-gradient(top, #62c462, #57a957);
  background-image: linear-gradient(top, #62c462, #57a957);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#62c462', endColorstr='#57a957', GradientType=0);
}
.progress-success.progress-striped .bar {
  background-color: #62c462;
  background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.15)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.15)), color-stop(0.75, rgba(255, 255, 255, 0.15)), color-stop(0.75, transparent), to(transparent));
  background-image: -webkit-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -ms-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -o-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
}
.progress-info .bar {
  background-color: #4bb1cf;
  background-image: -moz-linear-gradient(top, #5bc0de, #339bb9);
  background-image: -ms-linear-gradient(top, #5bc0de, #339bb9);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#5bc0de), to(#339bb9));
  background-image: -webkit-linear-gradient(top, #5bc0de, #339bb9);
  background-image: -o-linear-gradient(top, #5bc0de, #339bb9);
  background-image: linear-gradient(top, #5bc0de, #339bb9);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#5bc0de', endColorstr='#339bb9', GradientType=0);
}
.progress-info.progress-striped .bar {
  background-color: #5bc0de;
  background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.15)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.15)), color-stop(0.75, rgba(255, 255, 255, 0.15)), color-stop(0.75, transparent), to(transparent));
  background-image: -webkit-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -ms-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -o-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
}
.progress-warning .bar {
  background-color: #faa732;
  background-image: -moz-linear-gradient(top, #fbb450, #f89406);
  background-image: -ms-linear-gradient(top, #fbb450, #f89406);
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#fbb450), to(#f89406));
  background-image: -webkit-linear-gradient(top, #fbb450, #f89406);
  background-image: -o-linear-gradient(top, #fbb450, #f89406);
  background-image: linear-gradient(top, #fbb450, #f89406);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fbb450', endColorstr='#f89406', GradientType=0);
}
.progress-warning.progress-striped .bar {
  background-color: #fbb450;
  background-image: -webkit-gradient(linear, 0 100%, 100% 0, color-stop(0.25, rgba(255, 255, 255, 0.15)), color-stop(0.25, transparent), color-stop(0.5, transparent), color-stop(0.5, rgba(255, 255, 255, 0.15)), color-stop(0.75, rgba(255, 255, 255, 0.15)), color-stop(0.75, transparent), to(transparent));
  background-image: -webkit-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -moz-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -ms-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: -o-linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
}
.accordion {
  margin-bottom: 18px;
}
.accordion-group {
  margin-bottom: 2px;
  border: 1px solid #e5e5e5;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.accordion-heading {
  border-bottom: 0;
}
.accordion-heading .accordion-toggle {
  display: block;
  padding: 8px 15px;
}
.accordion-inner {
  padding: 9px 15px;
  border-top: 1px solid #e5e5e5;
}
.carousel {
  position: relative;
  margin-bottom: 18px;
  line-height: 1;
}
.carousel-inner {
  overflow: hidden;
  width: 100%;
  position: relative;
}
.carousel .item {
  display: none;
  position: relative;
  -webkit-transition: 0.6s ease-in-out left;
  -moz-transition: 0.6s ease-in-out left;
  -ms-transition: 0.6s ease-in-out left;
  -o-transition: 0.6s ease-in-out left;
  transition: 0.6s ease-in-out left;
}
.carousel .item > img {
  display: block;
  line-height: 1;
}
.carousel .active,
.carousel .next,
.carousel .prev {
  display: block;
}
.carousel .active {
  left: 0;
}
.carousel .next,
.carousel .prev {
  position: absolute;
  top: 0;
  width: 100%;
}
.carousel .next {
  left: 100%;
}
.carousel .prev {
  left: -100%;
}
.carousel .next.left,
.carousel .prev.right {
  left: 0;
}
.carousel .active.left {
  left: -100%;
}
.carousel .active.right {
  left: 100%;
}
.carousel-control {
  position: absolute;
  top: 40%;
  left: 15px;
  width: 40px;
  height: 40px;
  margin-top: -20px;
  font-size: 60px;
  font-weight: 100;
  line-height: 30px;
  color: #ffffff;
  text-align: center;
  background: #222222;
  border: 3px solid #ffffff;
  -webkit-border-radius: 23px;
  -moz-border-radius: 23px;
  border-radius: 23px;
  opacity: 0.5;
  filter: alpha(opacity=50);
}
.carousel-control.right {
  left: auto;
  right: 15px;
}
.carousel-control:hover {
  color: #ffffff;
  text-decoration: none;
  opacity: 0.9;
  filter: alpha(opacity=90);
}
.carousel-caption {
  position: absolute;
  left: 0;
  right: 0;
  bottom: 0;
  padding: 10px 15px 5px;
  background: #333333;
  background: rgba(0, 0, 0, 0.75);
}
.carousel-caption h4,
.carousel-caption p {
  color: #ffffff;
}
.hero-unit {
  padding: 60px;
  margin-bottom: 30px;
  background-color: #eeeeee;
  -webkit-border-radius: 6px;
  -moz-border-radius: 6px;
  border-radius: 6px;
}
.hero-unit h1 {
  margin-bottom: 0;
  font-size: 60px;
  line-height: 1;
  color: inherit;
  letter-spacing: -1px;
}
.hero-unit p {
  font-size: 18px;
  font-weight: 200;
  line-height: 27px;
  color: inherit;
}
.pull-right {
  float: right;
}
.pull-left {
  float: left;
}
.hide {
  display: none;
}
.show {
  display: block;
}
.invisible {
  visibility: hidden;
}
/*!
 * Bootstrap Responsive v2.0.2
 *
 * Copyright 2012 Twitter, Inc
 * Licensed under the Apache License v2.0
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Designed and built with all the love in the world @twitter by @mdo and @fat.
 */
.clearfix {
  *zoom: 1;
}
.clearfix:before,
.clearfix:after {
  display: table;
  content: "";
}
.clearfix:after {
  clear: both;
}
.hide-text {
  overflow: hidden;
  text-indent: 100%;
  white-space: nowrap;
}
.input-block-level {
  display: block;
  width: 100%;
  min-height: 28px;
  /* Make inputs at least the height of their button counterpart */

  /* Makes inputs behave like true block-level elements */

  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  -ms-box-sizing: border-box;
  box-sizing: border-box;
}
.hidden {
  display: none;
  visibility: hidden;
}
.visible-phone {
  display: none;
}
.visible-tablet {
  display: none;
}
.visible-desktop {
  display: block;
}
.hidden-phone {
  display: block;
}
.hidden-tablet {
  display: block;
}
.hidden-desktop {
  display: none;
}
@media (max-width: 767px) {
  .visible-phone {
    display: block;
  }
  .hidden-phone {
    display: none;
  }
  .hidden-desktop {
    display: block;
  }
  .visible-desktop {
    display: none;
  }
}
@media (min-width: 768px) and (max-width: 979px) {
  .visible-tablet {
    display: block;
  }
  .hidden-tablet {
    display: none;
  }
  .hidden-desktop {
    display: block;
  }
  .visible-desktop {
    display: none;
  }
}
@media (max-width: 480px) {
  .nav-collapse {
    -webkit-transform: translate3d(0, 0, 0);
  }
  .page-header h1 small {
    display: block;
    line-height: 18px;
  }
  input[type="checkbox"],
  input[type="radio"] {
    border: 1px solid #ccc;
  }
  .form-horizontal .control-group > label {
    float: none;
    width: auto;
    padding-top: 0;
    text-align: left;
  }
  .form-horizontal .controls {
    margin-left: 0;
  }
  .form-horizontal .control-list {
    padding-top: 0;
  }
  .form-horizontal .form-actions {
    padding-left: 10px;
    padding-right: 10px;
  }
  .modal {
    position: absolute;
    top: 10px;
    left: 10px;
    right: 10px;
    width: auto;
    margin: 0;
  }
  .modal.fade.in {
    top: auto;
  }
  .modal-header .close {
    padding: 10px;
    margin: -10px;
  }
  .carousel-caption {
    position: static;
  }
}
@media (max-width: 767px) {
  body {
    padding-left: 20px;
    padding-right: 20px;
  }
  .navbar-fixed-top {
    margin-left: -20px;
    margin-right: -20px;
  }
  .container {
    width: auto;
  }
  .row-fluid {
    width: 100%;
  }
  .row {
    margin-left: 0;
  }
  .row > [class*="span"],
  .row-fluid > [class*="span"] {
    float: none;
    display: block;
    width: auto;
    margin: 0;
  }
  .thumbnails [class*="span"] {
    width: auto;
  }
  input[class*="span"],
  select[class*="span"],
  textarea[class*="span"],
  .uneditable-input {
    display: block;
    width: 100%;
    min-height: 28px;
    /* Make inputs at least the height of their button counterpart */
  
    /* Makes inputs behave like true block-level elements */
  
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    -ms-box-sizing: border-box;
    box-sizing: border-box;
  }
  .input-prepend input[class*="span"],
  .input-append input[class*="span"] {
    width: auto;
  }
}
@media (min-width: 768px) and (max-width: 979px) {
  .row {
    margin-left: -20px;
    *zoom: 1;
  }
  .row:before,
  .row:after {
    display: table;
    content: "";
  }
  .row:after {
    clear: both;
  }
  [class*="span"] {
    float: left;
    margin-left: 20px;
  }
  .container,
  .navbar-fixed-top .container,
  .navbar-fixed-bottom .container {
    width: 724px;
  }
  .span12 {
    width: 724px;
  }
  .span11 {
    width: 662px;
  }
  .span10 {
    width: 600px;
  }
  .span9 {
    width: 538px;
  }
  .span8 {
    width: 476px;
  }
  .span7 {
    width: 414px;
  }
  .span6 {
    width: 352px;
  }
  .span5 {
    width: 290px;
  }
  .span4 {
    width: 228px;
  }
  .span3 {
    width: 166px;
  }
  .span2 {
    width: 104px;
  }
  .span1 {
    width: 42px;
  }
  .offset12 {
    margin-left: 764px;
  }
  .offset11 {
    margin-left: 702px;
  }
  .offset10 {
    margin-left: 640px;
  }
  .offset9 {
    margin-left: 578px;
  }
  .offset8 {
    margin-left: 516px;
  }
  .offset7 {
    margin-left: 454px;
  }
  .offset6 {
    margin-left: 392px;
  }
  .offset5 {
    margin-left: 330px;
  }
  .offset4 {
    margin-left: 268px;
  }
  .offset3 {
    margin-left: 206px;
  }
  .offset2 {
    margin-left: 144px;
  }
  .offset1 {
    margin-left: 82px;
  }
  .row-fluid {
    width: 100%;
    *zoom: 1;
  }
  .row-fluid:before,
  .row-fluid:after {
    display: table;
    content: "";
  }
  .row-fluid:after {
    clear: both;
  }
  .row-fluid > [class*="span"] {
    float: left;
    margin-left: 2.762430939%;
  }
  .row-fluid > [class*="span"]:first-child {
    margin-left: 0;
  }
  .row-fluid > .span12 {
    width: 99.999999993%;
  }
  .row-fluid > .span11 {
    width: 91.436464082%;
  }
  .row-fluid > .span10 {
    width: 82.87292817100001%;
  }
  .row-fluid > .span9 {
    width: 74.30939226%;
  }
  .row-fluid > .span8 {
    width: 65.74585634900001%;
  }
  .row-fluid > .span7 {
    width: 57.182320438000005%;
  }
  .row-fluid > .span6 {
    width: 48.618784527%;
  }
  .row-fluid > .span5 {
    width: 40.055248616%;
  }
  .row-fluid > .span4 {
    width: 31.491712705%;
  }
  .row-fluid > .span3 {
    width: 22.928176794%;
  }
  .row-fluid > .span2 {
    width: 14.364640883%;
  }
  .row-fluid > .span1 {
    width: 5.801104972%;
  }
  input,
  textarea,
  .uneditable-input {
    margin-left: 0;
  }
  input.span12, textarea.span12, .uneditable-input.span12 {
    width: 714px;
  }
  input.span11, textarea.span11, .uneditable-input.span11 {
    width: 652px;
  }
  input.span10, textarea.span10, .uneditable-input.span10 {
    width: 590px;
  }
  input.span9, textarea.span9, .uneditable-input.span9 {
    width: 528px;
  }
  input.span8, textarea.span8, .uneditable-input.span8 {
    width: 466px;
  }
  input.span7, textarea.span7, .uneditable-input.span7 {
    width: 404px;
  }
  input.span6, textarea.span6, .uneditable-input.span6 {
    width: 342px;
  }
  input.span5, textarea.span5, .uneditable-input.span5 {
    width: 280px;
  }
  input.span4, textarea.span4, .uneditable-input.span4 {
    width: 218px;
  }
  input.span3, textarea.span3, .uneditable-input.span3 {
    width: 156px;
  }
  input.span2, textarea.span2, .uneditable-input.span2 {
    width: 94px;
  }
  input.span1, textarea.span1, .uneditable-input.span1 {
    width: 32px;
  }
}
@media (max-width: 979px) {
  body {
    padding-top: 0;
  }
  .navbar-fixed-top {
    position: static;
    margin-bottom: 18px;
  }
  .navbar-fixed-top .navbar-inner {
    padding: 5px;
  }
  .navbar .container {
    width: auto;
    padding: 0;
  }
  .navbar .brand {
    padding-left: 10px;
    padding-right: 10px;
    margin: 0 0 0 -5px;
  }
  .navbar .nav-collapse {
    clear: left;
  }
  .navbar .nav {
    float: none;
    margin: 0 0 9px;
  }
  .navbar .nav > li {
    float: none;
  }
  .navbar .nav > li > a {
    margin-bottom: 2px;
  }
  .navbar .nav > .divider-vertical {
    display: none;
  }
  .navbar .nav .nav-header {
    color: #999999;
    text-shadow: none;
  }
  .navbar .nav > li > a,
  .navbar .dropdown-menu a {
    padding: 6px 15px;
    font-weight: bold;
    color: #999999;
    -webkit-border-radius: 3px;
    -moz-border-radius: 3px;
    border-radius: 3px;
  }
  .navbar .dropdown-menu li + li a {
    margin-bottom: 2px;
  }
  .navbar .nav > li > a:hover,
  .navbar .dropdown-menu a:hover {
    background-color: #222222;
  }
  .navbar .dropdown-menu {
    position: static;
    top: auto;
    left: auto;
    float: none;
    display: block;
    max-width: none;
    margin: 0 15px;
    padding: 0;
    background-color: transparent;
    border: none;
    -webkit-border-radius: 0;
    -moz-border-radius: 0;
    border-radius: 0;
    -webkit-box-shadow: none;
    -moz-box-shadow: none;
    box-shadow: none;
  }
  .navbar .dropdown-menu:before,
  .navbar .dropdown-menu:after {
    display: none;
  }
  .navbar .dropdown-menu .divider {
    display: none;
  }
  .navbar-form,
  .navbar-search {
    float: none;
    padding: 9px 15px;
    margin: 9px 0;
    border-top: 1px solid #222222;
    border-bottom: 1px solid #222222;
    -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.1);
    -moz-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.1);
    box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.1), 0 1px 0 rgba(255, 255, 255, 0.1);
  }
  .navbar .nav.pull-right {
    float: none;
    margin-left: 0;
  }
  .navbar-static .navbar-inner {
    padding-left: 10px;
    padding-right: 10px;
  }
  .btn-navbar {
    display: block;
  }
  .nav-collapse {
    overflow: hidden;
    height: 0;
  }
}
@media (min-width: 980px) {
  .nav-collapse.collapse {
    height: auto !important;
    overflow: visible !important;
  }
}
@media (min-width: 1200px) {
  .row {
    margin-left: -30px;
    *zoom: 1;
  }
  .row:before,
  .row:after {
    display: table;
    content: "";
  }
  .row:after {
    clear: both;
  }
  [class*="span"] {
    float: left;
    margin-left: 30px;
  }
  .container,
  .navbar-fixed-top .container,
  .navbar-fixed-bottom .container {
    width: 1170px;
  }
  .span12 {
    width: 1170px;
  }
  .span11 {
    width: 1070px;
  }
  .span10 {
    width: 970px;
  }
  .span9 {
    width: 870px;
  }
  .span8 {
    width: 770px;
  }
  .span7 {
    width: 670px;
  }
  .span6 {
    width: 570px;
  }
  .span5 {
    width: 470px;
  }
  .span4 {
    width: 370px;
  }
  .span3 {
    width: 270px;
  }
  .span2 {
    width: 170px;
  }
  .span1 {
    width: 70px;
  }
  .offset12 {
    margin-left: 1230px;
  }
  .offset11 {
    margin-left: 1130px;
  }
  .offset10 {
    margin-left: 1030px;
  }
  .offset9 {
    margin-left: 930px;
  }
  .offset8 {
    margin-left: 830px;
  }
  .offset7 {
    margin-left: 730px;
  }
  .offset6 {
    margin-left: 630px;
  }
  .offset5 {
    margin-left: 530px;
  }
  .offset4 {
    margin-left: 430px;
  }
  .offset3 {
    margin-left: 330px;
  }
  .offset2 {
    margin-left: 230px;
  }
  .offset1 {
    margin-left: 130px;
  }
  .row-fluid {
    width: 100%;
    *zoom: 1;
  }
  .row-fluid:before,
  .row-fluid:after {
    display: table;
    content: "";
  }
  .row-fluid:after {
    clear: both;
  }
  .row-fluid > [class*="span"] {
    float: left;
    margin-left: 2.564102564%;
  }
  .row-fluid > [class*="span"]:first-child {
    margin-left: 0;
  }
  .row-fluid > .span12 {
    width: 100%;
  }
  .row-fluid > .span11 {
    width: 91.45299145300001%;
  }
  .row-fluid > .span10 {
    width: 82.905982906%;
  }
  .row-fluid > .span9 {
    width: 74.358974359%;
  }
  .row-fluid > .span8 {
    width: 65.81196581200001%;
  }
  .row-fluid > .span7 {
    width: 57.264957265%;
  }
  .row-fluid > .span6 {
    width: 48.717948718%;
  }
  .row-fluid > .span5 {
    width: 40.170940171000005%;
  }
  .row-fluid > .span4 {
    width: 31.623931624%;
  }
  .row-fluid > .span3 {
    width: 23.076923077%;
  }
  .row-fluid > .span2 {
    width: 14.529914530000001%;
  }
  .row-fluid > .span1 {
    width: 5.982905983%;
  }
  input,
  textarea,
  .uneditable-input {
    margin-left: 0;
  }
  input.span12, textarea.span12, .uneditable-input.span12 {
    width: 1160px;
  }
  input.span11, textarea.span11, .uneditable-input.span11 {
    width: 1060px;
  }
  input.span10, textarea.span10, .uneditable-input.span10 {
    width: 960px;
  }
  input.span9, textarea.span9, .uneditable-input.span9 {
    width: 860px;
  }
  input.span8, textarea.span8, .uneditable-input.span8 {
    width: 760px;
  }
  input.span7, textarea.span7, .uneditable-input.span7 {
    width: 660px;
  }
  input.span6, textarea.span6, .uneditable-input.span6 {
    width: 560px;
  }
  input.span5, textarea.span5, .uneditable-input.span5 {
    width: 460px;
  }
  input.span4, textarea.span4, .uneditable-input.span4 {
    width: 360px;
  }
  input.span3, textarea.span3, .uneditable-input.span3 {
    width: 260px;
  }
  input.span2, textarea.span2, .uneditable-input.span2 {
    width: 160px;
  }
  input.span1, textarea.span1, .uneditable-input.span1 {
    width: 60px;
  }
  .thumbnails {
    margin-left: -30px;
  }
  .thumbnails > li {
    margin-left: 30px;
  }
}
/*  Font Awesome 3.0
    the iconic font designed for use with Twitter Bootstrap
    -------------------------------------------------------
    The full suite of pictographic icons, examples, and documentation
    can be found at: http://fortawesome.github.com/Font-Awesome/

    License
    -------------------------------------------------------
    • The Font Awesome font is licensed under the SIL Open Font License - http://scripts.sil.org/OFL
    • Font Awesome CSS, LESS, and SASS files are licensed under the MIT License -
      http://opensource.org/licenses/mit-license.html
    • The Font Awesome pictograms are licensed under the CC BY 3.0 License - http://creativecommons.org/licenses/by/3.0/
    • Attribution is no longer required in Font Awesome 3.0, but much appreciated:
      "Font Awesome by Dave Gandy - http://fortawesome.github.com/Font-Awesome"

    Contact
    -------------------------------------------------------
    Email: dave@davegandy.com
    Twitter: http://twitter.com/fortaweso_me
    Work: Lead Product Designer @ http://kyruus.com

    */
@font-face {
  font-family: 'FontAwesome';
  src: url('/assets/thirdParty/FortAwesome-Font-Awesome-12c7cc8/font/fontawesome-webfont.eot');
  src: url('/assets/thirdParty/FortAwesome-Font-Awesome-12c7cc8/font/fontawesome-webfont.eot?#iefix') format('embedded-opentype'),
    url('/assets/thirdParty/FortAwesome-Font-Awesome-12c7cc8/font/fontawesome-webfont.woff') format('woff'),
    url('/assets/thirdParty/FortAwesome-Font-Awesome-12c7cc8/font/fontawesome-webfont.ttf') format('truetype');
  font-weight: normal;
  font-style: normal;
}
/*  Font Awesome styles
    ------------------------------------------------------- */
/* includes sprites.less reset */
[class^="icon-"],
[class*=" icon-"] {
  font-family: FontAwesome;
  font-weight: normal;
  font-style: normal;
  text-decoration: inherit;
  display: inline;
  width: auto;
  height: auto;
  line-height: normal;
  vertical-align: baseline;
  background-image: none !important;
  background-position: 0% 0%;
  background-repeat: repeat;
}
[class^="icon-"]:before,
[class*=" icon-"]:before {
  text-decoration: inherit;
  display: inline-block;
  speak: none;
}
/* makes sure icons active on rollover in links */
a [class^="icon-"],
a [class*=" icon-"] {
  display: inline-block;
}
/* makes the font 33% larger relative to the icon container */
.icon-large:before {
  vertical-align: -10%;
  font-size: 1.3333333333333333em;
}
.btn [class^="icon-"],
.nav [class^="icon-"],
.btn [class*=" icon-"],
.nav [class*=" icon-"] {
  display: inline;
  /* keeps button heights with and without icons the same */

  line-height: .6em;
}
.btn [class^="icon-"].icon-spin,
.nav [class^="icon-"].icon-spin,
.btn [class*=" icon-"].icon-spin,
.nav [class*=" icon-"].icon-spin {
  display: inline-block;
}
li [class^="icon-"],
li [class*=" icon-"] {
  display: inline-block;
  width: 1.25em;
  text-align: center;
}
li [class^="icon-"].icon-large,
li [class*=" icon-"].icon-large {
  /* increased font size for icon-large */

  width: 1.5625em;
}
ul.icons {
  list-style-type: none;
  text-indent: -0.75em;
}
ul.icons li [class^="icon-"],
ul.icons li [class*=" icon-"] {
  width: .75em;
}
.icon-muted {
  color: #eeeeee;
}
.icon-border {
  border: solid 1px #eeeeee;
  padding: .2em .25em .15em;
  -webkit-border-radius: 3px;
  -moz-border-radius: 3px;
  border-radius: 3px;
}
.icon-2x {
  font-size: 2em;
}
.icon-2x.icon-border {
  border-width: 2px;
  -webkit-border-radius: 4px;
  -moz-border-radius: 4px;
  border-radius: 4px;
}
.icon-3x {
  font-size: 3em;
}
.icon-3x.icon-border {
  border-width: 3px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
}
.icon-4x {
  font-size: 4em;
}
.icon-4x.icon-border {
  border-width: 4px;
  -webkit-border-radius: 6px;
  -moz-border-radius: 6px;
  border-radius: 6px;
}
.pull-right {
  float: right;
}
.pull-left {
  float: left;
}
[class^="icon-"].pull-left,
[class*=" icon-"].pull-left {
  margin-right: .35em;
}
[class^="icon-"].pull-right,
[class*=" icon-"].pull-right {
  margin-left: .35em;
}
.btn [class^="icon-"].pull-left.icon-2x,
.btn [class*=" icon-"].pull-left.icon-2x,
.btn [class^="icon-"].pull-right.icon-2x,
.btn [class*=" icon-"].pull-right.icon-2x {
  margin-top: .35em;
}
.btn [class^="icon-"].icon-spin.icon-large,
.btn [class*=" icon-"].icon-spin.icon-large {
  height: .75em;
}
.btn.btn-small [class^="icon-"].pull-left.icon-2x,
.btn.btn-small [class*=" icon-"].pull-left.icon-2x,
.btn.btn-small [class^="icon-"].pull-right.icon-2x,
.btn.btn-small [class*=" icon-"].pull-right.icon-2x {
  margin-top: .45em;
}
.btn.btn-large [class^="icon-"].pull-left.icon-2x,
.btn.btn-large [class*=" icon-"].pull-left.icon-2x,
.btn.btn-large [class^="icon-"].pull-right.icon-2x,
.btn.btn-large [class*=" icon-"].pull-right.icon-2x {
  margin-top: .2em;
}
.icon-spin {
  display: inline-block;
  -moz-animation: spin 2s infinite linear;
  -o-animation: spin 2s infinite linear;
  -webkit-animation: spin 2s infinite linear;
  animation: spin 2s infinite linear;
}
@-moz-keyframes spin {
  0% { -moz-transform: rotate(0deg); }
  100% { -moz-transform: rotate(359deg); }
}
@-webkit-keyframes spin {
  0% { -webkit-transform: rotate(0deg); }
  100% { -webkit-transform: rotate(359deg); }
}
@-o-keyframes spin {
  0% { -o-transform: rotate(0deg); }
  100% { -o-transform: rotate(359deg); }
}
@-ms-keyframes spin {
  0% { -ms-transform: rotate(0deg); }
  100% { -ms-transform: rotate(359deg); }
}
@keyframes spin {
  0% { transform: rotate(0deg); }
  100% { transform: rotate(359deg); }
}
/*  Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
    readers do not read off random characters that represent icons */
.icon-glass:before                { content: "\f000"; }
.icon-music:before                { content: "\f001"; }
.icon-search:before               { content: "\f002"; }
.icon-envelope:before             { content: "\f003"; }
.icon-heart:before                { content: "\f004"; }
.icon-star:before                 { content: "\f005"; }
.icon-star-empty:before           { content: "\f006"; }
.icon-user:before                 { content: "\f007"; }
.icon-film:before                 { content: "\f008"; }
.icon-th-large:before             { content: "\f009"; }
.icon-th:before                   { content: "\f00a"; }
.icon-th-list:before              { content: "\f00b"; }
.icon-ok:before                   { content: "\f00c"; }
.icon-remove:before               { content: "\f00d"; }
.icon-zoom-in:before              { content: "\f00e"; }

.icon-zoom-out:before             { content: "\f010"; }
.icon-off:before                  { content: "\f011"; }
.icon-signal:before               { content: "\f012"; }
.icon-cog:before                  { content: "\f013"; }
.icon-trash:before                { content: "\f014"; }
.icon-home:before                 { content: "\f015"; }
.icon-file:before                 { content: "\f016"; }
.icon-time:before                 { content: "\f017"; }
.icon-road:before                 { content: "\f018"; }
.icon-download-alt:before         { content: "\f019"; }
.icon-download:before             { content: "\f01a"; }
.icon-upload:before               { content: "\f01b"; }
.icon-inbox:before                { content: "\f01c"; }
.icon-play-circle:before          { content: "\f01d"; }
.icon-repeat:before               { content: "\f01e"; }

/* \f020 doesn't work in Safari. all shifted one down */
.icon-refresh:before              { content: "\f021"; }
.icon-list-alt:before             { content: "\f022"; }
.icon-lock:before                 { content: "\f023"; }
.icon-flag:before                 { content: "\f024"; }
.icon-headphones:before           { content: "\f025"; }
.icon-volume-off:before           { content: "\f026"; }
.icon-volume-down:before          { content: "\f027"; }
.icon-volume-up:before            { content: "\f028"; }
.icon-qrcode:before               { content: "\f029"; }
.icon-barcode:before              { content: "\f02a"; }
.icon-tag:before                  { content: "\f02b"; }
.icon-tags:before                 { content: "\f02c"; }
.icon-book:before                 { content: "\f02d"; }
.icon-bookmark:before             { content: "\f02e"; }
.icon-print:before                { content: "\f02f"; }

.icon-camera:before               { content: "\f030"; }
.icon-font:before                 { content: "\f031"; }
.icon-bold:before                 { content: "\f032"; }
.icon-italic:before               { content: "\f033"; }
.icon-text-height:before          { content: "\f034"; }
.icon-text-width:before           { content: "\f035"; }
.icon-align-left:before           { content: "\f036"; }
.icon-align-center:before         { content: "\f037"; }
.icon-align-right:before          { content: "\f038"; }
.icon-align-justify:before        { content: "\f039"; }
.icon-list:before                 { content: "\f03a"; }
.icon-indent-left:before          { content: "\f03b"; }
.icon-indent-right:before         { content: "\f03c"; }
.icon-facetime-video:before       { content: "\f03d"; }
.icon-picture:before              { content: "\f03e"; }

.icon-pencil:before               { content: "\f040"; }
.icon-map-marker:before           { content: "\f041"; }
.icon-adjust:before               { content: "\f042"; }
.icon-tint:before                 { content: "\f043"; }
.icon-edit:before                 { content: "\f044"; }
.icon-share:before                { content: "\f045"; }
.icon-check:before                { content: "\f046"; }
.icon-move:before                 { content: "\f047"; }
.icon-step-backward:before        { content: "\f048"; }
.icon-fast-backward:before        { content: "\f049"; }
.icon-backward:before             { content: "\f04a"; }
.icon-play:before                 { content: "\f04b"; }
.icon-pause:before                { content: "\f04c"; }
.icon-stop:before                 { content: "\f04d"; }
.icon-forward:before              { content: "\f04e"; }

.icon-fast-forward:before         { content: "\f050"; }
.icon-step-forward:before         { content: "\f051"; }
.icon-eject:before                { content: "\f052"; }
.icon-chevron-left:before         { content: "\f053"; }
.icon-chevron-right:before        { content: "\f054"; }
.icon-plus-sign:before            { content: "\f055"; }
.icon-minus-sign:before           { content: "\f056"; }
.icon-remove-sign:before          { content: "\f057"; }
.icon-ok-sign:before              { content: "\f058"; }
.icon-question-sign:before        { content: "\f059"; }
.icon-info-sign:before            { content: "\f05a"; }
.icon-screenshot:before           { content: "\f05b"; }
.icon-remove-circle:before        { content: "\f05c"; }
.icon-ok-circle:before            { content: "\f05d"; }
.icon-ban-circle:before           { content: "\f05e"; }

.icon-arrow-left:before           { content: "\f060"; }
.icon-arrow-right:before          { content: "\f061"; }
.icon-arrow-up:before             { content: "\f062"; }
.icon-arrow-down:before           { content: "\f063"; }
.icon-share-alt:before            { content: "\f064"; }
.icon-resize-full:before          { content: "\f065"; }
.icon-resize-small:before         { content: "\f066"; }
.icon-plus:before                 { content: "\f067"; }
.icon-minus:before                { content: "\f068"; }
.icon-asterisk:before             { content: "\f069"; }
.icon-exclamation-sign:before     { content: "\f06a"; }
.icon-gift:before                 { content: "\f06b"; }
.icon-leaf:before                 { content: "\f06c"; }
.icon-fire:before                 { content: "\f06d"; }
.icon-eye-open:before             { content: "\f06e"; }

.icon-eye-close:before            { content: "\f070"; }
.icon-warning-sign:before         { content: "\f071"; }
.icon-plane:before                { content: "\f072"; }
.icon-calendar:before             { content: "\f073"; }
.icon-random:before               { content: "\f074"; }
.icon-comment:before              { content: "\f075"; }
.icon-magnet:before               { content: "\f076"; }
.icon-chevron-up:before           { content: "\f077"; }
.icon-chevron-down:before         { content: "\f078"; }
.icon-retweet:before              { content: "\f079"; }
.icon-shopping-cart:before        { content: "\f07a"; }
.icon-folder-close:before         { content: "\f07b"; }
.icon-folder-open:before          { content: "\f07c"; }
.icon-resize-vertical:before      { content: "\f07d"; }
.icon-resize-horizontal:before    { content: "\f07e"; }

.icon-bar-chart:before            { content: "\f080"; }
.icon-twitter-sign:before         { content: "\f081"; }
.icon-facebook-sign:before        { content: "\f082"; }
.icon-camera-retro:before         { content: "\f083"; }
.icon-key:before                  { content: "\f084"; }
.icon-cogs:before                 { content: "\f085"; }
.icon-comments:before             { content: "\f086"; }
.icon-thumbs-up:before            { content: "\f087"; }
.icon-thumbs-down:before          { content: "\f088"; }
.icon-star-half:before            { content: "\f089"; }
.icon-heart-empty:before          { content: "\f08a"; }
.icon-signout:before              { content: "\f08b"; }
.icon-linkedin-sign:before        { content: "\f08c"; }
.icon-pushpin:before              { content: "\f08d"; }
.icon-external-link:before        { content: "\f08e"; }

.icon-signin:before               { content: "\f090"; }
.icon-trophy:before               { content: "\f091"; }
.icon-github-sign:before          { content: "\f092"; }
.icon-upload-alt:before           { content: "\f093"; }
.icon-lemon:before                { content: "\f094"; }
.icon-phone:before                { content: "\f095"; }
.icon-check-empty:before          { content: "\f096"; }
.icon-bookmark-empty:before       { content: "\f097"; }
.icon-phone-sign:before           { content: "\f098"; }
.icon-twitter:before              { content: "\f099"; }
.icon-facebook:before             { content: "\f09a"; }
.icon-github:before               { content: "\f09b"; }
.icon-unlock:before               { content: "\f09c"; }
.icon-credit-card:before          { content: "\f09d"; }
.icon-rss:before                  { content: "\f09e"; }

.icon-hdd:before                  { content: "\f0a0"; }
.icon-bullhorn:before             { content: "\f0a1"; }
.icon-bell:before                 { content: "\f0a2"; }
.icon-certificate:before          { content: "\f0a3"; }
.icon-hand-right:before           { content: "\f0a4"; }
.icon-hand-left:before            { content: "\f0a5"; }
.icon-hand-up:before              { content: "\f0a6"; }
.icon-hand-down:before            { content: "\f0a7"; }
.icon-circle-arrow-left:before    { content: "\f0a8"; }
.icon-circle-arrow-right:before   { content: "\f0a9"; }
.icon-circle-arrow-up:before      { content: "\f0aa"; }
.icon-circle-arrow-down:before    { content: "\f0ab"; }
.icon-globe:before                { content: "\f0ac"; }
.icon-wrench:before               { content: "\f0ad"; }
.icon-tasks:before                { content: "\f0ae"; }

.icon-filter:before               { content: "\f0b0"; }
.icon-briefcase:before            { content: "\f0b1"; }
.icon-fullscreen:before           { content: "\f0b2"; }

.icon-group:before                { content: "\f0c0"; }
.icon-link:before                 { content: "\f0c1"; }
.icon-cloud:before                { content: "\f0c2"; }
.icon-beaker:before               { content: "\f0c3"; }
.icon-cut:before                  { content: "\f0c4"; }
.icon-copy:before                 { content: "\f0c5"; }
.icon-paper-clip:before           { content: "\f0c6"; }
.icon-save:before                 { content: "\f0c7"; }
.icon-sign-blank:before           { content: "\f0c8"; }
.icon-reorder:before              { content: "\f0c9"; }
.icon-list-ul:before              { content: "\f0ca"; }
.icon-list-ol:before              { content: "\f0cb"; }
.icon-strikethrough:before        { content: "\f0cc"; }
.icon-underline:before            { content: "\f0cd"; }
.icon-table:before                { content: "\f0ce"; }

.icon-magic:before                { content: "\f0d0"; }
.icon-truck:before                { content: "\f0d1"; }
.icon-pinterest:before            { content: "\f0d2"; }
.icon-pinterest-sign:before       { content: "\f0d3"; }
.icon-google-plus-sign:before     { content: "\f0d4"; }
.icon-google-plus:before          { content: "\f0d5"; }
.icon-money:before                { content: "\f0d6"; }
.icon-caret-down:before           { content: "\f0d7"; }
.icon-caret-up:before             { content: "\f0d8"; }
.icon-caret-left:before           { content: "\f0d9"; }
.icon-caret-right:before          { content: "\f0da"; }
.icon-columns:before              { content: "\f0db"; }
.icon-sort:before                 { content: "\f0dc"; }
.icon-sort-down:before            { content: "\f0dd"; }
.icon-sort-up:before              { content: "\f0de"; }

.icon-envelope-alt:before         { content: "\f0e0"; }
.icon-linkedin:before             { content: "\f0e1"; }
.icon-undo:before                 { content: "\f0e2"; }
.icon-legal:before                { content: "\f0e3"; }
.icon-dashboard:before            { content: "\f0e4"; }
.icon-comment-alt:before          { content: "\f0e5"; }
.icon-comments-alt:before         { content: "\f0e6"; }
.icon-bolt:before                 { content: "\f0e7"; }
.icon-sitemap:before              { content: "\f0e8"; }
.icon-umbrella:before             { content: "\f0e9"; }
.icon-paste:before                { content: "\f0ea"; }
.icon-lightbulb:before            { content: "\f0eb"; }
.icon-exchange:before             { content: "\f0ec"; }
.icon-cloud-download:before       { content: "\f0ed"; }
.icon-cloud-upload:before         { content: "\f0ee"; }

.icon-user-md:before              { content: "\f0f0"; }
.icon-stethoscope:before          { content: "\f0f1"; }
.icon-suitcase:before             { content: "\f0f2"; }
.icon-bell-alt:before             { content: "\f0f3"; }
.icon-coffee:before               { content: "\f0f4"; }
.icon-food:before                 { content: "\f0f5"; }
.icon-file-alt:before             { content: "\f0f6"; }
.icon-building:before             { content: "\f0f7"; }
.icon-hospital:before             { content: "\f0f8"; }
.icon-ambulance:before            { content: "\f0f9"; }
.icon-medkit:before               { content: "\f0fa"; }
.icon-fighter-jet:before          { content: "\f0fb"; }
.icon-beer:before                 { content: "\f0fc"; }
.icon-h-sign:before               { content: "\f0fd"; }
.icon-plus-sign-alt:before        { content: "\f0fe"; }

.icon-double-angle-left:before    { content: "\f100"; }
.icon-double-angle-right:before   { content: "\f101"; }
.icon-double-angle-up:before      { content: "\f102"; }
.icon-double-angle-down:before    { content: "\f103"; }
.icon-angle-left:before           { content: "\f104"; }
.icon-angle-right:before          { content: "\f105"; }
.icon-angle-up:before             { content: "\f106"; }
.icon-angle-down:before           { content: "\f107"; }
.icon-desktop:before              { content: "\f108"; }
.icon-laptop:before               { content: "\f109"; }
.icon-tablet:before               { content: "\f10a"; }
.icon-mobile-phone:before         { content: "\f10b"; }
.icon-circle-blank:before         { content: "\f10c"; }
.icon-quote-left:before           { content: "\f10d"; }
.icon-quote-right:before          { content: "\f10e"; }

.icon-spinner:before              { content: "\f110"; }
.icon-circle:before               { content: "\f111"; }
.icon-reply:before                { content: "\f112"; }
.icon-github-alt:before           { content: "\f113"; }
.icon-folder-close-alt:before     { content: "\f114"; }
.icon-folder-open-alt:before      { content: "\f115"; }
/*!
 *
 *  Project:  GLYPHICONS
 *  Author:   Jan Kovarik - www.glyphicons.com
 *  Twitter:  @jankovarik
 *
 */
html,
html .halflings {
  -webkit-font-smoothing: antialiased !important;
}
@font-face {
  font-family: 'Glyphicons';
  src: url('/assets/thirdParty/glyphicons_pro/glyphicons/web/html_css/html_css/fonts/glyphicons-regular.eot');
  src: url('/assets/thirdParty/glyphicons_pro/glyphicons/web/html_css/fonts/glyphicons-regular.eot?#iefix') format('embedded-opentype'), url('/assets/thirdParty/glyphicons_pro/glyphicons/web/html_css/fonts/glyphicons-regular.woff') format('woff'), url('/assets/thirdParty/glyphicons_pro/glyphicons/web/html_css/fonts/glyphicons-regular.ttf') format('truetype'), url('/assets/thirdParty/glyphicons_pro/glyphicons/web/html_css/fonts/glyphicons-regular.svg#glyphicons_halflingsregular') format('svg');
  font-weight: normal;
  font-style: normal;
}

/*@font-face {
  font-family: 'FontAwesome';
  src: url('../font/fontawesome-webfont.eot');
  src: url('../font/fontawesome-webfont.eot') format('embedded-opentype'), url('../font/fontawesome-webfont.woff') format('woff'), url('../font/fontawesome-webfont.ttf') format('truetype'), url('../font/fontawesome-webfont.svgz#FontAwesomeRegular') format('svg'), url('../font/fontawesome-webfont.svg#FontAwesomeRegular') format('svg');
  font-weight: normal;
  font-style: normal;
}
*/
.glyphicons {
  display: inline-block;
  position: relative;
  padding: 5px 0 5px 35px;
  color: #1d1d1b;
  text-decoration: none;
  *display: inline;
  *zoom: 1;
}
.glyphicons i::before {
  position: absolute;
  left: 0;
  top: 0;
  font: 24px/1em 'Glyphicons';
  font-style: normal;
  color: #1d1d1b;
}
.glyphicons.white i:before {
  color: #fff;
}

.glyphicons.glass i:before, .glyph-glass:before  {
  content: "\e001";
}
.glyphicons.leaf i:before {
  content: "\e002";
}
.glyphicons.dog i:before {
  content: "\e003";
}
.glyphicons.user i:before,.glyph-user:before {
  content: "\e004";
}
.glyphicons.girl i:before {
  content: "\e005";
}
.glyphicons.car i:before {
  content: "\e006";
}
.glyphicons.user_add i:before,.glyph-user_add:before {
  content: "\e007";
}
.glyphicons.user_remove i:before,.glyph-user_remove:before {
  content: "\e008";
}
.glyphicons.film i:before,.glyph-film:before {
  content: "\e009";
}
.glyphicons.magic i:before, .glyph-magic:before  {
  content: "\e010";
}
.glyphicons.envelope i:before {
  content: "\2709";
}
.glyphicons.camera i:before,.glyph-camera:before {
  content: "\e012";
}
.glyphicons.heart i:before,.glyph-heart:before  {
  content: "\e013";
}
.glyphicons.beach_umbrella i:before {
  content: "\e014";
}
.glyphicons.train i:before {
  content: "\e015";
}
.glyphicons.print i:before {
  content: "\e016";
}
.glyphicons.bin i:before {
  content: "\e017";
}
.glyphicons.music i:before,.glyph-music:before{
  content: "\e018";
}
.glyphicons.note i:before,.glyph-note:before {
  content: "\e019";
}
.glyphicons.heart_empty i:before,.glyph-heart_empty:before{
  content: "\e020";
}
.glyphicons.home i:before,.glyph-home:before{
  content: "\e021";
}
.glyphicons.snowflake i:before {
  content: "\2744";
}
.glyphicons.fire i:before {
  content: "\e023";
}
.glyphicons.magnet i:before {
  content: "\e024";
}
.glyphicons.parents i:before,.glyph-parents:before {
  content: "\e025";
}
.glyphicons.binoculars i:before {
  content: "\e026";
}
.glyphicons.road i:before,.glyph-road:before {
  content: "\e027";
}
.glyphicons.search i:before,.glyph-search:before{
  content: "\e028";
}
.glyphicons.cars i:before {
  content: "\e029";
}
.glyphicons.notes_2 i:before {
  content: "\e030";
}
.glyphicons.pencil i:before,.glyph-pencil:before {
  content: "\270F";
}
.glyphicons.bus i:before {
  content: "\e032";
}
.glyphicons.wifi_alt i:before {
  content: "\e033";
}
.glyphicons.luggage i:before,.glyph-luggage:before {
  content: "\e034";
}
.glyphicons.old_man i:before {
  content: "\e035";
}
.glyphicons.woman i:before,.glyph-woman:before {
  content: "\e036";
}
.glyphicons.file i:before,.glyph-file:before {
  content: "\e037";
}
.glyphicons.coins i:before, .glyph-coins:before {
  content: "\e038";
}
.glyphicons.airplane i:before {
  content: "\2708";
}
.glyphicons.notes i:before {
  content: "\e040";
}
.glyphicons.stats i:before, .glyph-stats:before {
  content: "\e041";
}
.glyphicons.charts i:before, .glyph-charts:before {
  content: "\e042";
}
.glyphicons.pie_chart i:before,.glyph-pie_chart:before {
  content: "\e043";
}
.glyphicons.group i:before,.glyph-group:before {
  content: "\e044";
}
.glyphicons.keys i:before {
  content: "\e045";
}
.glyphicons.calendar i:before {
  content: "\e046";
}
.glyphicons.router i:before {
  content: "\e047";
}
.glyphicons.camera_small i:before {
  content: "\e048";
}
.glyphicons.dislikes i:before {
  content: "\e049";
}
.glyphicons.star i:before {
  content: "\e050";
}
.glyphicons.link i:before {
  content: "\e051";
}
.glyphicons.eye_open i:before,.glyph-eye_open:before {
  content: "\e052";
}
.glyphicons.eye_close i:before,.glyph-eye_close:before {
  content: "\e053";
}
.glyphicons.alarm i:before {
  content: "\e054";
}
.glyphicons.clock i:before,.glyph-clock:before {
  content: "\e055";
}
.glyphicons.stopwatch i:before {
  content: "\e056";
}
.glyphicons.projector i:before {
  content: "\e057";
}
.glyphicons.history i:before,.glyph-history:before {
  content: "\e058";
}
.glyphicons.truck i:before {
  content: "\e059";
}
.glyphicons.cargo i:before {
  content: "\e060";
}
.glyphicons.compass i:before {
  content: "\e061";
}
.glyphicons.keynote i:before {
  content: "\e062";
}
.glyphicons.paperclip i:before,.glyph-paperclip:before {
  content: "\e063";
}
.glyphicons.power i:before,.glyph-power:before {
  content: "\e064";
}
.glyphicons.lightbulb i:before,.glyph-lightbulb:before {
  content: "\e065";
}
.glyphicons.tag i:before,.glyph-tag:before {
  content: "\e066";
}
.glyphicons.tags i:before,.glyph-tags:before {
  content: "\e067";
}
.glyphicons.cleaning i:before {
  content: "\e068";
}
.glyphicons.ruller i:before {
  content: "\e069";
}
.glyphicons.gift i:before,.glyph-gift:before {
  content: "\e070";
}
.glyphicons.umbrella i:before {
  content: "\2602";
}
.glyphicons.book i:before,.glyph-book:before {
  content: "\e072";
}
.glyphicons.bookmark i:before,.glyph-bookmark:before {
  content: "\e073";
}
.glyphicons.wifi i:before {
  content: "\e074";
}
.glyphicons.cup i:before, .glyph-cup:before {
  content: "\e075";
}
.glyphicons.stroller i:before {
  content: "\e076";
}
.glyphicons.headphones i:before,.glyph-headphones:before {
  content: "\e077";
}
.glyphicons.headset i:before {
  content: "\e078";
}
.glyphicons.warning_sign i:before,.glyph-warning_sign:before {
  content: "\e079";
}
.glyphicons.signal i:before {
  content: "\e080";
}
.glyphicons.retweet i:before {
  content: "\e081";
}
.glyphicons.refresh i:before,.glyph-refresh:before  {
  content: "\e082";
}
.glyphicons.roundabout i:before,.glyph-roundabout:before {
  content: "\e083";
}
.glyphicons.random i:before,.glyph-random:before {
  content: "\e084";
}
.glyphicons.heat i:before {
  content: "\e085";
}
.glyphicons.repeat i:before,.glyph-repeat:before {
  content: "\e086";
}
.glyphicons.display i:before {
  content: "\e087";
}
.glyphicons.log_book i:before {
  content: "\e088";
}
.glyphicons.adress_book i:before {
  content: "\e089";
}
.glyphicons.building i:before {
  content: "\e090";
}
.glyphicons.eyedropper i:before {
  content: "\e091";
}
.glyphicons.adjust i:before {
  content: "\e092";
}
.glyphicons.tint i:before {
  content: "\e093";
}
.glyphicons.crop i:before {
  content: "\e094";
}
.glyphicons.vector_path_square i:before {
  content: "\e095";
}
.glyphicons.vector_path_circle i:before {
  content: "\e096";
}
.glyphicons.vector_path_polygon i:before {
  content: "\e097";
}
.glyphicons.vector_path_line i:before {
  content: "\e098";
}
.glyphicons.vector_path_curve i:before {
  content: "\e099";
}
.glyphicons.vector_path_all i:before {
  content: "\e100";
}
.glyphicons.font i:before,.glyph-font:before  {
  content: "\e101";
}
.glyphicons.italic i:before,.glyph-italic:before {
  content: "\e102";
}
.glyphicons.bold i:before,.glyph-bold:before {
  content: "\e103";
}
.glyphicons.text_underline i:before,.glyph-text_underline:before  {
  content: "\e104";
}
.glyphicons.text_strike i:before {
  content: "\e105";
}
.glyphicons.text_height i:before {
  content: "\e106";
}
.glyphicons.text_width i:before {
  content: "\e107";
}
.glyphicons.text_resize i:before,.glyph-text_resize:before {
  content: "\e108";
}
.glyphicons.left_indent i:before {
  content: "\e109";
}
.glyphicons.right_indent i:before {
  content: "\e110";
}
.glyphicons.align_left i:before {
  content: "\e111";
}
.glyphicons.align_center i:before {
  content: "\e112";
}
.glyphicons.align_right i:before {
  content: "\e113";
}
.glyphicons.justify i:before {
  content: "\e114";
}
.glyphicons.list i:before,.glyph-list:before {
  content: "\e115";
}
.glyphicons.text_smaller i:before,.glyph-text_smaller:before {
  content: "\e116";
}
.glyphicons.text_bigger i:before,.glyph-text_bigger:before {
  content: "\e117";
}
.glyphicons.embed i:before,.glyph-embed:before {
  content: "\e118";
}
.glyphicons.embed_close i:before,.glyph-embed_close:before {
  content: "\e119";
}
.glyphicons.table i:before,.glyph-table:before{
  content: "\e120";
}
.glyphicons.message_full i:before,.glyph-message_full:before {
  content: "\e121";
}
.glyphicons.message_empty i:before,.glyph-message_empty:before {
  content: "\e122";
}
.glyphicons.message_in i:before,.glyph-message_in:before {
  content: "\e123";
}
.glyphicons.message_out i:before,.glyph-message_out:before {
  content: "\e124";
}
.glyphicons.message_plus i:before,.glyph-message_plus:before {
  content: "\e125";
}
.glyphicons.message_minus i:before,.glyph-message_minus:before {
  content: "\e126";
}
.glyphicons.message_ban i:before,.glyph-message_ban:before{
  content: "\e127";
}
.glyphicons.message_flag i:before,.glyph-message_flag:before {
  content: "\e128";
}
.glyphicons.message_lock i:before,.glyph-message_lock:before {
  content: "\e129";
}
.glyphicons.message_new i:before,.glyph-message_new:before  {
  content: "\e130";
}
.glyphicons.inbox i:before,.glyph-inbox:before {
  content: "\e131";
}
.glyphicons.inbox_plus i:before,.glyph-inbox_plus:before {
  content: "\e132";
}
.glyphicons.inbox_minus i:before,.glyph-inbox_minus:before {
  content: "\e133";
}
.glyphicons.inbox_lock i:before,.glyph-inbox_lock:before{
  content: "\e134";
}
.glyphicons.inbox_in i:before,.glyph-inbox_in:before {
  content: "\e135";
}
.glyphicons.inbox_out i:before,.glyph-inbox_out:before {
  content: "\e136";
}
.glyphicons.cogwheel i:before,.glyph-cogwheel:before {
  content: "\e137";
}
.glyphicons.cogwheels i:before,.glyph-cogwheels:before {
  content: "\e138";
}
.glyphicons.picture i:before,.glyph-picture:before {
  content: "\e139";
}
.glyphicons.adjust_alt i:before {
  content: "\e140";
}
.glyphicons.database_lock i:before {
  content: "\e141";
}
.glyphicons.database_plus i:before {
  content: "\e142";
}
.glyphicons.database_minus i:before {
  content: "\e143";
}
.glyphicons.database_ban i:before {
  content: "\e144";
}
.glyphicons.folder_open i:before,.glyph-folder_open:before {
  content: "\e145";
}
.glyphicons.folder_plus i:before,.glyph-folder_plus:before {
  content: "\e146";
}
.glyphicons.folder_minus i:before,.glyph-folder_minus:before {
  content: "\e147";
}
.glyphicons.folder_lock i:before {
  content: "\e148";
}
.glyphicons.folder_flag i:before {
  content: "\e149";
}
.glyphicons.folder_new i:before,.glyph-folder_new:before  {
  content: "\e150";
}
.glyphicons.edit i:before,.glyph-edit:before{
  content: "\e151";
}
.glyphicons.new_window i:before,.glyph-new_window:before {
  content: "\e152";
}
.glyphicons.check i:before {
  content: "\e153";
}
.glyphicons.unchecked i:before {
  content: "\e154";
}
.glyphicons.more_windows i:before,.glyph-more_windows:before {
  content: "\e155";
}
.glyphicons.show_big_thumbnails i:before {
  content: "\e156";
}
.glyphicons.show_thumbnails i:before,.glyph-show_thumbnails:before {
  content: "\e157";
}
.glyphicons.show_thumbnails_with_lines i:before {
  content: "\e158";
}
.glyphicons.show_lines i:before {
  content: "\e159";
}
.glyphicons.playlist i:before,.glyph-playlist:before {
  content: "\e160";
}
.glyphicons.imac i:before {
  content: "\e161";
}
.glyphicons.macbook i:before {
  content: "\e162";
}
.glyphicons.ipad i:before {
  content: "\e163";
}
.glyphicons.iphone i:before {
  content: "\e164";
}
.glyphicons.iphone_transfer i:before {
  content: "\e165";
}
.glyphicons.iphone_exchange i:before {
  content: "\e166";
}
.glyphicons.ipod i:before {
  content: "\e167";
}
.glyphicons.ipod_shuffle i:before {
  content: "\e168";
}
.glyphicons.ear_plugs i:before {
  content: "\e169";
}
.glyphicons.phone i:before {
  content: "\e170";
}
.glyphicons.step_backward i:before,.glyph-step_backward:before {
  content: "\e171";
}
.glyphicons.fast_backward i:before,.glyph-fast_backward:before {
  content: "\e172";
}
.glyphicons.rewind i:before,.glyph-rewind:before {
  content: "\e173";
}
.glyphicons.play i:before,.glyph-play:before, .galleria-play {
  content: "\e174";
}
.glyphicons.pause i:before,.glyph-pause:before {
  content: "\e175";
}
.glyphicons.stop i:before,.glyph-stop:before {
  content: "\e176";
}
.glyphicons.forward i:before,.glyph-forward:before {
  content: "\e177";
}
.glyphicons.fast_forward i:before,.glyph-fast_forward:before  {
  content: "\e178";
}
.glyphicons.step_forward i:before,.glyph-step_forward:before {
  content: "\e179";
}
.glyphicons.eject i:before {
  content: "\e180";
}
.glyphicons.facetime_video i:before {
  content: "\e181";
}
.glyphicons.download_alt i:before {
  content: "\e182";
}
.glyphicons.mute i:before,.glyph-mute:before {
  content: "\e183";
}
.glyphicons.volume_down i:before,.glyph-volume_down:before {
  content: "\e184";
}
.glyphicons.volume_up i:before,.glyph-volume_up:before {
  content: "\e185";
}
.glyphicons.screenshot i:before {
  content: "\e186";
}
.glyphicons.move i:before,.glyph-move:before {
  content: "\e187";
}
.glyphicons.more i:before,.glyph-more:before {
  content: "\e188";
}
.glyphicons.brightness_reduce i:before {
  content: "\e189";
}
.glyphicons.brightness_increase i:before {
  content: "\e190";
}
.glyphicons.circle_plus i:before,.glyph-circle_plus:before {
  content: "\e191";
}
.glyphicons.circle_minus i:before,.glyph-circle_minus:before  {
  content: "\e192";
}
.glyphicons.circle_remove i:before,.glyph-circle_remove:before  {
  content: "\e193";
}
.glyphicons.circle_ok i:before,.glyph-circle_ok:before {
  content: "\e194";
}
.glyphicons.circle_question_mark i:before,.glyph-circle_question_mark:before  {
  content: "\e195";
}
.glyphicons.circle_info i:before,.glyph-circle_info:before  {
  content: "\e196";
}
.glyphicons.circle_exclamation_mark i:before,.glyph-exclamation_mark:before  {
  content: "\e197";
}
.glyphicons.remove i:before,.glyph-remove:before {
  content: "\e198";
}
.glyphicons.ok i:before,.glyph-ok:before {
  content: "\e199";
}
.glyphicons.ban i:before,.glyph-ban:before {
  content: "\e200";
}
.glyphicons.download i:before,.glyph-download:before {
  content: "\e201";
}
.glyphicons.upload i:before,.glyph-upload:before {
  content: "\e202";
}
.glyphicons.shopping_cart i:before,.glyph-shopping_cart:before {
  content: "\e203";
}
.glyphicons.lock i:before,.glyph-lock:before {
  content: "\e204";
}
.glyphicons.unlock i:before,.glyph-unlock:before {
  content: "\e205";
}
.glyphicons.electricity i:before {
  content: "\e206";
}
.glyphicons.ok_2 i:before,.glyph-ok_2:before {
  content: "\e207";
}
.glyphicons.remove_2 i:before,.glyph-remove_2:before{
  content: "\e208";
}
.glyphicons.cart_out i:before {
  content: "\e209";
}
.glyphicons.cart_in i:before {
  content: "\e210";
}
.glyphicons.left_arrow i:before,.glyph-left_arrow:before {
  content: "\e211";
}
.glyphicons.right_arrow i:before,.glyph-right_arrow:before {
  content: "\e212";
}
.glyphicons.down_arrow i:before,.glyph-down_arrow:before {
  content: "\e213";
}
.glyphicons.up_arrow i:before,.glyph-up_arrow:before {
  content: "\e214";
}
.glyphicons.resize_small i:before,.glyph-resize_small:before {
  content: "\e215";
}
.glyphicons.resize_full i:before,.glyph-resize_full:before {
  content: "\e216";
}
.glyphicons.circle_arrow_left i:before,.glyph-circle_arrow_left:before {
  content: "\e217";
}
.glyphicons.circle_arrow_right i:before,.glyph-circle_arrow_right:before {
  content: "\e218";
}
.glyphicons.circle_arrow_top i:before,.glyph-circle_arrow_top:before {
  content: "\e219";
}
.glyphicons.circle_arrow_down i:before,.glyph-circle_arrow_down:before {
  content: "\e220";
}
.glyphicons.play_button i:before, .glyph-play_button:before{
  content: "\e221";
}
.glyphicons.unshare i:before,.glyph-unshare:before {
  content: "\e222";
}
.glyphicons.share i:before,.glyph-share:before {
  content: "\e223";
}
.glyphicons.chevron-right i:before,.glyph-chevron-right:before {
  content: "\e224";
}
.glyphicons.chevron-left i:before,.glyph-chevron-left:before {
  content: "\e225";
}
.glyphicons.bluetooth i:before {
  content: "\e226";
}
.glyphicons.euro i:before {
  content: "\20AC";
}
.glyphicons.usd i:before {
  content: "\e228";
}
.glyphicons.gbp i:before {
  content: "\e229";
}
.glyphicons.retweet_2 i:before {
  content: "\e230";
}
.glyphicons.moon i:before {
  content: "\e231";
}
.glyphicons.sun i:before {
  content: "\2609";
}
.glyphicons.cloud i:before {
  content: "\2601";
}
.glyphicons.direction i:before {
  content: "\e234";
}
.glyphicons.brush i:before {
  content: "\e235";
}
.glyphicons.pen i:before {
  content: "\e236";
}
.glyphicons.zoom_in i:before,.glyph-zoom_in:before {
  content: "\e237";
}
.glyphicons.zoom_out i:before,.glyph-zoom_out:before {
  content: "\e238";
}
.glyphicons.pin i:before {
  content: "\e239";
}
.glyphicons.albums i:before {
  content: "\e240";
}
.glyphicons.rotation_lock i:before {
  content: "\e241";
}
.glyphicons.flash i:before {
  content: "\e242";
}
.glyphicons.google_maps i:before {
  content: "\e243";
}
.glyphicons.anchor i:before {
  content: "\2693";
}
.glyphicons.conversation i:before,.glyph-conversation:before {
  content: "\e245";
}
.glyphicons.chat i:before {
  content: "\e246";
}
.glyphicons.male i:before {
  content: "\e247";
}
.glyphicons.female i:before {
  content: "\e248";
}
.glyphicons.asterisk i:before,.glyph-asterisk:before {
  content: "\002A";
}
.glyphicons.divide i:before {
  content: "\00F7";
}
.glyphicons.snorkel_diving i:before {
  content: "\e251";
}
.glyphicons.scuba_diving i:before {
  content: "\e252";
}
.glyphicons.oxygen_bottle i:before {
  content: "\e253";
}
.glyphicons.fins i:before {
  content: "\e254";
}
.glyphicons.fishes i:before {
  content: "\e255";
}
.glyphicons.boat i:before {
  content: "\e256";
}
.glyphicons.delete i:before,.glyph-delete:before{
  content: "\e257";
}
.glyphicons.sheriffs_star i:before {
  content: "\e258";
}
.glyphicons.qrcode i:before {
  content: "\e259";
}
.glyphicons.barcode i:before {
  content: "\e260";
}
.glyphicons.pool i:before {
  content: "\e261";
}
.glyphicons.buoy i:before {
  content: "\e262";
}
.glyphicons.spade i:before {
  content: "\e263";
}
.glyphicons.bank i:before,.glyph-bank:before{
  content: "\e264";
}
.glyphicons.vcard i:before {
  content: "\e265";
}
.glyphicons.electrical_plug i:before {
  content: "\e266";
}
.glyphicons.flag i:before,.glyph-flag:before {
  content: "\e267";
}
.glyphicons.credit_card i:before,.glyph-credit_card:before {
  content: "\e268";
}
.glyphicons.keyboard-wireless i:before {
  content: "\e269";
}
.glyphicons.keyboard-wired i:before {
  content: "\e270";
}
.glyphicons.shield i:before {
  content: "\e271";
}
.glyphicons.ring i:before {
  content: "\02DA";
}
.glyphicons.cake i:before {
  content: "\e273";
}
.glyphicons.drink i:before {
  content: "\e274";
}
.glyphicons.beer i:before {
  content: "\e275";
}
.glyphicons.fast_food i:before {
  content: "\e276";
}
.glyphicons.cutlery i:before {
  content: "\e277";
}
.glyphicons.pizza i:before {
  content: "\e278";
}
.glyphicons.birthday_cake i:before {
  content: "\e279";
}
.glyphicons.tablet i:before {
  content: "\e280";
}
.glyphicons.settings i:before {
  content: "\e281";
}
.glyphicons.bullets i:before {
  content: "\e282";
}
.glyphicons.cardio i:before {
  content: "\e283";
}
.glyphicons.t-shirt i:before {
  content: "\e284";
}
.glyphicons.pants i:before {
  content: "\e285";
}
.glyphicons.sweater i:before {
  content: "\e286";
}
.glyphicons.fabric i:before {
  content: "\e287";
}
.glyphicons.leather i:before {
  content: "\e288";
}
.glyphicons.scissors i:before {
  content: "\e289";
}
.glyphicons.bomb i:before {
  content: "\e290";
}
.glyphicons.skull i:before {
  content: "\e291";
}
.glyphicons.celebration i:before {
  content: "\e292";
}
.glyphicons.tea_kettle i:before {
  content: "\e293";
}
.glyphicons.french_press i:before {
  content: "\e294";
}
.glyphicons.coffe_cup i:before {
  content: "\e295";
}
.glyphicons.pot i:before {
  content: "\e296";
}
.glyphicons.grater i:before {
  content: "\e297";
}
.glyphicons.kettle i:before {
  content: "\e298";
}
.glyphicons.hospital i:before {
  content: "\e299";
}
.glyphicons.hospital_h i:before {
  content: "\e300";
}
.glyphicons.microphone i:before, .glyph-microphone:before{
  content: "\e301";
}
.glyphicons.webcam i:before {
  content: "\e302";
}
.glyphicons.temple_christianity_church i:before {
  content: "\e303";
}
.glyphicons.temple_islam i:before {
  content: "\e304";
}
.glyphicons.temple_hindu i:before {
  content: "\e305";
}
.glyphicons.temple_buddhist i:before {
  content: "\e306";
}
.glyphicons.bicycle i:before {
  content: "\e307";
}
.glyphicons.life_preserver i:before {
  content: "\e308";
}
.glyphicons.share_alt i:before,.glyph-share_alt:before  {
  content: "\e309";
}
.glyphicons.comments i:before,.glyph-comments:before {
  content: "\e310";
}
.glyphicons.flower i:before {
  content: "\2698";
}
.glyphicons.baseball i:before {
  content: "\e312";
}
.glyphicons.rugby i:before {
  content: "\e313";
}
.glyphicons.ax i:before {
  content: "\e314";
}
.glyphicons.table_tennis i:before {
  content: "\e315";
}
.glyphicons.bowling i:before {
  content: "\e316";
}
.glyphicons.tree_conifer i:before,.glyph-tree_conifer:before{
  content: "\e317";
}
.glyphicons.tree_deciduous i:before,.glyph-tree_deciduous:before {
  content: "\e318";
}
.glyphicons.more_items i:before {
  content: "\e319";
}
.glyphicons.sort i:before,.glyph-sort:before {
  content: "\e320";
}
.glyphicons.filter i:before,.glyph-filter:before {
  content: "\e321";
}
.glyphicons.gamepad i:before {
  content: "\e322";
}
.glyphicons.playing_dices i:before {
  content: "\e323";
}
.glyphicons.calculator i:before {
  content: "\e324";
}
.glyphicons.tie i:before {
  content: "\e325";
}
.glyphicons.wallet i:before {
  content: "\e326";
}
.glyphicons.piano i:before {
  content: "\e327";
}
.glyphicons.sampler i:before {
  content: "\e328";
}
.glyphicons.podium i:before {
  content: "\e329";
}
.glyphicons.soccer_ball i:before {
  content: "\e330";
}
.glyphicons.blog i:before {
  content: "\e331";
}
.glyphicons.dashboard i:before {
  content: "\e332";
}
.glyphicons.certificate i:before,.glyph-certificate:before  {
  content: "\e333";
}
.glyphicons.bell i:before {
  content: "\e334";
}
.glyphicons.candle i:before {
  content: "\e335";
}
.glyphicons.pushpin i:before {
  content: "\e336";
}
.glyphicons.iphone_shake i:before {
  content: "\e337";
}
.glyphicons.pin_flag i:before {
  content: "\e338";
}
.glyphicons.turtle i:before {
  content: "\e339";
}
.glyphicons.rabbit i:before {
  content: "\e340";
}
.glyphicons.globe i:before,.glyph-globe:before {
  content: "\e341";
}
.glyphicons.briefcase i:before {
  content: "\e342";
}
.glyphicons.hdd i:before {
  content: "\e343";
}
.glyphicons.thumbs_up i:before {
  content: "\e344";
}
.glyphicons.thumbs_down i:before {
  content: "\e345";
}
.glyphicons.hand_right i:before {
  content: "\e346";
}
.glyphicons.hand_left i:before {
  content: "\e347";
}
.glyphicons.hand_up i:before {
  content: "\e348";
}
.glyphicons.hand_down i:before {
  content: "\e349";
}
.glyphicons.fullscreen i:before,.glyph-fullscreen:before {
  content: "\e350";
}
.glyphicons.shopping_bag i:before {
  content: "\e351";
}
.glyphicons.book_open i:before {
  content: "\e352";
}
.glyphicons.nameplate i:before {
  content: "\e353";
}
.glyphicons.nameplate_alt i:before {
  content: "\e354";
}
.glyphicons.vases i:before {
  content: "\e355";
}
.glyphicons.bullhorn i:before {
  content: "\e356";
}
.glyphicons.dumbbell i:before {
  content: "\e357";
}
.glyphicons.suitcase i:before {
  content: "\e358";
}
.glyphicons.file_import i:before {
  content: "\e359";
}
.glyphicons.file_export i:before {
  content: "\e360";
}
.glyphicons.bug i:before {
  content: "\e361";
}
.glyphicons.crown i:before {
  content: "\e362";
}
.glyphicons.smoking i:before {
  content: "\e363";
}
.glyphicons.cloud-upload i:before {
  content: "\e364";
}
.glyphicons.cloud-download i:before {
  content: "\e365";
}
.glyphicons.restart i:before,.glyph-restart:before {
  content: "\e366";
}
.glyphicons.security_camera i:before {
  content: "\e367";
}
.glyphicons.expand i:before,.glyph-expand:before {
  content: "\e368";
}
.glyphicons.collapse i:before,.glyph-collapse:before  {
  content: "\e369";
}
.glyphicons.collapse_top i:before,.glyph-collapse_top:before {
  content: "\e370";
}
.glyphicons.globe_af i:before {
  content: "\e371";
}
.glyphicons.global i:before {
  content: "\e372";
}
.glyphicons.spray i:before {
  content: "\e373";
}
.glyphicons.nails i:before {
  content: "\e374";
}
.glyphicons.claw_hammer i:before {
  content: "\e375";
}
.glyphicons.classic_hammer i:before {
  content: "\e376";
}
.glyphicons.hand_saw i:before {
  content: "\e377";
}
.glyphicons.riflescope i:before {
  content: "\e378";
}
.glyphicons.electrical_socket_eu i:before {
  content: "\e379";
}
.glyphicons.electrical_socket_us i:before {
  content: "\e380";
}
.glyphicons.pinterest i:before,.glyph-pinterest:before {
  content: "\e381";
}
.glyphicons.dropbox i:before,.glyph-dropbox:before {
  content: "\e382";
}
.glyphicons.google_plus i:before,.glyph-google_plus:before {
  content: "\e383";
}
.glyphicons.jolicloud i:before,.glyph-jolicloud:before{
  content: "\e384";
}
.glyphicons.yahoo i:before,.glyph-yahoo:before {
  content: "\e385";
}
.glyphicons.blogger i:before,.glyph-blogger:before {
  content: "\e386";
}
.glyphicons.picasa i:before,.glyph-picasa:before {
  content: "\e387";
}
.glyphicons.amazon i:before,.glyph-amazon:before {
  content: "\e388";
}
.glyphicons.tumblr i:before,.glyph-tumblr:before {
  content: "\e389";
}
.glyphicons.wordpress i:before,.glyph-wordpress:before {
  content: "\e390";
}
.glyphicons.instapaper i:before,.glyph-instapaper:before {
  content: "\e391";
}
.glyphicons.evernote i:before,.glyph-evernote:before {
  content: "\e392";
}
.glyphicons.xing i:before {
  content: "\e393";
}
.glyphicons.zootool i:before {
  content: "\e394";
}
.glyphicons.dribbble i:before,.glyph-dribbble:before  {
  content: "\e395";
}
.glyphicons.deviantart i:before {
  content: "\e396";
}
.glyphicons.read_it_later i:before {
  content: "\e397";
}
.glyphicons.linked_in i:before,.glyph-linked_in:before {
  content: "\e398";
}
.glyphicons.forrst i:before,.glyph-forrst:before {
  content: "\e399";
}
.glyphicons.pinboard i:before {
  content: "\e400";
}
.glyphicons.behance i:before {
  content: "\e401";
}
.glyphicons.github i:before,.glyph-github:before {
  content: "\e402";
}
.glyphicons.youtube i:before,.glyph-youtube:before {
  content: "\e403";
}
.glyphicons.skitch i:before {
  content: "\e404";
}
.glyphicons.foursquare i:before,.glyph-foursquare:before {
  content: "\e405";
}
.glyphicons.quora i:before,.glyph-quora:before {
  content: "\e406";
}
.glyphicons.badoo i:before {
  content: "\e407";
}
.glyphicons.spotify i:before ,.glyph-spotify:before {
  content: "\e408";
}
.glyphicons.stumbleupon i:before {
  content: "\e409";
}
.glyphicons.readability i:before {
  content: "\e410";
}
.glyphicons.facebook i:before,.glyph-facebook:before{
  content: "\e411";
}
.glyphicons.twitter i:before,.glyph-twitter:before  {
  content: "\e412";
}
.glyphicons.instagram i:before,.glyph-instagram:before {
  content: "\e413";
}
.glyphicons.posterous_spaces i:before,.glyph-posterous_spaces:before {
  content: "\e414";
}
.glyphicons.vimeo i:before,.glyph-vimeo:before {
  content: "\e415";
}
.glyphicons.flickr i:before,.glyph-flickr:before {
  content: "\e416";
}
.glyphicons.last_fm i:before,.glyph-last_fm:before {
  content: "\e417";
}
.glyphicons.rss i:before,.glyph-rss:before {
  content: "\e418";
}
.glyphicons.skype i:before,.glyph-skype:before {
  content: "\e419";
}
.glyphicons.e-mail i:before,.glyph-e-mail:before {
  content: "\e420";
}
.glyphicons-icon {
  display: inline-block;
  width: 48px;
  height: 48px;
  line-height: 48px;
  vertical-align: text-top;
  background-image: url(../images/glyphicons.png);
  background-position: 0 0;
  background-repeat: no-repeat;
  vertical-align: top;
  *display: inline;
  *zoom: 1;
  *margin-right: .3em;
}
.no-inlinesvg .glyphicons-icon {
  background-image: url(../images/glyphicons.png);
}
.glyphicons-icon.white {
  background-image: url(../images/glyphicons-white.svg);
}
.no-inlinesvg .glyphicons-icon.white {
  background-image: url(../images/glyphicons-white.png);
}
.glyphicons-icon.glass {
  background-position: 4px 11px;
}
.glyphicons-icon.leaf {
  background-position: -44px 11px;
}
.glyphicons-icon.dog {
  background-position: -92px 11px;
}
.glyphicons-icon.user {
  background-position: -140px 11px;
}
.glyphicons-icon.girl {
  background-position: -188px 11px;
}
.glyphicons-icon.car {
  background-position: -236px 11px;
}
.glyphicons-icon.user_add {
  background-position: -284px 11px;
}
.glyphicons-icon.user_remove {
  background-position: -332px 11px;
}
.glyphicons-icon.film {
  background-position: -380px 11px;
}
.glyphicons-icon.magic {
  background-position: -428px 11px;
}
.glyphicons-icon.envelope {
  background-position: 4px -37px;
}
.glyphicons-icon.camera {
  background-position: -44px -37px;
}
.glyphicons-icon.heart {
  background-position: -92px -37px;
}
.glyphicons-icon.beach_umbrella {
  background-position: -140px -37px;
}
.glyphicons-icon.train {
  background-position: -188px -37px;
}
.glyphicons-icon.print {
  background-position: -236px -37px;
}
.glyphicons-icon.bin {
  background-position: -284px -37px;
}
.glyphicons-icon.music {
  background-position: -332px -37px;
}
.glyphicons-icon.note {
  background-position: -380px -37px;
}
.glyphicons-icon.heart_empty {
  background-position: -428px -37px;
}
.glyphicons-icon.home {
  background-position: 4px -85px;
}
.glyphicons-icon.snowflake {
  background-position: -44px -85px;
}
.glyphicons-icon.fire {
  background-position: -92px -85px;
}
.glyphicons-icon.magnet {
  background-position: -140px -85px;
}
.glyphicons-icon.parents {
  background-position: -188px -85px;
}
.glyphicons-icon.binoculars {
  background-position: -236px -85px;
}
.glyphicons-icon.road {
  background-position: -284px -85px;
}
.glyphicons-icon.search {
  background-position: -332px -85px;
}
.glyphicons-icon.cars {
  background-position: -380px -85px;
}
.glyphicons-icon.notes_2 {
  background-position: -428px -85px;
}
.glyphicons-icon.pencil {
  background-position: 4px -133px;
}
.glyphicons-icon.bus {
  background-position: -44px -133px;
}
.glyphicons-icon.wifi_alt {
  background-position: -92px -133px;
}
.glyphicons-icon.luggage {
  background-position: -140px -133px;
}
.glyphicons-icon.old_man {
  background-position: -188px -133px;
}
.glyphicons-icon.woman {
  background-position: -236px -133px;
}
.glyphicons-icon.file {
  background-position: -284px -133px;
}
.glyphicons-icon.coins {
  background-position: -332px -133px;
}
.glyphicons-icon.airplane {
  background-position: -380px -133px;
}
.glyphicons-icon.notes {
  background-position: -428px -133px;
}
.glyphicons-icon.stats {
  background-position: 4px -181px;
}
.glyphicons-icon.charts {
  background-position: -44px -181px;
}
.glyphicons-icon.pie_chart {
  background-position: -92px -181px;
}
.glyphicons-icon.group {
  background-position: -140px -181px;
}
.glyphicons-icon.keys {
  background-position: -188px -181px;
}
.glyphicons-icon.calendar {
  background-position: -236px -181px;
}
.glyphicons-icon.router {
  background-position: -284px -181px;
}
.glyphicons-icon.camera_small {
  background-position: -332px -181px;
}
.glyphicons-icon.dislikes {
  background-position: -380px -181px;
}
.glyphicons-icon.star {
  background-position: -428px -181px;
}
.glyphicons-icon.link {
  background-position: 4px -229px;
}
.glyphicons-icon.eye_open {
  background-position: -44px -229px;
}
.glyphicons-icon.eye_close {
  background-position: -92px -229px;
}
.glyphicons-icon.alarm {
  background-position: -140px -229px;
}
.glyphicons-icon.clock {
  background-position: -188px -229px;
}
.glyphicons-icon.stopwatch {
  background-position: -236px -229px;
}
.glyphicons-icon.projector {
  background-position: -284px -229px;
}
.glyphicons-icon.history {
  background-position: -332px -229px;
}
.glyphicons-icon.truck {
  background-position: -380px -229px;
}
.glyphicons-icon.cargo {
  background-position: -428px -229px;
}
.glyphicons-icon.compass {
  background-position: 4px -277px;
}
.glyphicons-icon.keynote {
  background-position: -44px -277px;
}
.glyphicons-icon.paperclip {
  background-position: -92px -277px;
}
.glyphicons-icon.power {
  background-position: -140px -277px;
}
.glyphicons-icon.lightbulb {
  background-position: -188px -277px;
}
.glyphicons-icon.tag {
  background-position: -236px -277px;
}
.glyphicons-icon.tags {
  background-position: -284px -277px;
}
.glyphicons-icon.cleaning {
  background-position: -332px -277px;
}
.glyphicons-icon.ruller {
  background-position: -380px -277px;
}
.glyphicons-icon.gift {
  background-position: -428px -277px;
}
.glyphicons-icon.umbrella {
  background-position: 4px -325px;
}
.glyphicons-icon.book {
  background-position: -44px -325px;
}
.glyphicons-icon.bookmark {
  background-position: -92px -325px;
}
.glyphicons-icon.wifi {
  background-position: -140px -325px;
}
.glyphicons-icon.cup {
  background-position: -188px -325px;
}
.glyphicons-icon.stroller {
  background-position: -236px -325px;
}
.glyphicons-icon.headphones {
  background-position: -284px -325px;
}
.glyphicons-icon.headset {
  background-position: -332px -325px;
}
.glyphicons-icon.warning_sign {
  background-position: -380px -325px;
}
.glyphicons-icon.signal {
  background-position: -428px -325px;
}
.glyphicons-icon.retweet {
  background-position: 4px -373px;
}
.glyphicons-icon.refresh {
  background-position: -44px -373px;
}
.glyphicons-icon.roundabout {
  background-position: -92px -373px;
}
.glyphicons-icon.random {
  background-position: -140px -373px;
}
.glyphicons-icon.heat {
  background-position: -188px -373px;
}
.glyphicons-icon.repeat {
  background-position: -236px -373px;
}
.glyphicons-icon.display {
  background-position: -284px -373px;
}
.glyphicons-icon.log_book {
  background-position: -332px -373px;
}
.glyphicons-icon.adress_book {
  background-position: -380px -373px;
}
.glyphicons-icon.building {
  background-position: -428px -373px;
}
.glyphicons-icon.eyedropper {
  background-position: 4px -421px;
}
.glyphicons-icon.adjust {
  background-position: -44px -421px;
}
.glyphicons-icon.tint {
  background-position: -92px -421px;
}
.glyphicons-icon.crop {
  background-position: -140px -421px;
}
.glyphicons-icon.vector_path_square {
  background-position: -188px -421px;
}
.glyphicons-icon.vector_path_circle {
  background-position: -236px -421px;
}
.glyphicons-icon.vector_path_polygon {
  background-position: -284px -421px;
}
.glyphicons-icon.vector_path_line {
  background-position: -332px -421px;
}
.glyphicons-icon.vector_path_curve {
  background-position: -380px -421px;
}
.glyphicons-icon.vector_path_all {
  background-position: -428px -421px;
}
.glyphicons-icon.font {
  background-position: 4px -469px;
}
.glyphicons-icon.italic {
  background-position: -44px -469px;
}
.glyphicons-icon.bold {
  background-position: -92px -469px;
}
.glyphicons-icon.text_underline {
  background-position: -140px -469px;
}
.glyphicons-icon.text_strike {
  background-position: -188px -469px;
}
.glyphicons-icon.text_height {
  background-position: -236px -469px;
}
.glyphicons-icon.text_width {
  background-position: -284px -469px;
}
.glyphicons-icon.text_resize {
  background-position: -332px -469px;
}
.glyphicons-icon.left_indent {
  background-position: -380px -469px;
}
.glyphicons-icon.right_indent {
  background-position: -428px -469px;
}
.glyphicons-icon.align_left {
  background-position: 4px -517px;
}
.glyphicons-icon.align_center {
  background-position: -44px -517px;
}
.glyphicons-icon.align_right {
  background-position: -92px -517px;
}
.glyphicons-icon.justify {
  background-position: -140px -517px;
}
.glyphicons-icon.list {
  background-position: -188px -517px;
}
.glyphicons-icon.text_smaller {
  background-position: -236px -517px;
}
.glyphicons-icon.text_bigger {
  background-position: -284px -517px;
}
.glyphicons-icon.embed {
  background-position: -332px -517px;
}
.glyphicons-icon.embed_close {
  background-position: -380px -517px;
}
.glyphicons-icon.table {
  background-position: -428px -517px;
}
.glyphicons-icon.message_full {
  background-position: 4px -565px;
}
.glyphicons-icon.message_empty {
  background-position: -44px -565px;
}
.glyphicons-icon.message_in {
  background-position: -92px -565px;
}
.glyphicons-icon.message_out {
  background-position: -140px -565px;
}
.glyphicons-icon.message_plus {
  background-position: -188px -565px;
}
.glyphicons-icon.message_minus {
  background-position: -236px -565px;
}
.glyphicons-icon.message_ban {
  background-position: -284px -565px;
}
.glyphicons-icon.message_flag {
  background-position: -332px -565px;
}
.glyphicons-icon.message_lock {
  background-position: -380px -565px;
}
.glyphicons-icon.message_new {
  background-position: -428px -565px;
}
.glyphicons-icon.inbox {
  background-position: 4px -613px;
}
.glyphicons-icon.inbox_plus {
  background-position: -44px -613px;
}
.glyphicons-icon.inbox_minus {
  background-position: -92px -613px;
}
.glyphicons-icon.inbox_lock {
  background-position: -140px -613px;
}
.glyphicons-icon.inbox_in {
  background-position: -188px -613px;
}
.glyphicons-icon.inbox_out {
  background-position: -236px -613px;
}
.glyphicons-icon.cogwheel {
  background-position: -284px -613px;
}
.glyphicons-icon.cogwheels {
  background-position: -332px -613px;
}
.glyphicons-icon.picture {
  background-position: -380px -613px;
}
.glyphicons-icon.adjust_alt {
  background-position: -428px -613px;
}
.glyphicons-icon.database_lock {
  background-position: 4px -661px;
}
.glyphicons-icon.database_plus {
  background-position: -44px -661px;
}
.glyphicons-icon.database_minus {
  background-position: -92px -661px;
}
.glyphicons-icon.database_ban {
  background-position: -140px -661px;
}
.glyphicons-icon.folder_open {
  background-position: -188px -661px;
}
.glyphicons-icon.folder_plus {
  background-position: -236px -661px;
}
.glyphicons-icon.folder_minus {
  background-position: -284px -661px;
}
.glyphicons-icon.folder_lock {
  background-position: -332px -661px;
}
.glyphicons-icon.folder_flag {
  background-position: -380px -661px;
}
.glyphicons-icon.folder_new {
  background-position: -428px -661px;
}
.glyphicons-icon.edit {
  background-position: 4px -709px;
}
.glyphicons-icon.new_window {
  background-position: -44px -709px;
}
.glyphicons-icon.check {
  background-position: -92px -709px;
}
.glyphicons-icon.unchecked {
  background-position: -140px -709px;
}
.glyphicons-icon.more_windows {
  background-position: -188px -709px;
}
.glyphicons-icon.show_big_thumbnails {
  background-position: -236px -709px;
}
.glyphicons-icon.show_thumbnails {
  background-position: -284px -709px;
}
.glyphicons-icon.show_thumbnails_with_lines {
  background-position: -332px -709px;
}
.glyphicons-icon.show_lines {
  background-position: -380px -709px;
}
.glyphicons-icon.playlist {
  background-position: -428px -709px;
}
.glyphicons-icon.imac {
  background-position: 4px -757px;
}
.glyphicons-icon.macbook {
  background-position: -44px -757px;
}
.glyphicons-icon.ipad {
  background-position: -92px -757px;
}
.glyphicons-icon.iphone {
  background-position: -140px -757px;
}
.glyphicons-icon.iphone_transfer {
  background-position: -188px -757px;
}
.glyphicons-icon.iphone_exchange {
  background-position: -236px -757px;
}
.glyphicons-icon.ipod {
  background-position: -284px -757px;
}
.glyphicons-icon.ipod_shuffle {
  background-position: -332px -757px;
}
.glyphicons-icon.ear_plugs {
  background-position: -380px -757px;
}
.glyphicons-icon.phone {
  background-position: -428px -757px;
}
.glyphicons-icon.step_backward {
  background-position: 4px -805px;
}
.glyphicons-icon.fast_backward {
  background-position: -44px -805px;
}
.glyphicons-icon.rewind {
  background-position: -92px -805px;
}
.glyphicons-icon.play {
  background-position: -140px -805px;
}
.glyphicons-icon.pause {
  background-position: -188px -805px;
}
.glyphicons-icon.stop {
  background-position: -236px -805px;
}
.glyphicons-icon.forward {
  background-position: -284px -805px;
}
.glyphicons-icon.fast_forward {
  background-position: -332px -805px;
}
.glyphicons-icon.step_forward {
  background-position: -380px -805px;
}
.glyphicons-icon.eject {
  background-position: -428px -805px;
}
.glyphicons-icon.facetime_video {
  background-position: 4px -853px;
}
.glyphicons-icon.download_alt {
  background-position: -44px -853px;
}
.glyphicons-icon.mute {
  background-position: -92px -853px;
}
.glyphicons-icon.volume_down {
  background-position: -140px -853px;
}
.glyphicons-icon.volume_up {
  background-position: -188px -853px;
}
.glyphicons-icon.screenshot {
  background-position: -236px -853px;
}
.glyphicons-icon.move {
  background-position: -284px -853px;
}
.glyphicons-icon.more {
  background-position: -332px -853px;
}
.glyphicons-icon.brightness_reduce {
  background-position: -380px -853px;
}
.glyphicons-icon.brightness_increase {
  background-position: -428px -853px;
}
.glyphicons-icon.circle_plus {
  background-position: 4px -901px;
}
.glyphicons-icon.circle_minus {
  background-position: -44px -901px;
}
.glyphicons-icon.circle_remove {
  background-position: -92px -901px;
}
.glyphicons-icon.circle_ok {
  background-position: -140px -901px;
}
.glyphicons-icon.circle_question_mark {
  background-position: -188px -901px;
}
.glyphicons-icon.circle_info {
  background-position: -236px -901px;
}
.glyphicons-icon.circle_exclamation_mark {
  background-position: -284px -901px;
}
.glyphicons-icon.remove {
  background-position: -332px -901px;
}
.glyphicons-icon.ok {
  background-position: -380px -901px;
}
.glyphicons-icon.ban {
  background-position: -428px -901px;
}
.glyphicons-icon.download {
  background-position: 4px -949px;
}
.glyphicons-icon.upload {
  background-position: -44px -949px;
}
.glyphicons-icon.shopping_cart {
  background-position: -92px -949px;
}
.glyphicons-icon.lock {
  background-position: -140px -949px;
}
.glyphicons-icon.unlock {
  background-position: -188px -949px;
}
.glyphicons-icon.electricity {
  background-position: -236px -949px;
}
.glyphicons-icon.ok_2 {
  background-position: -284px -949px;
}
.glyphicons-icon.remove_2 {
  background-position: -332px -949px;
}
.glyphicons-icon.cart_out {
  background-position: -380px -949px;
}
.glyphicons-icon.cart_in {
  background-position: -428px -949px;
}
.glyphicons-icon.left_arrow {
  background-position: 4px -997px;
}
.glyphicons-icon.right_arrow {
  background-position: -44px -997px;
}
.glyphicons-icon.down_arrow {
  background-position: -92px -997px;
}
.glyphicons-icon.up_arrow {
  background-position: -140px -997px;
}
.glyphicons-icon.resize_small {
  background-position: -188px -997px;
}
.glyphicons-icon.resize_full {
  background-position: -236px -997px;
}
.glyphicons-icon.circle_arrow_left {
  background-position: -284px -997px;
}
.glyphicons-icon.circle_arrow_right {
  background-position: -332px -997px;
}
.glyphicons-icon.circle_arrow_top {
  background-position: -380px -997px;
}
.glyphicons-icon.circle_arrow_down {
  background-position: -428px -997px;
}
.glyphicons-icon.play_button {
  background-position: 4px -1045px;
}
.glyphicons-icon.unshare {
  background-position: -44px -1045px;
}
.glyphicons-icon.share {
  background-position: -92px -1045px;
}
.glyphicons-icon.chevron-right {
  background-position: -140px -1045px;
}
.glyphicons-icon.chevron-left {
  background-position: -188px -1045px;
}
.glyphicons-icon.bluetooth {
  background-position: -236px -1045px;
}
.glyphicons-icon.euro {
  background-position: -284px -1045px;
}
.glyphicons-icon.usd {
  background-position: -332px -1045px;
}
.glyphicons-icon.gbp {
  background-position: -380px -1045px;
}
.glyphicons-icon.retweet_2 {
  background-position: -428px -1045px;
}
.glyphicons-icon.moon {
  background-position: 4px -1093px;
}
.glyphicons-icon.sun {
  background-position: -44px -1093px;
}
.glyphicons-icon.cloud {
  background-position: -92px -1093px;
}
.glyphicons-icon.direction {
  background-position: -140px -1093px;
}
.glyphicons-icon.brush {
  background-position: -188px -1093px;
}
.glyphicons-icon.pen {
  background-position: -236px -1093px;
}
.glyphicons-icon.zoom_in {
  background-position: -284px -1093px;
}
.glyphicons-icon.zoom_out {
  background-position: -332px -1093px;
}
.glyphicons-icon.pin {
  background-position: -380px -1093px;
}
.glyphicons-icon.albums {
  background-position: -428px -1093px;
}
.glyphicons-icon.rotation_lock {
  background-position: 4px -1141px;
}
.glyphicons-icon.flash {
  background-position: -44px -1141px;
}
.glyphicons-icon.google_maps {
  background-position: -92px -1141px;
}
.glyphicons-icon.anchor {
  background-position: -140px -1141px;
}
.glyphicons-icon.conversation {
  background-position: -188px -1141px;
}
.glyphicons-icon.chat {
  background-position: -236px -1141px;
}
.glyphicons-icon.male {
  background-position: -284px -1141px;
}
.glyphicons-icon.female {
  background-position: -332px -1141px;
}
.glyphicons-icon.asterisk {
  background-position: -380px -1141px;
}
.glyphicons-icon.divide {
  background-position: -428px -1141px;
}
.glyphicons-icon.snorkel_diving {
  background-position: 4px -1189px;
}
.glyphicons-icon.scuba_diving {
  background-position: -44px -1189px;
}
.glyphicons-icon.oxygen_bottle {
  background-position: -92px -1189px;
}
.glyphicons-icon.fins {
  background-position: -140px -1189px;
}
.glyphicons-icon.fishes {
  background-position: -188px -1189px;
}
.glyphicons-icon.boat {
  background-position: -236px -1189px;
}
.glyphicons-icon.delete {
  background-position: -284px -1189px;
}
.glyphicons-icon.sheriffs_star {
  background-position: -332px -1189px;
}
.glyphicons-icon.qrcode {
  background-position: -380px -1189px;
}
.glyphicons-icon.barcode {
  background-position: -428px -1189px;
}
.glyphicons-icon.pool {
  background-position: 4px -1237px;
}
.glyphicons-icon.buoy {
  background-position: -44px -1237px;
}
.glyphicons-icon.spade {
  background-position: -92px -1237px;
}
.glyphicons-icon.bank {
  background-position: -140px -1237px;
}
.glyphicons-icon.vcard {
  background-position: -188px -1237px;
}
.glyphicons-icon.electrical_plug {
  background-position: -236px -1237px;
}
.glyphicons-icon.flag {
  background-position: -284px -1237px;
}
.glyphicons-icon.credit_card {
  background-position: -332px -1237px;
}
.glyphicons-icon.keyboard-wireless {
  background-position: -380px -1237px;
}
.glyphicons-icon.keyboard-wired {
  background-position: -428px -1237px;
}
.glyphicons-icon.shield {
  background-position: 4px -1285px;
}
.glyphicons-icon.ring {
  background-position: -44px -1285px;
}
.glyphicons-icon.cake {
  background-position: -92px -1285px;
}
.glyphicons-icon.drink {
  background-position: -140px -1285px;
}
.glyphicons-icon.beer {
  background-position: -188px -1285px;
}
.glyphicons-icon.fast_food {
  background-position: -236px -1285px;
}
.glyphicons-icon.cutlery {
  background-position: -284px -1285px;
}
.glyphicons-icon.pizza {
  background-position: -332px -1285px;
}
.glyphicons-icon.birthday_cake {
  background-position: -380px -1285px;
}
.glyphicons-icon.tablet {
  background-position: -428px -1285px;
}
.glyphicons-icon.settings {
  background-position: 4px -1333px;
}
.glyphicons-icon.bullets {
  background-position: -44px -1333px;
}
.glyphicons-icon.cardio {
  background-position: -92px -1333px;
}
.glyphicons-icon.t-shirt {
  background-position: -140px -1333px;
}
.glyphicons-icon.pants {
  background-position: -188px -1333px;
}
.glyphicons-icon.sweater {
  background-position: -236px -1333px;
}
.glyphicons-icon.fabric {
  background-position: -284px -1333px;
}
.glyphicons-icon.leather {
  background-position: -332px -1333px;
}
.glyphicons-icon.scissors {
  background-position: -380px -1333px;
}
.glyphicons-icon.bomb {
  background-position: -428px -1333px;
}
.glyphicons-icon.skull {
  background-position: 4px -1381px;
}
.glyphicons-icon.celebration {
  background-position: -44px -1381px;
}
.glyphicons-icon.tea_kettle {
  background-position: -92px -1381px;
}
.glyphicons-icon.french_press {
  background-position: -140px -1381px;
}
.glyphicons-icon.coffe_cup {
  background-position: -188px -1381px;
}
.glyphicons-icon.pot {
  background-position: -236px -1381px;
}
.glyphicons-icon.grater {
  background-position: -284px -1381px;
}
.glyphicons-icon.kettle {
  background-position: -332px -1381px;
}
.glyphicons-icon.hospital {
  background-position: -380px -1381px;
}
.glyphicons-icon.hospital_h {
  background-position: -428px -1381px;
}
.glyphicons-icon.microphone {
  background-position: 4px -1429px;
}
.glyphicons-icon.webcam {
  background-position: -44px -1429px;
}
.glyphicons-icon.temple_christianity_church {
  background-position: -92px -1429px;
}
.glyphicons-icon.temple_islam {
  background-position: -140px -1429px;
}
.glyphicons-icon.temple_hindu {
  background-position: -188px -1429px;
}
.glyphicons-icon.temple_buddhist {
  background-position: -236px -1429px;
}
.glyphicons-icon.bicycle {
  background-position: -284px -1429px;
}
.glyphicons-icon.life_preserver {
  background-position: -332px -1429px;
}
.glyphicons-icon.share_alt {
  background-position: -380px -1429px;
}
.glyphicons-icon.comments {
  background-position: -428px -1429px;
}
.glyphicons-icon.flower {
  background-position: 4px -1477px;
}
.glyphicons-icon.baseball {
  background-position: -44px -1477px;
}
.glyphicons-icon.rugby {
  background-position: -92px -1477px;
}
.glyphicons-icon.ax {
  background-position: -140px -1477px;
}
.glyphicons-icon.table_tennis {
  background-position: -188px -1477px;
}
.glyphicons-icon.bowling {
  background-position: -236px -1477px;
}
.glyphicons-icon.tree_conifer {
  background-position: -284px -1477px;
}
.glyphicons-icon.tree_deciduous {
  background-position: -332px -1477px;
}
.glyphicons-icon.more_items {
  background-position: -380px -1477px;
}
.glyphicons-icon.sort {
  background-position: -428px -1477px;
}
.glyphicons-icon.filter {
  background-position: 4px -1525px;
}
.glyphicons-icon.gamepad {
  background-position: -44px -1525px;
}
.glyphicons-icon.playing_dices {
  background-position: -92px -1525px;
}
.glyphicons-icon.calculator {
  background-position: -140px -1525px;
}
.glyphicons-icon.tie {
  background-position: -188px -1525px;
}
.glyphicons-icon.wallet {
  background-position: -236px -1525px;
}
.glyphicons-icon.piano {
  background-position: -284px -1525px;
}
.glyphicons-icon.sampler {
  background-position: -332px -1525px;
}
.glyphicons-icon.podium {
  background-position: -380px -1525px;
}
.glyphicons-icon.soccer_ball {
  background-position: -428px -1525px;
}
.glyphicons-icon.blog {
  background-position: 4px -1573px;
}
.glyphicons-icon.dashboard {
  background-position: -44px -1573px;
}
.glyphicons-icon.certificate {
  background-position: -92px -1573px;
}
.glyphicons-icon.bell {
  background-position: -140px -1573px;
}
.glyphicons-icon.candle {
  background-position: -188px -1573px;
}
.glyphicons-icon.pushpin {
  background-position: -236px -1573px;
}
.glyphicons-icon.iphone_shake {
  background-position: -284px -1573px;
}
.glyphicons-icon.pin_flag {
  background-position: -332px -1573px;
}
.glyphicons-icon.turtle {
  background-position: -380px -1573px;
}
.glyphicons-icon.rabbit {
  background-position: -428px -1573px;
}
.glyphicons-icon.globe {
  background-position: 4px -1621px;
}
.glyphicons-icon.briefcase {
  background-position: -44px -1621px;
}
.glyphicons-icon.hdd {
  background-position: -92px -1621px;
}
.glyphicons-icon.thumbs_up {
  background-position: -140px -1621px;
}
.glyphicons-icon.thumbs_down {
  background-position: -188px -1621px;
}
.glyphicons-icon.hand_right {
  background-position: -236px -1621px;
}
.glyphicons-icon.hand_left {
  background-position: -284px -1621px;
}
.glyphicons-icon.hand_up {
  background-position: -332px -1621px;
}
.glyphicons-icon.hand_down {
  background-position: -380px -1621px;
}
.glyphicons-icon.fullscreen {
  background-position: -428px -1621px;
}
.glyphicons-icon.shopping_bag {
  background-position: 4px -1669px;
}
.glyphicons-icon.book_open {
  background-position: -44px -1669px;
}
.glyphicons-icon.nameplate {
  background-position: -92px -1669px;
}
.glyphicons-icon.nameplate_alt {
  background-position: -140px -1669px;
}
.glyphicons-icon.vases {
  background-position: -188px -1669px;
}
.glyphicons-icon.bullhorn {
  background-position: -236px -1669px;
}
.glyphicons-icon.dumbbell {
  background-position: -284px -1669px;
}
.glyphicons-icon.suitcase {
  background-position: -332px -1669px;
}
.glyphicons-icon.file_import {
  background-position: -380px -1669px;
}
.glyphicons-icon.file_export {
  background-position: -428px -1669px;
}
.glyphicons-icon.bug {
  background-position: 4px -1717px;
}
.glyphicons-icon.crown {
  background-position: -44px -1717px;
}
.glyphicons-icon.smoking {
  background-position: -92px -1717px;
}
.glyphicons-icon.cloud-upload {
  background-position: -140px -1717px;
}
.glyphicons-icon.cloud-download {
  background-position: -188px -1717px;
}
.glyphicons-icon.restart {
  background-position: -236px -1717px;
}
.glyphicons-icon.security_camera {
  background-position: -284px -1717px;
}
.glyphicons-icon.expand {
  background-position: -332px -1717px;
}
.glyphicons-icon.collapse {
  background-position: -380px -1717px;
}
.glyphicons-icon.collapse_top {
  background-position: -428px -1717px;
}
.glyphicons-icon.globe_af {
  background-position: 4px -1765px;
}
.glyphicons-icon.global {
  background-position: -44px -1765px;
}
.glyphicons-icon.spray {
  background-position: -92px -1765px;
}
.glyphicons-icon.nails {
  background-position: -140px -1765px;
}
.glyphicons-icon.claw_hammer {
  background-position: -188px -1765px;
}
.glyphicons-icon.classic_hammer {
  background-position: -236px -1765px;
}
.glyphicons-icon.hand_saw {
  background-position: -284px -1765px;
}
.glyphicons-icon.riflescope {
  background-position: -332px -1765px;
}
.glyphicons-icon.electrical_socket_eu {
  background-position: -380px -1765px;
}
.glyphicons-icon.electrical_socket_us {
  background-position: -428px -1765px;
}
.glyphicons-icon.pinterest {
  background-position: 4px -1813px;
}
.glyphicons-icon.dropbox {
  background-position: -44px -1813px;
}
.glyphicons-icon.google_plus {
  background-position: -92px -1813px;
}
.glyphicons-icon.jolicloud {
  background-position: -140px -1813px;
}
.glyphicons-icon.yahoo {
  background-position: -188px -1813px;
}
.glyphicons-icon.blogger {
  background-position: -236px -1813px;
}
.glyphicons-icon.picasa {
  background-position: -284px -1813px;
}
.glyphicons-icon.amazon {
  background-position: -332px -1813px;
}
.glyphicons-icon.tumblr {
  background-position: -380px -1813px;
}
.glyphicons-icon.wordpress {
  background-position: -428px -1813px;
}
.glyphicons-icon.instapaper {
  background-position: 4px -1861px;
}
.glyphicons-icon.evernote {
  background-position: -44px -1861px;
}
.glyphicons-icon.xing {
  background-position: -92px -1861px;
}
.glyphicons-icon.zootool {
  background-position: -140px -1861px;
}
.glyphicons-icon.dribbble {
  background-position: -188px -1861px;
}
.glyphicons-icon.deviantart {
  background-position: -236px -1861px;
}
.glyphicons-icon.read_it_later {
  background-position: -284px -1861px;
}
.glyphicons-icon.linked_in {
  background-position: -332px -1861px;
}
.glyphicons-icon.forrst {
  background-position: -380px -1861px;
}
.glyphicons-icon.pinboard {
  background-position: -428px -1861px;
}
.glyphicons-icon.behance {
  background-position: 4px -1909px;
}
.glyphicons-icon.github {
  background-position: -44px -1909px;
}
.glyphicons-icon.youtube {
  background-position: -92px -1909px;
}
.glyphicons-icon.skitch {
  background-position: -140px -1909px;
}
.glyphicons-icon.foursquare {
  background-position: -188px -1909px;
}
.glyphicons-icon.quora {
  background-position: -236px -1909px;
}
.glyphicons-icon.badoo {
  background-position: -284px -1909px;
}
.glyphicons-icon.spotify {
  background-position: -332px -1909px;
}
.glyphicons-icon.stumbleupon {
  background-position: -380px -1909px;
}
.glyphicons-icon.readability {
  background-position: -428px -1909px;
}
.glyphicons-icon.facebook {
  background-position: 4px -1957px;
}
.glyphicons-icon.twitter {
  background-position: -44px -1957px;
}
.glyphicons-icon.instagram {
  background-position: -92px -1957px;
}
.glyphicons-icon.posterous_spaces {
  background-position: -140px -1957px;
}
.glyphicons-icon.vimeo {
  background-position: -188px -1957px;
}
.glyphicons-icon.flickr {
  background-position: -236px -1957px;
}
.glyphicons-icon.last_fm {
  background-position: -284px -1957px;
}
.glyphicons-icon.rss {
  background-position: -332px -1957px;
}
.glyphicons-icon.skype {
  background-position: -380px -1957px;
}
.glyphicons-icon.e-mail {
  background-position: -428px -1957px;
}
@font-face {font-family: 'ProximaNova-Regular';src: url('/assets/thirdParty/css/1415F2_1.eot');src: url('/assets/thirdParty/css/1415F2_1IE.eot') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_1.woff') format('woff'),url('/assets/thirdParty/css/1415F2_1.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_1.svg') format('svg');font-style: normal;font-weight: normal;}

/*@font-face {font-family: 'ProximaNovaT-Thin';src: url('/assets/thirdParty/css/1415F2_0.eot');src: url('/assets/thirdParty/css/1415F2_0.eot?#iefix') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_0.woff') format('woff'),url('/assets/thirdParty/css/1415F2_0.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_0.svg#wf') format('svg');font-style: normal;font-weight: normal;}
@font-face {font-family: 'ProximaNova-Regular';src: url('/assets/thirdParty/css/1415F2_1.eot');src: url('/assets/thirdParty/css/1415F2_1.eot?#iefix') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_1.woff') format('woff'),url('/assets/thirdParty/css/1415F2_1.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_1.svg#wf') format('svg');font-style: normal;font-weight: normal;}
@font-face {font-family: 'ProximaNova-Semibold';src: url('/assets/thirdParty/css/1415F2_2.eot');src: url('/assets/thirdParty/css/1415F2_2.eot?#iefix') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_2.woff') format('woff'),url('/assets/thirdParty/css/1415F2_2.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_2.svg#wf') format('svg');font-style: normal;font-weight: normal;}
@font-face {font-family: 'ProximaNova-Light';src: url('/assets/thirdParty/css/1415F2_3.eot');src: url('/assets/thirdParty/css/1415F2_3.eot?#iefix') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_3.woff') format('woff'),url('/assets/thirdParty/css/1415F2_3.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_3.svg#wf') format('svg');font-style: normal;font-weight: normal;}
@font-face {font-family: 'ProximaNova-Bold';src: url('/assets/thirdParty/css/1415F2_4.eot');src: url('/assets/thirdParty/css/1415F2_4.eot?#iefix') format('embedded-opentype'),url('/assets/thirdParty/css/1415F2_4.woff') format('woff'),url('/assets/thirdParty/css/1415F2_4.ttf') format('truetype'),url('/assets/thirdParty/css/1415F2_4.svg#wf') format('svg');font-style: normal;font-weight: normal;}*/@charset "UTF-8";
/* (c) 2012 Instagram, Inc, */
/* @license
 * MyFonts Webfont Build ID 2164953, 2012-03-23T23:06:30-0400
 *
 * The fonts listed in this notice are subject to the End User License
 * Agreement(s) entered into by the website owner. All other parties are
 * explicitly restricted from using the Licensed Webfonts(s).
 *
 * You may obtain a valid license at the URLs below.
 *
 *
 * Webfont: Proxima Nova Regular by Mark Simonson
 * URL: http://www.myfonts.com/fonts/marksimonson/proxima-nova/regular/
 * Licensed pageviews: unlimited
 *
 * Webfont: Proxima Nova Semibold by Mark Simonson
 * URL: http://www.myfonts.com/fonts/marksimonson/proxima-nova/semibold/
 * Licensed pageviews: unlimited
 *
 * Webfont: Proxima Nova Bold Italic by Mark Simonson
 * URL: http://www.myfonts.com/fonts/marksimonson/proxima-nova/bold-it/
 * Licensed pageviews: unlimited
 *
 * Webfont: Proxima Nova Bold by Mark Simonson
 * URL: http://www.myfonts.com/fonts/marksimonson/proxima-nova/bold/
 * Licensed pageviews: unlimited
 *
 * Webfont: Proxima Nova Italic by Mark Simonson
 * URL: http://www.myfonts.com/fonts/marksimonson/proxima-nova/regular-it/
 * Licensed pageviews: unlimited
 *
 *
 * License: http://www.myfonts.com/viewlicense?type=web&buildid=2164953
 * Webfonts copyright: Copyright (c) Mark Simonson, 2005. All rights reserved.
 *
 * © 2012 Bitstream Inc
 */
@font-face {
  font-family: 'proxima-nova';
  src: url("/assets/thirdParty/webfonts/proximanova-bold-webfont.eot");
  src: url("/assets/thirdParty/webfonts/proximanova-bold-webfont.eot?#iefix") format("embedded-opentype"), url("/assets/thirdParty/webfonts/proximanova-bold-webfont.woff") format("woff"), url("/assets/thirdParty/webfonts/proximanova-bold-webfont.ttf") format("truetype"), url("/assets/thirdParty/webfonts/proximanova-bold-webfont.svg#ProximaNovaBold") format("svg");
  font-weight: bold;
  font-style: normal;
}

@font-face {
  font-family: 'proxima-nova';
  src: url("/assets/thirdParty/webfonts/proximanova-boldit-webfont.eot");
  src: url("/assets/thirdParty/webfonts/proximanova-boldit-webfont.eot?#iefix") format("embedded-opentype"), url("/assets/thirdParty/webfonts/proximanova-boldit-webfont.woff") format("woff"), url("/assets/thirdParty/webfonts/proximanova-boldit-webfont.ttf") format("truetype"), url("/assets/thirdParty/webfonts/proximanova-boldit-webfont.svg#ProximaNovaBoldItalic") format("svg");
  font-weight: bold;
  font-style: italic;
}

@font-face {
  font-family: 'proxima-nova';
  src: url("/assets/thirdParty/webfonts/proximanova-reg-webfont.eot");
  src: url("/assets/thirdParty/webfonts/proximanova-reg-webfont.eot?#iefix") format("embedded-opentype"), url("/assets/thirdParty/webfonts/proximanova-reg-webfont.woff") format("woff"), url("/assets/thirdParty/webfonts/proximanova-reg-webfont.ttf") format("truetype"), url("/assets/thirdParty/webfonts/proximanova-reg-webfont.svg#ProximaNovaRegular") format("svg");
  font-weight: normal;
  font-style: normal;
}

@font-face {
  font-family: 'proxima-nova';
  src: url("/assets/thirdParty/webfonts/proximanova-regit-webfont.eot");
  src: url("/assets/thirdParty/webfonts/proximanova-regit-webfont.eot?#iefix") format("embedded-opentype"), url("/assets/thirdParty/webfonts/proximanova-regit-webfont.woff") format("woff"), url("/assets/thirdParty/webfonts/proximanova-regit-webfont.ttf") format("truetype"), url("/assets/thirdParty/webfonts/proximanova-regit-webfont.svg#ProximaNovaRegularItalic") format("svg");
  font-weight: normal;
  font-style: italic;
}

@font-face {
  font-family: 'proxima-nova';
  src: url("/assets/thirdParty/webfonts/proximanova-sbold-webfont.eot");
  src: url("/assets/thirdParty/webfonts/proximanova-sbold-webfont.eot?#iefix") format("embedded-opentype"), url("/assets/thirdParty/webfonts/proximanova-sbold-webfont.woff") format("woff"), url("/assets/thirdParty/webfonts/proximanova-sbold-webfont.ttf") format("truetype"), url("/assets/thirdParty/webfonts/proximanova-sbold-webfont.svg#ProximaNovaSemibold") format("svg");
  font-weight: 600;
  font-style: normal;
}

/* begin wysihtml5 fixes *******************************************************************************************/

#simpleIFrame{
	height:250px;
}
.wysihtml5-sandbox{
	height:200px;
}
/* end wysihtml5 fixes *******************************************************************************************/

/* begin bootstrap fixes---------------------------------------------------------------------------------*/
body,label, .btn, .ui-widget,.ajaxLink, i, p{
	font-size:16px;	
	color:#666;	
	font-family: 'ProximaNova-Regular', "Helvetica Neue", Arial, Helvetica, sans-serif;
/*font-family: 'proxima-nova', 'Helvetica Neue', Arial, Helvetica, sans-serif;*/

}

body{
	background:url('/assets/img/bg.png');
	background-attachment: fixed; height: 100%;
}
::-webkit-input-placeholder { color:gray; font-size:14px;}
::-moz-placeholder { color:gray; font-size:14px;} /* firefox 19+ */
input:-moz-placeholder { color:gray; font-size:14px; }

.border-image {
-moz-border-image: url('/assets/img/bg.png') 25% repeat repeat;
-webkit-border-image:url('/assets/img/bg.png') 25% repeat repeat;
	border-image: url('/assets/img/bg.png') 25% repeat repeat;
}

.oneSideBorderShadow{

	-moz-box-shadow: 10px 0px 5px 2px #EDF1F5;
    -webkit-box-shadow: 10px 0px 5px 2px #EDF1F5;
    box-shadow: 10px 0px 5px 2px #EDF1F5;
}

input[type="file"]{
    opacity:0;
}
input, input[type="text"], input[type="password"], button, select, textarea{
	font-size:20px;
	line-height:24px;
}

::-webkit-input-placeholder {
    color:    #999;
    font-size:16px;
}
:-moz-placeholder {
    color:    #999;
    font-size:16px;
}
::-moz-placeholder {
    color:    #999;
    font-size:16px;
}
:-ms-input-placeholder {
    color:    #999;
    font-size:16px;
}

p, code{
 font-size:16px;
 line-height:20px;
}

textarea, input, form {
	width:100%;	
}
form{
	margin:0 0 5px 0;
}
textarea{
	height:150px !important;
}
input{
	line-height:24px;
	/*height:28px !important;*/
	height:32px !important;
	height:36px !important;
}
legend{
	margin-bottom:0px;
}
.alertClass{
	margin:auto;width:50%;
}
.alert .close{
	position: relative;
	top: 0px;
	right: 0px;
}
.badge {
	padding: 4px 9px 2px;
}
.btn-large{
	font-size:18px;
}
.btn.disabled{
	opacity:0.4;
}
.btnLookLikeLink {
    background:none!important;
     border:none; 
     color:#666;
     padding:0!important;
    /*border is optional*/
     border-bottom:none; 
}
.btnLookLikeLink:hover, .btnLookLikeLink:active{
	 border-bottom:1px solid #666;
}
.table th, .table td {
	border-top:1px solid#D2D9E7;
}
.table tbody tr:hover td,
.table tbody tr:hover th {
  	background-color: #f5f5f5;
	background:#EDEFF4;
}
.modal{
	width:auto;
	border: 1px solid #EDEFF4;
}
.modal-footer{
	background:none;
}
.modal-footer button{
	color:#555;
	font-size:24px;
}
.brand, .badge, .brand:hover {
	text-decoration:none;
	font-weight: bold;
	text-shadow: 0 1px 0 
	rgba(255, 255, 255, .1), 0 0 30px 
	rgba(255, 255, 255, .125);
	-webkit-transition: all .2s linear;
	-moz-transition: all .2s linear;
	transition: all .2s linear;
}
.dropdown-menu{	
	border:none;
}
.dropdown-menu.well{
	padding:0px;
}
.dropdown-menu li > a:hover, .dropdown-menu .active > a, .dropdown-menu .active > a:hover {
	color:#315B7E;	
	text-decoration: none;
	background:none;
}
.dropdown-menu li:hover, .dropdown-menu .active, .dropdown-menu .active:hover{
	background-color: #ECF6FF;
	border-radius:5px;
	-moz-border-radius:10px;
}
.divider {
  height: 2px;
  margin: 8px 1px;
  overflow: hidden;
  background-color: #ECF6FF;
  border-bottom: 1px solid #ECF6FF;
  *width: 100%;
  *margin: -5px 0 5px;
}
.clearfix::after{
	clear:both;
	content: ".";
	display:block;
	font-size:0;
	height:0;
	line-height:0;
	visibility:hidden;
}
a:not(.glyphicons):active, a:not(.glyphicons) > i:active {
	position: relative;
	top: 1px;
}

.btn-babyBlue {
	background-color: hsl(204, 56%, 69%) !important;
	background-repeat: repeat-x;
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#d3e6f2", endColorstr="#83b8dc");
	background-image: -khtml-gradient(linear, left top, left bottom, from(#d3e6f2), to(#83b8dc));
	background-image: -moz-linear-gradient(top, #d3e6f2, #83b8dc);
	background-image: -ms-linear-gradient(top, #d3e6f2, #83b8dc);
	background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #d3e6f2), color-stop(100%, #83b8dc));
	background-image: -webkit-linear-gradient(top, #d3e6f2, #83b8dc);
	background-image: -o-linear-gradient(top, #d3e6f2, #83b8dc);
	background-image: linear-gradient(#d3e6f2, #83b8dc);
	border-color: #83b8dc #83b8dc hsl(204, 56%, 64%);
	color:#666;
	text-shadow: 0 1px 1px rgba(255, 255, 255, 0.33);
	-webkit-font-smoothing: antialiased;	
}
.buyButton{
	font-weight:bold;
	text-align: center;
	text-shadow: 0 1px 1px rgba(255, 255, 255, 0.75);
	vertical-align: middle;
}
.buyButton:active, .buyButton:hover{
	color:#666;
}
.btn-fbGray{
	background-color: hsl(223, 24%, 94%) !important;
	background-repeat: repeat-x;
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#eceef3", endColorstr="#eceef3");
	background-image: -khtml-gradient(linear, left top, left bottom, from(#eceef3), to(#eceef3));
	background-image: -moz-linear-gradient(top, #eceef3, #eceef3);
	background-image: -ms-linear-gradient(top, #eceef3, #eceef3);
	background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #eceef3), color-stop(100%, #eceef3));
	background-image: -webkit-linear-gradient(top, #eceef3, #eceef3);
	background-image: -o-linear-gradient(top, #eceef3, #eceef3);
	background-image: linear-gradient(#eceef3, #eceef3);
	border-color: #eceef3 #eceef3 hsl(223, 24%, 94%);
	color: #333 !important;
	text-shadow: 0 1px 1px rgba(255, 255, 255, 0.00);
	-webkit-font-smoothing: antialiased;
}
.btn-gray{
	background-color: hsl(0, 0%, 75%) ;
	background-repeat: repeat-x;
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#bfbfbf", endColorstr="#bfbfbf");
	background-image: -khtml-gradient(linear, left top, left bottom, from(#bfbfbf), to(#bfbfbf));
	background-image: -moz-linear-gradient(top, #bfbfbf, #bfbfbf);
	background-image: -ms-linear-gradient(top, #bfbfbf, #bfbfbf);
	background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #bfbfbf), color-stop(100%, #bfbfbf));
	background-image: -webkit-linear-gradient(top, #bfbfbf, #bfbfbf);
	background-image: -o-linear-gradient(top, #bfbfbf, #bfbfbf);
	background-image: linear-gradient(#bfbfbf, #bfbfbf);
	border-color: #bfbfbf #bfbfbf hsl(0, 0%, 75%);
	color: #fff ;
	text-shadow: 0 1px 1px rgba(255, 255, 255, 0.00);
	-webkit-font-smoothing: antialiased;
	border-radius:5px;
}
.button.gray { /*http://swiftype.com/mobile*/
color: white;
border: 1px solid #5B6474;
background-color: #647084;
background: -webkit-gradient(linear, 50% 0%, 50% 100%, color-stop(0%, #99A9B1), color-stop(100%, #647084));
background: -webkit-linear-gradient(#99A9B1,#647084);
background: -moz-linear-gradient(#99A9B1,#647084);
background: -o-linear-gradient(#99A9B1,#647084);
background: linear-gradient(#99A9B1,#647084);
-webkit-box-shadow: inset 0px 1px 0px #C2D3DC;
-moz-box-shadow: inset 0px 1px 0px #c2d3dc;
box-shadow: inset 0px 1px 0px #C2D3DC;
text-shadow: 0px -1px 0px rgba(0, 0, 0, 0.4);
}
/*.btn-gray:active, .btn-gray:hover{
	color: #58A279;
}*/
.btn-lightestBlue{
  background-color: hsl(216, 45%, 98%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#106104101", endColorstr="#f7f9fc");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#106104101), to(#f7f9fc));
  background-image: -moz-linear-gradient(top, #106104101, #f7f9fc);
  background-image: -ms-linear-gradient(top, #106104101, #f7f9fc);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #106104101), color-stop(100%, #f7f9fc));
  background-image: -webkit-linear-gradient(top, #106104101, #f7f9fc);
  background-image: -o-linear-gradient(top, #106104101, #f7f9fc);
  background-image: linear-gradient(#106104101, #f7f9fc);
  border-color: #f7f9fc #f7f9fc hsl(216, 45%, 97%);
  color: #333 !important;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.06);
  -webkit-font-smoothing: antialiased;
}
.btn-green, .btn-success {
  background-color: hsl(148, 30%, 49%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#84bc9e", endColorstr="#57a27a");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#84bc9e), to(#57a27a));
  background-image: -moz-linear-gradient(top, #84bc9e, #57a27a);
  background-image: -ms-linear-gradient(top, #84bc9e, #57a27a);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #84bc9e), color-stop(100%, #57a27a));
  background-image: -webkit-linear-gradient(top, #84bc9e, #57a27a);
  background-image: -o-linear-gradient(top, #84bc9e, #57a27a);
  background-image: linear-gradient(#84bc9e, #57a27a);
  border-color: #57a27a #57a27a hsl(148, 30%, 45.5%);
  color: #fff !important;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.23);
  -webkit-font-smoothing: antialiased;
}
.btn-red, .btn-danger {
  background-color: hsl(348, 83%, 47%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#ee496a", endColorstr="#db143c");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ee496a), to(#db143c));
  background-image: -moz-linear-gradient(top, #ee496a, #db143c);
  background-image: -ms-linear-gradient(top, #ee496a, #db143c);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ee496a), color-stop(100%, #db143c));
  background-image: -webkit-linear-gradient(top, #ee496a, #db143c);
  background-image: -o-linear-gradient(top, #ee496a, #db143c);
  background-image: linear-gradient(#ee496a, #db143c);
  border-color: #db143c #db143c hsl(348, 83%, 43.5%);
  color: #fff !important;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.23);
  -webkit-font-smoothing: antialiased;
}
.btn-darkGray{
  background-color: hsl(0, 0%, 18%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#606060", endColorstr="#2d2d2d");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#606060), to(#2d2d2d));
  background-image: -moz-linear-gradient(top, #606060, #2d2d2d);
  background-image: -ms-linear-gradient(top, #606060, #2d2d2d);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #606060), color-stop(100%, #2d2d2d));
  background-image: -webkit-linear-gradient(top, #606060, #2d2d2d);
  background-image: -o-linear-gradient(top, #606060, #2d2d2d);
  background-image: linear-gradient(#606060, #2d2d2d);
  border-color: #2d2d2d #2d2d2d hsl(0, 0%, 13%);
  color: #fff !important;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.33);
  -webkit-font-smoothing: antialiased;
}
.btn-blue, .modal.btn-blue:active  {
  background-color: hsl(206, 44%, 34%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#4481b0", endColorstr="#305b7c");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#4481b0), to(#305b7c));
  background-image: -moz-linear-gradient(top, #4481b0, #305b7c);
  background-image: -ms-linear-gradient(top, #4481b0, #305b7c);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #4481b0), color-stop(100%, #305b7c));
  background-image: -webkit-linear-gradient(top, #4481b0, #305b7c);
  background-image: -o-linear-gradient(top, #4481b0, #305b7c);
  background-image: linear-gradient(#4481b0, #305b7c);
  border-color: #305b7c #305b7c hsl(206, 44%, 30.5%);
  color: #fff !important;
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.23);
  -webkit-font-smoothing: antialiased;
}
.btn-blue2{
	background-color: hsl(208, 100%, 40%) !important;
	background-repeat: repeat-x;
	filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#329fff", endColorstr="#006ccc");
	background-image: -khtml-gradient(linear, left top, left bottom, from(#329fff), to(#006ccc));
	background-image: -moz-linear-gradient(top, #329fff, #006ccc);
	background-image: -ms-linear-gradient(top, #329fff, #006ccc);
	background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #329fff), color-stop(100%, #006ccc));
	background-image: -webkit-linear-gradient(top, #329fff, #006ccc);
	background-image: -o-linear-gradient(top, #329fff, #006ccc);
	background-image: linear-gradient(#329fff, #006ccc);
	border-color: #006ccc #006ccc hsl(208, 100%, 35%);
	color: #333 !important;
	text-shadow: 0 1px 1px rgba(255, 255, 255, 0.33);
	-webkit-font-smoothing: antialiased;
}

.btn-primary, btn-primary:active, .label-info, .label-info:hover  {
  background-color: hsl(210, 53%, 41%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#669bd0", endColorstr="#31689f");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#669bd0), to(#31689f));
  background-image: -moz-linear-gradient(top, #669bd0, #31689f);
  background-image: -ms-linear-gradient(top, #669bd0, #31689f);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #669bd0), color-stop(100%, #31689f));
  background-image: -webkit-linear-gradient(top, #669bd0, #31689f);
  background-image: -o-linear-gradient(top, #669bd0, #31689f);
  background-image: linear-gradient(#669bd0, #31689f);
  border-color: #31689f #31689f hsl(210, 53%, 36%);
  /*color: #333 !important;*/
  color:white;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.33);
  -webkit-font-smoothing: antialiased;
  /*border:1px solid rgba(205,205,205, 1);*/
}
.btn-primary:hover{
	color:white;
	
}
.btn-yellow, .btn-warning {
  background-color: hsl(40, 100%, 65%) !important;
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr="#ffdb93", endColorstr="#fec34c");
  background-image: -khtml-gradient(linear, left top, left bottom, from(#ffdb93), to(#fec34c));
  background-image: -moz-linear-gradient(top, #ffdb93, #fec34c);
  background-image: -ms-linear-gradient(top, #ffdb93, #fec34c);
  background-image: -webkit-gradient(linear, left top, left bottom, color-stop(0%, #ffdb93), color-stop(100%, #fec34c));
  background-image: -webkit-linear-gradient(top, #ffdb93, #fec34c);
  background-image: -o-linear-gradient(top, #ffdb93, #fec34c);
  background-image: linear-gradient(#ffdb93, #fec34c);
  border-color: #fec34c #fec34c hsl(40, 100%, 61.5%);
  color: #333 !important;
  text-shadow: 0 1px 1px rgba(255, 255, 255, 0.23);
  -webkit-font-smoothing: antialiased;
}
.btn-bold{
	font-weight:bold;
}
.btn-italics{
	font-style:italics;
}
.btn-underline{
	text-decoration:underline;
}
input:focus:required:invalid:focus, textarea:focus:required:invalid:focus, select:focus:required:invalid:focus {
	border-color: #E9322D;
	border-color: #DC143C;
	-webkit-box-shadow: 0 0 6px #F8B9B7;
	-moz-box-shadow: 0 0 6px #f8b9b7;
	box-shadow: 0 0 6px #F8B9B7;
}
.navbar-inner {
	background: url("/assets/img/bg.png");
	box-shadow: none;
}
.navbar .nav .active > a, .navbar .nav .active > a:hover {
	color: white;
	color:#F7F9FC;
	text-decoration: none;
	background-color: #222;
	background:#006DCC;
	 box-shadow: inset 12px 12px 12px -12px #315B7E;
	-webkit-box-shadow: inset 12px 12px 12px -12px #315B7E;
	-moz-box-shadow: inset 12px 12px 12px -12px #315B7E;
}
.thumbnails > li  {	
	border-radius:10px;
	-moz-border-radius:10px;	
	-moz-box-shadow:inset 0 0 5px #315B7E;
	-webkit-box-shadow:inset 0 0 5px #315B7E;
	box-shadow:inset 0 0 5px #315B7E; 
}
.btn-success{
	background-color: #58A279;
	color:#F7F9FC;
}
.btn-navbar{
	background: #315B7E;
}
.btn-navbar:hover {
	background: #58A279;
}
span.icon-bar{
	color: #FFC14D;
	background: #FFC14D;
}
#tabs, .ui-autocomplete,  {
	background: #F7F9FC;
}
.navbar .nav .active > a, .navbar .nav .active > a:hover {
	background: #ECF6FF;
	color: #ECF6FF;
	border: 1px solid #8EC8FF;
}
.navbar .nav > li > a{
	color:#315B7E;
}
.navbar .nav > li > a:hover{
	color:#58A279;
}
.popover{
	position:absolute;
	z-index:10007;
}
.popover.bottom .arrow {
  	border-bottom: 5px solid #315B7E;
}
.popover-inner,.ui-widget-content  {
	border:1px solid #EDEFF4;
  background:  #F7F9FC;
  /*-webkit-box-shadow: 0 3px 7px  #EDEFF4;
  -moz-box-shadow: 0 3px 7px #EDEFF4;
  box-shadow: 0 3px 7px #EDEFF4;*/
}
.popover-title {
	color: #315B7E;
  	background-color: #F7F9FC;
  	border-bottom: 1px solid #eee;
}
.popover-content {
	color: #315B7E;
  padding: 14px;
  background-color: #F7F9FC;
}
.btn-group {
    display: inline-block;
    vertical-align: middle;   
}
.close{
	float:none;
}
.modal-backdrop{
	background-color:gray;
	background-color:#cccccc;
}
.modal-header{
	border-bottom:1px solid #D2D9E7;
}
.modal-footer{
	border-top:1px solid #D2D9E7;	
}
.well{
	/*background:	#F7F9FC;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;	*/
	-webkit-box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 3px rgba(0, 0, 0, 0.31);
	-moz-box-shadow: 0 0 1px rgba(0,0,0,0.1),0 2px 3px rgba(0,0,0,0.31);
	box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 3px rgba(0, 0, 0, 0.31);
}
.oLightestBlue, .form-actions, .well {
	/*background:#F7F9FC;*/
	background:whiteSmoke;
	 /* github near-white background */
	border-radius: 6px;
	-webkit-border-radius: 6px;
	-moz-border-radius:6px;
}
.lightestBlueBackground{
	background:#F7F9FC;
}
.lightestBlueBackground:hover{
	background: #EBEFF3;
}
.well,.whiteSmokeLighter{
 background: #FAFAFB;
}
.whiteSmoke{
	background:whiteSmoke;
}

.badge-info{
	background:#4183C4;
}

.alert-success {
color: #468847;
background-color: #DFF0D8;
border-color: #D6E9C6;
}
.popover.right .arrow{
	border-right: 5px solid #315b7e;
}
.popover.left .arrow{
 		border-left: 5px solid #315b7e;
 }
.popover.top .arrow{
	border-top: 5px solid #315b7e;
}
.popover.bottom .arrow{
		border-bottom: 5px solid #315b7e;
}


/*.thumbnails > li a {

margin: 0 0 18px 20px;
}*/

.dropdown.open .dropdown-toggle {
color: rgba(0, 0, 0, 0.3);

background: none;
}
input[type="file"]{
	height:4px !important;
}

.popover-inner{
	width:100%;
}


.btn-primary > i, .btn-success > i{
	color:#F7F9FC;
}
/*https://eligibleapi.com/about-us  "jobs button" */
.btn.large {
font-size: 21px;
line-height: 22px;
width: 245px;
height: 22px;
border-radius: 5px;
-moz-border-radius: 5px;
-webkit-border-radius: 5px;
padding: 12px 30px 12px 0;
}

.btn.large::before {
content: '';
display: block;
width: 275px;
height: 46px;
padding: 10px;
position: absolute;
top: -10px;
left: -10px;
z-index: -1;
opacity: .75;
-moz-border-radius: 14px;
-webkit-border-radius: 14px;
border-radius: 14px;
-moz-background-clip: padding;
-webkit-background-clip: padding-box;
background-clip: padding-box;
background-color: rgba(255,255,255,0.28);
-moz-box-shadow: 0 1px 0 rgba(255,255,255,0.5),inset 0 1px 2px rgba(0,0,0,0.42);
-webkit-box-shadow: 0 1px 0 rgba(255,255,255,0.5),inset 0 1px 2px rgba(0,0,0,0.42);
box-shadow: 0 1px 0 rgba(255,255,255,0.5),inset 0 1px 2px rgba(0,0,0,0.42);
}


/* end bootstrap fixes---------------------------------------------------------------------------------------------------*/

/* begin jqueryUI fixes---------------------------------------------------------------------------------------------------*/
.ui-autocomplete-loading {
	background: white url('/assets/thirdParty/jqueryui/development-bundle/demos/autocomplete/images/ui-anim_basic_16x16.gif') right center no-repeat;
	/*  background: white url('/assets/img/video/loading.gif') right center no-repeat;*/
}
.ui-widget{
	font-size:18px;
}
.ui-widget-header{
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;
	border-radius-bottom-left:0px;
	border-radius-bottom-right:0px;
}
.ui-widget-content a {
	color:#315B7E;
}
.ui-tabs .ui-tabs-nav li a, .ui-tabs.ui-tabs-collapsible .ui-tabs-nav li.ui-tabs-selected a {
	color: gray;
}
.ui-tabs .ui-tabs-nav li:hover, .ui-tabs .ui-tabs-nav li a:hover {
	background: #EDEFF4;
}
.ui-tabs-selected i{
	color:#315B7E;
}
.ui-menu .ui-menu-item a.ui-state-hover,
.ui-menu .ui-menu-item a.ui-state-active {
  background:#315B7E; 
}
a.ui-state-hover {
	background: #006DCC;
	color: #F7F9FC;
}

.popover-inner,.ui-widget-content  {
	border:1px solid #EDEFF4;
  background:  #F7F9FC;
  /*-webkit-box-shadow: 0 3px 7px  #EDEFF4;
  -moz-box-shadow: 0 3px 7px #EDEFF4;
  box-shadow: 0 3px 7px #EDEFF4;*/
}

.ui-autocomplete li{
	border-bottom:1px solid #EDEFF4;
}
/* Interaction Cues
----------------------------------*/
.ui-state-highlight, .ui-widget-content .ui-state-highlight, .ui-widget-header .ui-state-highlight  {border: none; background: #FFF4D2; color: #363636; }
.ui-state-highlight a, .ui-widget-content .ui-state-highlight a,.ui-widget-header .ui-state-highlight a { color: #363636; }
.ui-state-error, .ui-widget-content .ui-state-error, .ui-widget-header .ui-state-error {border: none; background: #FFF4D2; color: #cd0a0a; }
.ui-state-error a, .ui-widget-content .ui-state-error a, .ui-widget-header .ui-state-error a { color: #cd0a0a; }
.ui-state-error-text, .ui-widget-content .ui-state-error-text, .ui-widget-header .ui-state-error-text { color: #cd0a0a; }
.ui-priority-primary, .ui-widget-content .ui-priority-primary, .ui-widget-header .ui-priority-primary { font-weight: bold; }
.ui-priority-secondary, .ui-widget-content .ui-priority-secondary,  .ui-widget-header .ui-priority-secondary { opacity: .7; filter:Alpha(Opacity=70); font-weight: normal; }
.ui-state-disabled, .ui-widget-content .ui-state-disabled, .ui-widget-header .ui-state-disabled { opacity: .35; filter:Alpha(Opacity=35); background-image: none; }
				/*end of modal dialog jQuery UI modifications */
input[disabled], textarea[disabled]{
	background:none;
	border-color:#F7F9FC;
}
/* end jqueryUI fixes---------------------------------------------------------------------------------------------------*/

/* begin general---------------------------------------------------------------------------------*/

.windowHash{
  background: #FFF4D2;
  padding:5px;
  border-radius: 2px;
}
.etchedText{
	text-shadow: 0px 1px 0px rgba(255,255,255,.3), 0px -1px 0px rgba(0,0,0,.7);
	/*http://css-snippets.com/engraved-text/*/
}
.alignCenter{
	margin:auto;
	width:50%;
	text-align:center;
}
.bootcamp, .bootcamp0{
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	/*-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;*/	
	-webkit-box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 3px rgba(0, 0, 0, 0.31);
	-moz-box-shadow: 0 0 1px rgba(0,0,0,0.1),0 2px 3px rgba(0,0,0,0.31);
	box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 3px rgba(0, 0, 0, 0.31);
}
.bootcamp h1 {
color: white;

font-weight: bold;
background-color: #405A6A;
background-image: -moz-linear-gradient(#829AA8, #405A6A);
background-image: -webkit-linear-gradient(#829AA8, #405A6A);
background-image: linear-gradient(#829AA8, #405A6A);
background-repeat: repeat-x;
border: 1px solid #677C89;
border-bottom-color: #6B808D;
border-radius: 5px 5px 0 0;
text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.7);
margin: 0;
padding: 8px 10px;
position: relative;
/*font-size: 16px;*/
/*https://github.com/ blue/gray header on homepage; possibly could be used on profile page h1*/
}
/*begin Github CSS*/
.bootcamp .bootcamp-body {
padding: 10px 0px 10px 10px;
background-color: #E9F1F4;
overflow: hidden;
border-style: solid;
border-width: 1px 1px 2px;
border-color: #E9F1F4 #D8DEE2 #D8DEE2;
border-radius: 0 0 5px 5px;

}
/*.pagehead .container > ul.tabs {*/
.pagehead .container0 > ul.tabs, .githubGrayTab { /* got rid of .btn.disabled cause made login button gray colored when data-loading-text'd by Twitter bootstrap button.js  */
position: relative; 
font-weight: bold;
background-color: #EAEAEA;
background-image: -moz-linear-gradient(#FAFAFA, #EAEAEA);
background-image: -webkit-linear-gradient(#FAFAFA, #EAEAEA);
background-image: linear-gradient(#FAFAFA, #EAEAEA);
background-repeat: repeat-x;
border: 1px solid #EAEAEA;
border-bottom-color: #CACACA;
border-radius: 3px;
padding:7px;
/*font-size: 12px;margin: 10px 0 0 0;*/
/*https://github.com/ gray header above blue/gray header on homepage*/
}
/** { -moz-box-sizing: border-box; -webkit-box-sizing: border-box; box-sizing: border-box}*/
.githubGrayTab, .btn.disabled {
	font-weight:normal;
}
.githubGrayTab, .borderBox{
	-moz-box-sizing: border-box; -webkit-box-sizing: border-box; box-sizing: border-box;
}
.pagehead0 .container0 > ul.tabs li {
list-style-type: none;
margin: 0;
display: table-cell;
width: 1%;
}
#footerGithub {
position: relative;
/*margin: 40px 0 0 0;
padding: 20px 0;*/
/*font-size: 12px;
line-height: 1.5;
color: #777;*/
text-shadow: 0 1px 0 white;
background-color: whiteSmoke;
border-top: 1px solid #E5E5E5;
}
.footer-divider {
margin: 0 0 10px;
border-top: 1px solid #E5E5E5;
border-bottom: 1px solid white;
}
.footer-divider-left {
margin: 0 10px 0 0;
border-left: 1px solid #E5E5E5;
border-right: 1px solid white;
width:10px;
height:100%;
}
.rule, hr {
clear: both;
/*margin: 15px 0;*/
height: 0px;
overflow: hidden;
border: none;
background: transparent;
border-bottom: 1px solid #DDD;
}
.footer-divider0 {
margin: 0 0 0px;
border-top: 3px solid #E5E5E5;
border-bottom: 3px solid white;

clear: both;
/*margin: 15px 0;*/
height: 0px;
overflow: hidden;
border: none;
background: transparent;
border-bottom: 3px solid #DDD;
}


.discussion-bubble .discussion-bubble-content::before {
content: "";
display: block;
height: 0;
width: 0;
border: 10px solid #EEE;
border-width: 10px 10px 10px 0;
border-color: transparent #EEE transparent transparent;
/*position: absolute;
left: -10px;
top: 15px;*/
}
.discussion-bubble-inner {
min-height: 60px;
border: 1px solid #CACACA;
background-color: white;
}
.bubble {
padding: 3px;
background: #EEE;
border-radius: 3px;
}
.comment-header {
/*height: 33px;*/
padding: 0 10px 0 6px;
border-bottom: 1px solid #CCC;
background-color: #E1E1E1;
background-image: -moz-linear-gradient(#F8F8F8, #E1E1E1);
background-image: -webkit-linear-gradient(#F8F8F8, #E1E1E1);
background-image: linear-gradient(#F8F8F8, #E1E1E1);
background-repeat: repeat-x;
/*font-size: 12px;*/
}
.comment-header ul li{
	display:inline-block;
}
.comment-content {
padding: 0;
/*color: #333;
font-size: 12px;*/
background: #FBFBFB;
}
.commit.file-history-tease {
margin: 10px 0;
/*padding: 9px 8px 0 8px;
font-size: 14px;*/
padding:9px 8px 8px 8px;
color: #7B878C;
background: #E7ECEE;
border: 1px solid #D2D9DD;
border-radius: 4px;
}
.githubOrange{
	color: #D26911;
}

/*end Github CSS*/


.profileUl{
	position:relative;
	
	/*background:	#F7F9FC;*/
}
/*http://dabblet.com/gist/4182495*/

.blue{background-color:#4A7496;}
.green{background-color:#49796B;}
.grey{background-color:#888}
/*.etched{
	width:50px;
	height:50px;
	border:1px solid rgba(0,0,0,0.2);
	position:relative;
	
	box-shadow:1px 1px 2px -1px #000;
}
.etched:nth-child(n+2){
	border-top:0;
}
.etched:after{
	content:'';
	position:absolute;
	top:0px;
	left:0px;
	z-index:1000;
	width:48px;
	height:48px;		
	border-radius:1px 0 0 1px;
	border:1px solid rgba(255,255,255,0.1);
	border-bottom:0;	
}
.etched:hover:after{
	background-color:rgba(255,255,255,0.1);
}*/
.etchedLine{
	border-top:1px solid rgba(0,0,0,0.2);
	border-bottom:1px solid rgba(255,255,255,0.1);
}
.etchedLine{
	border-bottom:2px solid rgba(0,0,0,0.2);
	border-top:2px solid rgba(255,255,255,0.2);
}

.etched{
		border-top:1px solid rgba(0,0,0,0.2);
	box-shadow:1px 1px 2px -1px #000;

	/*width:50px;
	height:50px;*/

	/*position:relative;*/
	
	
}
.etched:nth-child(n+2){
	border-top:0;
}
.etched:after{
	content:'';

	border-radius:1px 0 0 1px;
	border-bottom:1px solid rgba(255,255,255,0.1);
	border-bottom:0;	
		/*position:absolute;
	top:0px;
	left:0px;
	z-index:1000;
	width:48px;
	height:48px;	*/	
}
.etched:hover:after{
	background-color:rgba(255,255,255,0.1);
}
.inlineList li{
 display:inline;
}
.inlineList.row > [class*="span"]{
  display:inline-block;
}
/*div.inlineList > div {
 display:inline-block;
}*/

/* end general---------------------------------------------------------------------------------*/

/* begin header---------------------------------------------------------------------------------*/
.circleLarge{
	width:100px;
	height:100px;
	display:block;
	border-radius:50px;
	-moz-border-radius:50px;
	-webkit-border-radius:50px;
	-khtml-border-radius:50px;
	font-size:20px;
	color:#fff;
	line-height:100px;
	text-align:center;
	background:#000;
}
.notificationCircle{
	position:absolute;
	top:-5px;
	left:35px;
	width:20px;
	height:20px;
	display:inline-block;
	border-radius:20px;
	-moz-border-radius:20px;
	-webkit-border-radius:20px;
	-khtml-border-radius:20px;
	line-height:20px;
	text-align:center;
	color:#F7F9FC;
	font-size:14px;
	background:#DC143C;
	background:#006DCC;
	background:#4183C4;
	border:2px solid #F7F9FC;
}

.sidebarListItemText{
	color:#F7F9FC;
	font-size:12px;
	text-shadow: 0 1px 0 rgba(255, 255, 255, 0.33);
	text-shadow: 0 1px 0 rgba(76, 96, 112, 0.5);
}
/*.sidebarListItem a > i{
	vertical-align:top;
}*/
/* begin instagram*/

.blog-header, .blog-footer, .blog-footer * {
text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
}
.blog-footer .ig-links, .blog-footer p, .blog-footer a, .blog-footer span{
/*text-transform: uppercase;*/
color: #999;
}
.blog-footer ul, .blog-footer p, .blog-footer a{
text-align: center !important;
}

/*a {
-webkit-tap-highlight-color: rgba(63, 114, 155, 0.3);
}*/

.media-bar { 
	width: 70px;
	height: 610px;
	background-color: #5F8CB0;
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),-webkit-gradient(linear, left center, right center, from(#5F8CB0), to(#4A7496));
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),-webkit-linear-gradient(left, #5F8CB0, #4A7496);
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),-moz-linear-gradient(left, #5F8CB0, #4A7496);
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),-o-linear-gradient(left, #5F8CB0, #4A7496);
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),-ms-linear-gradient(left, #5F8CB0, #4A7496);
	background-image: url(/assets/thirdParty/img/instagram-noise-2.png),linear-gradient(left, #5F8CB0, #4A7496);
	filter: progid:DXImageTransform.Microsoft.gradient(GradientType=1,StartColorStr='#5f8cb0', EndColorStr='#4a7496');
	background-position: 50% 50%;
	position: fixed;
	/*float:left;*/
	z-index:10003;
	/*right: 0;*/
	left:0px;
	top: 0;
	margin: -1px 0 0;
	-webkit-border-radius: 0 4px 4px 0;
	-moz-border-radius: 0 4px 4px 0;
	-khtml-border-radius: 0 4px 4px 0;
	border-radius: 0 4px 4px 0;
	-webkit-box-shadow: inset 0 0 1px rgba(255, 255, 255, 0.4), 0 1px 3px rgba(0, 0, 0, 0.1), -2px 0 2px rgba(0, 0, 0, 0.1);
	-moz-box-shadow: inset 0 0 1px rgba(255,255,255,0.4), 0 1px 3px rgba(0,0,0,0.1), -2px 0 2px rgba(0,0,0,0.1);
	-ms-box-shadow: inset 0 0 1px rgba(255,255,255,0.4), 0 1px 3px rgba(0,0,0,0.1), -2px 0 2px rgba(0,0,0,0.1);
	-o-box-shadow: inset 0 0 1px rgba(255,255,255,0.4), 0 1px 3px rgba(0,0,0,0.1), -2px 0 2px rgba(0,0,0,0.1);
	box-shadow: inset 0 0 1px rgba(255, 255, 255, 0.4), 0 1px 3px rgba(0, 0, 0, 0.1), -2px 0 2px rgba(0, 0, 0, 0.1);
	border: 1px solid #1C5380;
}
.media-bar ul {
	position: relative;
	min-height: 102px;
	padding: 0 0 1px;
	list-style: none;
	border-bottom: 1px solid rgba(255, 255, 255, 0.12);

}
.can-open-media .media-open .media-bar .like-control {
	height: 50px;
	-webkit-transition-delay: 0.1s;
	-moz-transition-delay: 0.1s;
	-o-transition-delay: 0.1s;
	transition-delay: 0.1s;
}
.media-bar ul li {
	-webkit-transition: height 0.2s ease-out;
	-moz-transition: height 0.2s ease-out;
	-o-transition: height 0.2s ease-out;
	transition: height 0.2s ease-out;
	-webkit-transition-delay: 0s;
	-moz-transition-delay: 0s;
	-o-transition-delay: 0s;
	transition-delay: 0s;

-moz-transition-property: background-color;
-webkit-transition-property: background-color;
-o-transition-property: background-color;
transition-property: background-color;
-moz-transition-duration: .4s;
-webkit-transition-duration: .4s;
-o-transition-duration: .4s;
transition-duration: .4s;
}
.media-bar ul li {
	position: relative;
	text-align: center;
	margin: 0 0 1px;
	z-index:10003;
	width:100%;
	padding-top:5px;
	padding-bottom:5px;
	/*margin: 5px 0 5px 0;*/
	/*height: 60px;*/
}
.media-bar ul li a{
	margin: 0px; display: block; width: 100%; height: 100%;
}
.media-bar ul li a i, .media-bar ul li i{
	padding-top:10px;
	
	
}
.media-bar ul li:hover{
	background-color: rgba(255, 255, 255, 0.1);
}
.glowingBackground {
	background-color: rgba(255, 255, 255, 0.1);
}
/* end instagram*/
#searchForm{
	display:inline-block;
}
#searchDivider{
	background:url('/assets/img/bg.png');
	position:relative; z-index:2;
	height:3000px;
}
.sidebarListItem i.txt32{
	color:lightSteelblue; /*rgb(176, 196, 222);*/
	text-shadow: 0 1px 0 rgba(225, 225, 225, 0.1);
	text-shadow: 0 1px 0 rgba(76, 96, 112, 0.5); 

}
.sidebarListItem .icon-cog, .sidebarListItem .icon-reorder{
	color:#315B7E;
	text-shadow: 0 1px 0 rgba(225, 225, 225, 0.1);
	text-shadow: 0 1px 0 rgba(76, 96, 112, 0.5);
	font-size:20px; 
}
#notificationsDropdownList{
	overflow-y:scroll;
}
#notificationsDropdownList li{
	text-align:left;
	padding:0px;
	
}
#notificationsDropdownList li i{
	font-size:12px;
	color:lightGrey;
}
#notificationsDropdownList hr{
	margin:0px;
}

#sidebarListItemNotifications .popover-title {
padding: 4px 10px;
padding:0px;
}
#sidebarListItemNotifications .popover-content{
padding:0px;
}
#sidebarListItemNotifications .popover-content li{
text-align:left;
}

#sidebarListItemNotifications .popover-content li a{
font-size:15px;
}

#sidebarListItemSettings .popover-inner, 
#sidebarListItemSettings .popover-content, 
#sidebarListItemSettings .popover-title{
	background:#555;	
	border:none;
}

#sidebarListItemSettings .popover-title{
	padding:0px;
}

#sidebarListItemSettings .popover-content ul li a,
#sidebarListItemSettings .popover-content ul li i{
	color:#F7F9FC !important;
}

.notificationsForm i.ajaxFormBtn{
 font-size:24px;
}
/*#settingsDropdown, #notificationsDropdown {	
	background:#555;
	-webkit-border-radius: 5px;
	-moz-border-radius:5px;
	border-radius:5px;
	min-width:0px;
	/*position:absolute;
	left: 75px;
	top: -20px;*/
/*}
#settingsDropdown li,  #notificationsDropdown li{
	text-align: left;
}
#settingsDropdown li a, #settingsDropdown li i,  #notificationsDropdown li a {
	color:#F7F9FC;
}
#settingsDropdown li a:hover, #notificationsDropdown li a:hover {
	text-decoration:underline;
}*/
#notifyByEmailUl li{
	padding:10px;
	margin:10px;
}
.highlightMe{
	-webkit-box-shadow: inset 0 1px 7px  #315B7E;
	-moz-box-shadow: inset 0 1px 7px #315B7E;
	box-shadow: inset 0 1px 7px #315B7E;
	border-top: 1px solid #1C5380;
	border-bottom: 1px solid #1C5380;
}
.left70{
	left:70px;
}
/*input[type="text"], input[type="password"], .inputLike {
	height:36px;
	font-size:20px;	
}*/
.inputLike {
	display: inline-block;
	padding: 4px;
	line-height: 18px;
	color: 
	gray;
	border: 1px solid 
	#CCC;
	-webkit-border-radius: 3px;
	-moz-border-radius: 3px;
	border-radius: 3px;
	background:white;
	-webkit-box-shadow: inset 0 1px 1px 
	rgba(0, 0, 0, 0.075);
	-moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	box-shadow: inset 0 1px 1px 
	rgba(0, 0, 0, 0.075);
	-webkit-transition: border linear 0.2s, box-shadow linear 0.2s;
	-moz-transition: border linear 0.2s, box-shadow linear 0.2s;
	-ms-transition: border linear 0.2s, box-shadow linear 0.2s;
	-o-transition: border linear 0.2s, box-shadow linear 0.2s;
	transition: border linear 0.2s, box-shadow linear 0.2s;
}
#iconReorder{
	position:relative;z-index:3;
	padding:5px 0 5px 0;	
}
#showNavigation{
	top:20px;
	left:20px;
	/*position: fixed;
	z-index: 2;*/
}
#loggedOutUl li{
	display:inline-block;
}
/*#iconSearch{
	position:absolute;position: absolute;top: 8px;left: 20px;
}*/
input#search_query{
	/*webkit-border-radius:14px;
	border-radius:14px;
	-moz-border-radius:14px;
	height:24px;
	width:50px !important;*/
	
	/*width:150px !important;
	display:none;
	/*background:whiteSmoke;*/	
	/* http://stackoverflow.com/questions/7062383/putting-a-image-inside-a-input-box
	background: url(images/comment-author.gif) no-repeat scroll right;
	padding-right: 30px;*/
 
}
/*#search_query:focus{
	background:white;
}*/

/* end header --------------------------------------------------------------------------------------------*/

/* begin bank MVC------------------------------------------------------------------------------------------*/
.yMax{
	margin-bottom:0px !important;
}

#barChart  {
height:32px;
width:145px;
border-bottom: 1px solid gray;
border-left: 1px solid #D2D9E7;
background: #EDEFF4;
border-bottom: 1px solid #D2D9E7;
-moz-border-radius:2px;
border-radius:2px;
padding:2px;
}
#barChart > canvas{
	-moz-border-radius:2px;
	border-radius:2px;


	/*
  width:50%;
  border-bottom: 1px solid gray;
	border-left: 1px solid #D2D9E7;
	
	border-bottom:1px solid #D2D9E7;*/
	background:#EDEFF4;
}
.moreProfileLink:hover{
	text-decoration:none;
}
/* end bank MVC--------------------------------------------------------------------------------------------*/

/* begin media MVC------------------------------------------------------------------------------------------*/
#priceInput{
	font-size: 48px;
	height: 60px;
	width:75%;
}
/* end media MVC-------------------------------------------------------------------------------------------------*/

/* begin search MVC----------------------------------------------------------------------------------------------*/

/* end search MVC-------------------------------------------------------------------------------------------------*/

/* begin messages MVC---------------------------------------------------------------------------------------------------*/
/*	width:90%;
	height:200px;
}*/

#tabs{
	background:	#F7F9FC;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;
}	
.oBtn {
	background: #ECF6FF;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	border: 1px solid #8EC8FF;
	-moz-box-shadow: 0 0 1px #8EC8FF;
	-webkit-box-shadow: 0 0 1px #8EC8FF;
	box-shadow: 0 0 1px #8EC8FF;	
}
.oBtn:hover{
	-moz-box-shadow: 0 0 1px #315B7E;
	-webkit-box-shadow: 0 0 1px #315B7E;
	box-shadow: 0 0 1px #315B7E;
}
.oBtn:active{
	-moz-box-shadow: inset 0 0 1px #315B7E;
	-webkit-box-shadow: inset 0 0 1px #315B7E;
	box-shadow: inset 0 0 1px #315B7E;
}
/* end messages MVC---------------------------------------------------------------------------------*/

/* begin account MVC---------------------------------------------------------------------------------*/
#accountModal > div{
	display:inline-block;
}
#loginBtnLink{
	border-radius:25px;
	-webkit-border-radius:25px;
	-moz-border-radius:25px;	
}
#loginBtnLink:hover, #signupBtnLink:hover, .localBtn:hover, .uploadBtn:hover{
	background:#f5f5f5;	
	border: 1px solid #006DCC;
	-moz-box-shadow: 0 0 1px #006DCC;
	-webkit-box-shadow: 0 0 1px #006DCC;
	box-shadow: 0 0 1px #006DCC;
	color:#333;	
}
#loginBtnLink:active, #signupBtnLink:active, .localBtn:active, .uploadBtn:active{
	background:#F7F9FC;		
}
.localBtn, .uploadBtn{
	overflow: hidden;
	height: 100%;
}
/*.signup_login{
	padding:9px;
	border-radius:10px;
	-webkit-border-radius: 10px;
	-moz-border-radius: 10px;
	-moz-box-shadow: 0 0 5px #315B7E;
	-webkit-box-shadow: 0 0 5px #315B7E;
	box-shadow: 0 0 5px #315B7E;
}*/
/* end account MVC---------------------------------------------------------------------------------*/

/* begin media MVC---------------------------------------------------------------------------------*/
.item {
	height: 35px;
	padding: 0;
	border-top: 1px solid 
	#3C3C3C;
	border-bottom: 1px solid 
	black;
	cursor: pointer;
	background-color: 
	#2D2D2D;
	-moz-transition-property: background-color;
	-webkit-transition-property: background-color;
	-o-transition-property: background-color;
	transition-property: background-color;
	-moz-transition-duration: .4s;
	-webkit-transition-duration: .4s;
	-o-transition-duration: .4s;
	transition-duration: .4s;
}
.item:hover {
	background-color: 
	#373737;
	-moz-transition-property: background-color;
	-webkit-transition-property: background-color;
	-o-transition-property: background-color;
	transition-property: background-color;
	-moz-transition-duration: .1s;
	-webkit-transition-duration: .1s;
	-o-transition-duration: .1s;
	transition-duration: .1s;
}
#moneyraised.NS-projects-ecom #banner {
	text-transform: uppercase;
	letter-spacing: 1px;
	color: 
	white;
	background: 
	gray;
	margin: 20px -30px;
	padding: 10px 30px 20px 30px;
	font-size: 12px;
	line-height: 16px;
	background-image: url("http://d297h9he240fqh.cloudfront.net/cache-a3fa9508e/images/projects/banner-wings.gif");
	background-position: bottom;
	background-repeat: no-repeat;
}
#moneyraised.NS-projects-ecom {	
	width:100%;
	background-color: 
	#E7E7E7;
	padding: 1em 20px 0 20px;
	border-bottom: 1px solid 
	#CCC;
	position: relative;
}
/* end media MVC---------------------------------------------------------------------------------*/

/* begin library MVC---------------------------------------------------------------------------------*/
#libraryNavbar{
	/*background:#FFF4D2;
	color:#315B7E;
	border-top:1px solid #D2D9E7;
	border-top:1px solid #FFC14D;
	border-bottom:1px solid #D2D9E7;*/

	/*height: 24px;
	line-height: 9px;*/		
}
#libraryListRow i{
  text-shadow: 0 1px 0 rgba(76, 96, 112, 0.5);

}
#libraryNavbar ul li{
	display:inline-block;
}
/*#libraryNavbar {
	position: fixed;
	z-index:1;
	width:77%;
}*/
.libraryTable{
	width:100%;
	
}
.navbar{
	margin-bottom:0px;
}
/*#sortSubject, #sortPeople{
	width:40%;
	display:inline-block;
}
#sortDate{
	width:15%;
	display:inline-block;
}*/
div.subject, div.people, div.date{
	/*display:inline-block;
	height:35px;
	margin:1px 0 1px 0;*/
}
div.subject, div.people{
	/*width:40%	*/
}
div.date{
	/*width:10%*/
}
.libraryTable{
	position: relative;
	top:-2px;
}
ul.libraryTable, ul.albumTable {
	background:#F7F9FC;
	/*color:#C0E0F6;*/
}
.helpRow{
	margin:10px; padding:20px;
}
.breadcrumbRow{
	margin:10px;
}
.libraryRow, .messageRow, .bankRow, .notificationRow{
	padding:5px;
}
.oLight-er{
	background:#ECF6FF;
	border-right: 1px solid #8EC8FF;
	-webkit-border-top-left-radius: 5px;
	-moz-border-top-left-radius:5px;
	-webkit-border-bottom-left-radius: 5px;
	-moz-border-bottom-left-radius:5px;
}
.notificationRow, .helpRow, 
.breadcrumbRow, .reviewRow, 
.messageRow, .bankRow{	
	border-bottom:2px solid #EDF1F5;
	

	
	padding:10px;
}
.folderRow{
	padding:10px;
}
.libraryRow{
	box-shadow: 0 1px 1px -1px #EDF1F5;
		
		margin-bottom:10px;
		margin-left:5px;
		margin-right:5px;
		/*border-bottom:2px solid rgba(0,0,0,0.1);*/
	/*margin:10px;
	margin:0px 10px;*/
}
.libraryRow:hover{
	
	background:#EDEFF4;
}
#libraryListRow .span1 div{
margin:auto;
width:90%;
}
.topRow{	

	border-top:1px solid #EDF1F5;
	margin:5px;
	padding:5px;
}

.bottomRow{	
	box-shadow: 0 4px 2px -2px white;
	border-bottom:1px solid #EDF1F5;
	margin:5px;
	padding:5px;
}

.borderBottom{	

	border-bottom:1px solid #EDF1F5;

}

.reviewRow:hover, .notificationRow:hover, .messageRow:hover{
	border-bottom:	1px solid #EDEFF4;
	background:#EDEFF4;
}
.hilite{
	 box-shadow: inset 5px 5px 5px -5px #315B7E;
	-webkit-box-shadow: inset 5px 5px 5px -5px #315B7E;
	-moz-box-shadow: inset 5px 5px 5px -5px #315B7E;
	background:#EDEFF4;
}
.hilite a{
	color: #83B8DC;
}
.hilite a:hover{
	color:#315B7E;
}
/* end library MVC----------------------------------------------------------------------------------------------------*/

/* begin bank MVC----------------------------------------------------------------------------------------------------*/
#bankContainer{
	height:100%;
}
#moneyVsRepTabs ul li{
	display:inline;
}
.overflowScroll{
	overflow:scroll;
}
#detail-graph{
	height: 300px;
	margin-bottom:20px;
}
#detail-container{
	border-top-left-radius:5px;
	border-top-right-radius:5px;	
}
#master-graph{
	height: 125px;
	margin-bottom:5px;
}
#master-container{	
	border-bottom-left-radius:0px;
	border-bottom-right-radius:0px;
}
#master-container, #detail-container{
	background:white;
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;
}
#graph-help{
	text-align:center; font-size: 14px;
}
.day-details{
	margin-top:0px;
}
#bankSummaryDiv{
	background:#F7F9FC;
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;
	border-top-left-radius:5px;
	border-top-right-radius:5px;
	margin-bottom:20px;
	margin-top:5px;
	padding:15px;
}
#bankSummaryDiv div{
	border-bottom:	1px solid #EDEFF4;
}
/* end bank MVC----------------------------------------------------------------------------------------------------*/

/* begin collaborators MVC--------------------------------------------------------------------------------------------*/
.collabVert, .collabList{
	list-style:none;
	padding:10px;
	margin:10px;
	background: #F7F9FC;
	border:1px solid #ECF6FF;
	border-radius:5px;
	-moz-border-radius:5px;
}
.collabHorz{
	float:left;
	list-style:none;
	padding:10px;
	margin:10px;
	background: #F7F9FC;
	border:1px solid #ECF6FF;
	border-radius:5px;
	-moz-border-radius:5px;
}
/* end collaborators MVC-----------------------------------------------------------------------------------------*/

/* begin reviews MVC---------------------------------------------------------------------------------------------*/

.comments p {
	margin:10px;
	padding:10px;
}
.characterTextarea500{
	width:100%;

}
#reviewsContainer{
	background:#ECF6FF;
	border-right: 1px solid #8EC8FF;
	-webkit-border-top-left-radius: 5px;
	-moz-border-top-left-radius:5px;
	-webkit-border-bottom-left-radius: 5px;
	-moz-border-bottom-left-radius:5px;
}
.flipHorizontal {
	-moz-transform: scaleX(-1);
	-o-transform: scaleX(-1);
	-webkit-transform: scaleX(-1);
	transform: scaleX(-1);
	filter: FlipH;
	-ms-filter: "FlipH";
}
.rotate7-5 {
	-moz-transform: rotate(7.5deg);  /* FF3.5+ */
	-o-transform: rotate(7.5deg);  /* Opera 10.5 */
	-webkit-transform: rotate(7.5deg);  /* Saf3.1+, Chrome */
	filter:  progid:DXImageTransform.Microsoft.BasicImage(rotation=0.083);  /* IE6,IE7 */
	-ms-filter: "progid:DXImageTransform.Microsoft.BasicImage(rotation=0.083)"; /* IE8 */
}
#fbGrayList{
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;	
	-moz-box-shadow: 0 0 5px #8B4513; /*brown*/
	-webkit-box-shadow:  0 0 5px #8B4513;
	box-shadow:  0 0 5px #8B4513;	
}

.fbGray{
	background:#EDEFF4;
	border-bottom:1px solid #D2D9E7;

	margin-top:1px;	
	display:block;
	overflow: hidden;
	width: 100%;

	/*	-moz-box-shadow: 0 0 1px #8B4513; /*brown*/
	/*-webkit-box-shadow:  0 0px 1px #8B4513;
	box-shadow:  0 0px 1px #8B4513;*/
}
div.fbGray{
		margin:auto;
	width:95%;	
}
.transition4s{
	-moz-transition-property: background-color;
-webkit-transition-property: background-color;
-o-transition-property: background-color;
transition-property: background-color;
-moz-transition-duration: .4s;
-webkit-transition-duration: .4s;
-o-transition-duration: .4s;
transition-duration: .4s;
}
/*begin Stripe */

#login div#main-body, .page-wrapper div#main-body, .well, .oLightestBlue, .popover-inner {
/*background: none;*/
-webkit-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 2px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-moz-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 2px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-ms-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 2px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-o-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 2px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 2px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);

/*-webkit-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 5px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-moz-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 5px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-ms-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 5px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
-o-box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 5px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);
box-shadow: 0 0 0 1px rgba(14,41,57,0.12),0 2px 5px rgba(14,41,57,0.44),inset 0 -1px 2px rgba(14,41,57,0.15);*/

/*background: #f2f6f9;
background: -webkit-linear-gradient(#fff, #f2f6f9);
background: -moz-linear-gradient(#fff, #f2f6f9);
background: -ms-linear-gradient(#fff, #f2f6f9);
background: -o-linear-gradient(#fff, #f2f6f9);
background: linear-gradient(#fff, #f2f6f9);*/
-webkit-border-radius: 6px;
-moz-border-radius: 6px;
-ms-border-radius: 6px;
-o-border-radius: 6px;
border-radius: 6px;
}

#login div#footer a, .page-wrapper div#footer a, .publishLink {
display: block;
border: 1px solid rgba(14,41,57,0.1);
-webkit-box-shadow: inset 0 1px 0 rgba(255,255,255,0.5),0 1px 0 rgba(255,255,255,0.5);
-moz-box-shadow: inset 0 1px 0 rgba(255,255,255,0.5),0 1px 0 rgba(255,255,255,0.5);
-ms-box-shadow: inset 0 1px 0 rgba(255,255,255,0.5),0 1px 0 rgba(255,255,255,0.5);
-o-box-shadow: inset 0 1px 0 rgba(255,255,255,0.5),0 1px 0 rgba(255,255,255,0.5);
box-shadow: inset 0 1px 0 rgba(255,255,255,0.5),0 1px 0 rgba(255,255,255,0.5);
text-align: center;
font-size: 13px;
padding: 15px;
-webkit-border-radius: 6px;
-moz-border-radius: 6px;
-ms-border-radius: 6px;
-o-border-radius: 6px;
border-radius: 6px;
font-weight: bold;
color: #455b67;
}
div#footer a {
color: #aaa;
text-decoration: none;
}
/*.app .payment input, .stripeInput, input, input[type="text"], input[type="password"], textarea {
-webkit-box-sizing: border-box;
-moz-box-sizing: border-box;
box-sizing: border-box;
width: 100%;
-moz-user-select: text;
-khtml-user-select: text;
-webkit-user-select: text;
-ms-user-select: text;
-o-user-select: text;
user-select: text;
/*font-size: 15px;*/
/*padding: 9px 7px;
border: 1px solid #B4BECD;
border-top-color: #B0BACA;
border-bottom-color: #D3D9E2;
-moz-box-shadow: inset 0 1px 2px rgba(130,137,150,0.23), 0 1px 0 rgba(255,255,255,0.85);
-webkit-box-shadow: inset 0 1px 2px rgba(130, 137, 150, 0.23), 0 1px 0 rgba(255, 255, 255, 0.85);
box-shadow: inset 0 1px 2px rgba(130, 137, 150, 0.23), 0 1px 0 rgba(255, 255, 255, 0.85);
-moz-border-radius: 6px;
-webkit-border-radius: 6px;
border-radius: 6px;
/*color: #333;*/
/*color:#555;
margin: 0;
-webkit-font-smoothing: subpixel-antialiased;
font-family: 'Helvetica Neue', Helvetica, Arial Geneva, sans-serif;

-webkit-tap-highlight-color: transparent;
-webkit-touch-callout: none;
}

.app .payment input:focus, 
.stripeInput:focus, 
input:focus,
input[type="text"]:focus, 
input[type="password"]:focus, 
textarea:focus{
border-color: #70A7E4;
outline: none;
-moz-box-shadow: inset 0 1px 2px rgba(70,123,181,0.35), 0 0 4px #5695db;
-webkit-box-shadow: inset 0 1px 2px rgba(70, 123, 181, 0.35), 0 0 4px #5695DB;
box-shadow: inset 0 1px 2px rgba(70, 123, 181, 0.35), 0 0 4px #5695DB;
}*/


.app .payment input, .stripeInput, input, input[type="text"], input[type="password"], textarea {
-webkit-box-sizing: border-box;
-moz-box-sizing: border-box;
box-sizing: border-box;
-moz-user-select: text;
-khtml-user-select: text;
-webkit-user-select: text;
-ms-user-select: text;
-o-user-select: text;
user-select: text;
/*font-size: 15px;*/
/*padding: 9px 7px;*/
border: 1px solid #B4BECD;
border-top-color: #B0BACA;
border-bottom-color: #D3D9E2;
-moz-box-shadow: inset 0 1px 2px rgba(130,137,150,0.23), 0 1px 0 rgba(255,255,255,0.85);
-webkit-box-shadow: inset 0 1px 2px rgba(130, 137, 150, 0.23), 0 1px 0 rgba(255, 255, 255, 0.85);
box-shadow: inset 0 1px 2px rgba(130, 137, 150, 0.23), 0 1px 0 rgba(255, 255, 255, 0.85);
-moz-border-radius: 6px;
-webkit-border-radius: 6px;
border-radius: 6px;
/*color: #333;*/
color:#555;
/*margin: 0;*/
-webkit-font-smoothing: subpixel-antialiased;
font-family: 'Helvetica Neue', Helvetica, Arial Geneva, sans-serif;

-webkit-tap-highlight-color: transparent;
-webkit-touch-callout: none;
}

.app .payment input:focus, 
.stripeInput:focus, 
input:focus,
input[type="text"]:focus, 
input[type="password"]:focus, 
textarea:focus{
border-color: #70A7E4;
outline: none;
-moz-box-shadow: inset 0 1px 2px rgba(70,123,181,0.35), 0 0 4px #5695db;
-webkit-box-shadow: inset 0 1px 2px rgba(70, 123, 181, 0.35), 0 0 4px #5695DB;
box-shadow: inset 0 1px 2px rgba(70, 123, 181, 0.35), 0 0 4px #5695DB;
}




#doc > nav h1::before {
content: "";
position: absolute;
width: 12px;
height: 14px;
margin-left: -18px;
background: url(/img/documentation/nav-icons.png);
background-size: 48px 14px;
}

.grayBorder{
	border: 1px solid #CCC;
	background:whiteSmoke;
	border-radius:6px;
	-webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	-moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
}

.search_input_div {
	height:28px;
	width: 160px;
	-webkit-transition: 0.2s linear;
	-moz-transition: 0.2s linear;
	-o-transition: 0.2s linear;
	-ms-transition: 0.2s linear;
	transition: 0.2s linear;
	width: 60px;
	margin: 2px;
	padding: 2px;
	color: gray;
	background-color: whiteSmoke;
	border-radius: 6px;
	border:1px solid lightGrey;

	border: 1px solid #CCC;
	background:whiteSmoke;
	border-radius:6px;
	-webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	-moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
	box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);

}
.search_input_div i{
	padding-top: 7px;
	float:left;
}
.search_input_div input {
	height: 22px !important;
	border:none !important;
	box-shadow:none !important;
}
.searchDivHighlight{
	width:180px;
	background:white;
	border-color: rgba(65, 131, 196, 0.5); /*Github .ajaxLink color*/
	-webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
	-moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
	box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075), 0 0 6px rgba(65, 131, 196, 0.4);
	outline: 0;
	outline: thin dotted \9;
}

.search_input{
	line-height:24px;
	font-size:18px;
	color:#666;
	background:whitesMoke;
	outline:none;
	width:20px;
	border:none;
	-webkit-transition: 0.2s linear;
	-moz-transition: 0.2s linear;
	-o-transition: 0.2s linear;
	-ms-transition: 0.2s linear;
	transition: 0.2s linear;
}

.searchInputHighlight{
	border: none !important;
	background:white !important;
	box-shadow:none !important;

	width: 150px;
}


div#answers div.main-content div#content-header, div#content-header, .content-header {

background: #FAFCFF;
border-bottom: 1px solid #E0E7EE;
-moz-border-top-left-radius: 4px;
-webkit-border-top-left-radius: 4px;
border-top-left-radius: 4px;
-moz-border-top-right-radius: 4px;
-webkit-border-top-right-radius: 4px;
border-top-right-radius: 4px;
box-sizing:border-box;
padding:16px;
}
.content-header input{
	background: white url(/assets/thirdParty/stripe/glyph.png) no-repeat;
	background-size: 18px 16px;
background-position: 11px 50%;
}

div#answers div.main-content div.questions-list div.question a:visited, .hashable:visited {
color: #7F87C2;
}

div.sidebar-view ul li.selected a,
.ui-state-active, .ui-widget-content .ui-state-active, .ui-widget-header .ui-state-active {
background: #E7EDF3;
border-top-color: #BED3EA;
border-bottom-color: #BED3EA;
border:1px solid #BED3EA;
}

.ui-tabs .ui-tabs-nav li:hover, .ui-tabs .ui-tabs-nav li a:hover {
background: #EDEFF4;

}

.ui-state-default, .ui-widget-content .ui-state-default, .ui-widget-header .ui-state-default {
border: 1px solid rgba(0,0,0,0.3);
font-weight: bold;
}

/*.ui-state-active, .ui-widget-content .ui-state-active, .ui-widget-header .ui-state-active {
border: 1px solid whiteSmoke;
font-weight: bold;
}*/

div.sidebar-view ul li a {
position: relative;
display: block;
height: 20px;
line-height: 12px;
/*font-size: 11px;*/
border-top: 1px solid #F1F3F5;
border-bottom: 1px solid #F1F3F5;
/*color: #555;*/
padding: 10px 0 0 20px;
text-decoration: none;
text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
}


div.frontpage-notification, ul.frontpage-notification, li.frontpage-notification{
background: #E6EAED;
background: rgba(49, 77, 94, 0.05);
border: 1px solid #CDD0D3;
border-top-color: #BFC1C4;
text-align: center;
-webkit-border-radius: 4px;
-moz-border-radius: 4px;
-ms-border-radius: 4px;
-o-border-radius: 4px;
border-radius: 4px;
-webkit-box-shadow: inset 0 1px 1px rgba(49, 77, 94, 0.06),0 1px 0 rgba(255, 255, 255, 0.33);
-moz-box-shadow: inset 0 1px 1px rgba(49,77,94,0.06),0 1px 0 rgba(255,255,255,0.33);
-ms-box-shadow: inset 0 1px 1px rgba(49,77,94,0.06),0 1px 0 rgba(255,255,255,0.33);
-o-box-shadow: inset 0 1px 1px rgba(49,77,94,0.06),0 1px 0 rgba(255,255,255,0.33);
box-shadow: inset 0 1px 1px rgba(49, 77, 94, 0.06),0 1px 0 rgba(255, 255, 255, 0.33);
/*width: 940px;
margin: 20px auto 2px;
height: 43px;
position: absolute;
z-index: 300;
left: 50%;
margin-left: -470px;
margin-top: -65px;*/
}

div.frontpage-notification a.animation, div.frontpage-notification iframe, li.frontpage-notification i  {
-webkit-transition: -webkit-transform 400ms cubic-bezier(0.33, 1.66, 0.66, 0.99),opacity 400ms cubic-bezier(0.33, 1.66, 0.66, 0.99),background 150ms ease;
-moz-transition: opacity 400ms ease,background 150ms ease;
}
div.frontpage-notification a, div.frontpage-notification iframe,li.frontpage-notification i {
/*width: 940px;
position: absolute;
left: 0;*/
background: rgba(49, 77, 94, 0);
display: block;
/*padding: 13px 0;*/
/*color: #61737C;
/*font-size: 14px;*/
-webkit-font-smoothing: antialiased;
text-decoration: none;
-webkit-border-radius: 4px;
-moz-border-radius: 4px;
-ms-border-radius: 4px;
-o-border-radius: 4px;
border-radius: 4px;
text-shadow: 0 1px 0 rgba(255, 255, 255, 0.33);
opacity: 1.0;
-webkit-transform: scale(1);
}

div.frontpage-notification a:hover {
background: rgba(49, 77, 94, 0.03);
}

div.toolbar-view div.switch-view label, .stripeTextShadow * {
/*position: absolute;
top: -1px;
font-size: 10px;*/
text-transform: uppercase;
font-weight: bold;
/*color: #A1C3E9;*/
-webkit-transition: color 0.2s ease-in-out;
text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.35);
}

.stripeBackground{ /* stripe background*/
background: rgba(255, 255, 255, 0);
background: -webkit-linear-gradient(rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
background: -moz-linear-gradient(rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
background: -ms-linear-gradient(rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
background: -o-linear-gradient(rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
background: linear-gradient(rgba(255, 255, 255, 0.3), rgba(255, 255, 255, 0));
background-color: #E8EBEF;
background-repeat: no-repeat;
}

div.dashboard-view div.summary, div.platform-overview-dashboard-view div.summary, .stripeBlueBackground {
/*font-family: "proxima-nova", arial, sans-serif;
position: relative;
padding: 25px 30px;*/
background: #3E4859;
background: -webkit-linear-gradient(#233143, #3E4859);
background: -moz-linear-gradient(#233143, #3E4859);
background: -ms-linear-gradient(#233143, #3E4859);
background: -o-linear-gradient(#233143, #3E4859);
background: linear-gradient(#233143, #3E4859);
-webkit-box-shadow: inset 0 1px 0 rgba(0, 0, 0, 0.27);
-moz-box-shadow: inset 0 1px 0 rgba(0,0,0,0.27);
-ms-box-shadow: inset 0 1px 0 rgba(0,0,0,0.27);
-o-box-shadow: inset 0 1px 0 rgba(0,0,0,0.27);
box-shadow: inset 0 1px 0 rgba(0, 0, 0, 0.27);
background-color: #233143;
/*left: -1px;
width: 700px;*/
-webkit-border-bottom-left-radius: 1px;
-moz-border-bottom-left-radius: 1px;
-ms-border-bottom-left-radius: 1px;
-o-border-bottom-left-radius: 1px;
border-bottom-left-radius: 1px;
-webkit-border-bottom-right-radius: 1px;
-moz-border-bottom-right-radius: 1px;
-ms-border-bottom-right-radius: 1px;
-o-border-bottom-right-radius: 1px;
border-bottom-right-radius: 1px;
border: 1px solid rgba(0, 0, 0, 0.15);
border-top: 0;
-webkit-font-smoothing: antialiased;
}

div.dashboard-view div.block, 
div.dashboard-view div.chart-view, 
div.platform-overview-dashboard-view div.block, 
div.platform-overview-dashboard-view div.chart-view {
border: 1px solid #DADADA;
border-top: 1px solid #BBB;
background: white;
/*padding: 20px 20px;
margin-bottom: 30px;*/
margin:20px;

-webkit-border-radius: 6px;
-moz-border-radius: 6px;
-ms-border-radius: 6px;
-o-border-radius: 6px;
border-radius: 6px;
-webkit-box-shadow: inset 0 1px 0 #EDEDED;
-moz-box-shadow: inset 0 1px 0 #ededed;
-ms-box-shadow: inset 0 1px 0 #ededed;
-o-box-shadow: inset 0 1px 0 #ededed;
box-shadow: inset 0 1px 0 #EDEDED;
}

html#blog {
/*background: #e8ebef url(/img/blog/background.png);*/
background: #e8ebef url(/assets/thirdParty/stripe/blog_background.png);
background-size: 20px 20px;
}

/*end Stripe*/
.fbGrayWO{
	background:#EDEFF4;	
	padding:5px 5px 4px;
	margin-top:1px;	
	display:block;
	overflow: hidden;
	width: 100%;
}
#reviewWell{

	border-width:1px;	
	border-style:dashed;
	border-color:#cccccc;
	padding:5px 0 5px 0;
	width:97.8%;
	border-radius:5px;
}

#profileContainer{

	border-width:3px;	
	border-style:dashed;
	border-color:#cccccc;
	padding:5px 0 5px 0;
	width:97.8%;
}
#profileFormDiv{
	clear:both;
}
#profileFormDiv .imageFormDiv{

display:inline-block;

}
#profileFormDiv div.btn{
	width:55%;
}
#profileReviewsTarget li{
	
}
#imageFormDiv{
	margin:auto;
	width:90%;
}
#imageFormDiv div{
 display:inline-block;
}

#imageFormDiv > div{
	text-align:center;
}
#imageFormDiv > .footer-divider{
	width:90%;
}
#imageFormDiv img{
	width:100px;
	height:100px;
}
.imageFormParentDiv{
	width:45%;
	vertical-align:bottom;

}
#folder_list .frontpage-notification{
	display:inline-block;
}

#sidebarListItemNotifications .popover{
	top:-40px !important;
}
/* end reviews MVC--------------------------------------------------------------------------------*/

/* begin mall mvc---------------------------------------------------------------------------------*/
.fullWidthBtn{
	width:100%;
	/*height:50px;
	font-size:36px;
	border:none;
	border-radius: 10px;
	-moz-border-radius: 10px;
	-moz-box-shadow: inset 0 0 5px #315B7E;
	-webkit-box-shadow: inset 0 0 5px #315B7E;
	box-shadow: inset 0 0 5px #315B7E;
	background: #F7F9FC;*/
}
/*.fullWidthBtn:active{
	-moz-box-shadow: inset 0 0 7px #315B7E;
	-webkit-box-shadow: inset 0 0 7px #315B7E;
	box-shadow:  inset 0 0 7px #315B7E;	
}*/
.faq_answer, .hashable, h4:target{
	padding:5px;
	webkit-border-radius:5px;
	border-radius:5px;
	-moz-border-radius:5px;
}
.showMore:active{
	background:#FFF4D2;
	webkit-border-radius:5px;
	border-radius:5px;
	-moz-border-radius:5px;
}
h4:target + div > p{
	background:#37aee4;
	color:white;
	padding:5px;
	webkit-border-radius:2px;
	border-radius:2px;
	-moz-border-radius:2px;
}
h4:target + div > p > a{
	color:#ccc;
}
.thumbnail{
	border:none;
	box-shadow:none;
	-moz-box-shadow:none;
}


/* end mall mvc---------------------------------------------------------------------------------*/

/* begin upload mvc---------------------------------------------------------------------------------*/

/*#dropzone {
    background: palegreen;
    width: 150px;
    height: 50px;
    line-height: 50px;
    text-align: center;
    font-weight: bold;
}
#dropzone.in {
    width: 600px;
    height: 200px;
    line-height: 200px;
    font-size: larger;
}
#dropzone.hover {
    background: lawngreen;
}
#dropzone.fade {
    -webkit-transition: all 0.3s ease-out;
    -moz-transition: all 0.3s ease-out;
    -ms-transition: all 0.3s ease-out;
    -o-transition: all 0.3s ease-out;
    transition: all 0.3s ease-out;
    opacity: 1;
}*/

#uploadProgressAll{
	width:200px;
}
.well .icon-move{
	font-size:14px;
	color:#cccccc;
	float:right;
}
#titleDiv{
	width:100%;
	height:30px;
}
#descriptionDiv{
	width:100%;
	height:60px;
}
div.newDiv{
 display: none;
}​
div#count{
	width:600px;
	height:600px;
}
.fileupload{
	width:15%;
}
#uploadTable tbody tr:hover td, .table tbody tr:hover th {
	background:	#F7F9FC;
}
#uploadTable tbody tr:hover td, .table tbody tr:hover th {
	background:	#F7F9FC;
}
#uploadTable th, .table td {
border-top: 1px solid #D2D9E7;
}
/* end upload MVC--------------------------------------------------------------------------------*/

/* begin voting MVC--------------------------------------------------------------------------------*/

.parentUlVoting li{
    /*-webkit-transition: all .4s linear;
    -o-transition: all .4s linear;
    -moz-transition: all .4s linear;
    -ms-transition: all .4s linear;
    -kthtml-transition: all .4s linear;
    transition: all .4s linear;*/
}
.parentUlVoting li.frontpage-notification:hover{
	/*background: #B9E6D5;*/
	/*	-moz-transition-property: background-color;
-webkit-transition-property: background-color;
-o-transition-property: background-color;
transition-property: background-color;
-moz-transition-duration: .4s;
-webkit-transition-duration: .4s;
-o-transition-duration: .4s;
transition-duration: .4s;*/

}
.voteTots{
	font-size:72px;
}
.greenShadow{
    -webkit-box-shadow: inset 0px 0px 1px 1px rgba(185, 230, 213, 0.7) !important; /*  #B9E6D5 rgba(185, 230, 213, 0.5);*/
    -moz-box-shadow: inset 0px 0px 1px 1px rgba(185, 230, 213, 0.7) !important;
    box-shadow: inset 0px 0px 1px 1px rgba(185, 230, 213, 0.7) !important;
    -webkit-transition: all .2s linear !important;
    -o-transition: all .2s linear !important;
    -moz-transition: all .2s linear !important;
    -ms-transition: all .2s linear !important;
    -kthtml-transition: all .2s linear !important;
    transition: all .2s linear !important;
}

.iVoted{
	color: #468847;
	background-color: #DFF0D8;
	border-color: #D6E9C6;	
	background: #DFF0D8;
	/*border: 1px solid #468847;*/
	box-shadow: inset 0 0 3px 3px #468847;
	box-shadow: inset 0 0 2px 2px #B9E6D5 !important;
	/* from http://imsky.github.com/holder/*/
}

.voteTots{
	font-size:48px;
    /*line-height:72px;*/
    padding-top:15px;
    padding-bottom:15px;
}

i.voteLighter {
	font-size:60px;
	line-height:36px;
	
	margin:auto;
	width:95%;
    /*margin: 5px 0 5px 0;
    border-radius:5px;
    -webkit-box-shadow: 0 2px 4px #ADAEB0;
    -moz-box-shadow:  0 2px 4px  #ADAEB0;
    box-shadow:  0 2px 4px #ADAEB0;*/
}
i.voteLighter:active {
   /* border-radius:5px;
    -webkit-box-shadow: inset 0 2px 4px #ADAEB0;
    -moz-box-shadow: inset 0 2px 4px  #ADAEB0;
    box-shadow: inset 0 2px 4px #ADAEB0;*/
}
/* end voting MVC--------------------------------------------------------------------------------*/

/* begin footer---------------------------------------------------------------------------------*/	
#back-top {
	bottom: 27px;
	margin: auto;
	width:50%;
}
#back-top a {
	width: 54px;
	display: block;
	text-align: center;
	margin:auto;
	-webkit-transition: 1s;
	-moz-transition: 1s;
	transition: 1s;
}
#back-top i {
	margin-bottom: 7px;
}
#back-top i:hover{
	color:gray;
	-webkit-transition: 1s;
	-moz-transition: 1s;
	transition: 1s;
}
/* end footer--------------------------------------------------------------------------------------*/

/* begin settings ---------------------------------------------------------------------------------*/
/*#settingsDropdown{
	top:-50px;
}*/
/*#settingsDropdown li{
	padding:5px;
	margin:5px;
}*/
#edit-notifications i{
font-size:48px;
float:right;
}
/* end settings -----------------------------------------------------------------------------------*/

/* begin modals -----------------------------------------------------------------------------------*/

#purchaseModal {	
	padding:10px; 
	border:none;
	border-radius:6px;-moz-border-radius:6px;
}
#purchaseModal.btn{
	text-shadow:none;
}
.purchaseModalTitle{
	font-weight:bold;
}
.purchaseHeader { 
	padding: .4em 1em; position: relative; border-color:#708090;
	border-radius:3px;-moz-border-radius:3px;
	cursor: default; background:#708090; height:40px;		
	background: rgb(205,205,205); 
	background: -moz-linear-gradient(top, rgba(205,205,205,1) 0%, rgba(218,210,205,1) 50%, rgba(200,193,193,1) 51%, rgba(205,205,205,1) 100%); /* FF3.6+ */
	background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,rgba(205,205,205,1)), color-stop(50%,rgba(62,96,144,1)), color-stop(51%,rgba(218,210,205,1)), color-stop(100%,rgba(200,193,193,1))); /* Chrome,Safari4+ */
	background: -webkit-linear-gradient(top, rgba(205,205,205,1) 0%,rgba(218,210,205,1) 50%,rgba(200,193,193,1) 51%,rgba(205,205,205,1) 100%); /* Chrome10+,Safari5.1+ */
	background: -o-linear-gradient(top, rgba(205,205,205,1) 0%,rgba(218,210,205,1) 50%,rgba(200,193,193,1) 51%,rgba(205,205,205,1) 100%); /* Opera11.10+ */
	background: -ms-linear-gradient(top, rgba(205,205,205,1) 0%,rgba(218,210,205,1) 50%,rgba(200,193,193,1) 51%,rgba(205,205,205,1) 100%); /* IE10+ */
	filter: progid:DXImageTransform.Microsoft.gradient( startColorstr='#365d7c', endColorstr='#315b7e',GradientType=0 ); /* IE6-9 */
	background: linear-gradient(top, rgba(205,205,205,1) 0%,rgba(218,210,205,1) 50%,rgba(200,193,193,1) 51%,rgba(205,205,205,1) 100%);/* W3C */
	border-color:#708090;
}
.modalLabel{
	font-size:16px;
	color:#F7F9FC;
}
/* end modals ----------------------------------------------------------------------------------------*/

/* begin general UX/UI---------------------------------------------------------------------------------*/
.wordWrap{
	/*-ms-word-break: break-all;
     word-break: break-all;

     // Non standard for webkit
     word-break: break-word;*/
word-break:hyphenate;
-webkit-hyphens: auto;
   -moz-hyphens: auto;
        hyphens: auto;
}
.highlight { 
	font-weight:bold; 
	background: #FFF4D2;
	background:rgba(255,244,210, 0.5);

}
.boldLink a {
	font-weight:bold;
}
#mailCheckSuggestion{
	background:#F7F9FC;
	border: 1px solid #CCC;
	border-radius: 5px;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	padding: 5px;		
}
.mailcheckSuggestion{
	cursor:pointer;
}
.notClickable:active, i.icon-green:active{
	position:relative;
	top:0px;
}

.dateFormat{
	color:#93989E;
	font-size:14px;
}
#modalPurchaseBody{
	/*height:280px;*/
}
.fastclick, .ajaxLink {
	font-size:16px;	
	color:#315B7E;
}
a, a:hover, .fastclick:hover, .ajaxLink:hover{
	color: #08C;	
color: #4183C4; /* in RGB border-color: rgba(65, 131, 196, 0.5); /*Github .ajaxLink color*/
/* bootstrap badge-info #3A87AD; */

	text-decoration:none;
}
.toggler{
	padding:2px;
	/*border-radius:4px;*/
}
.toggler:hover, .toggler:active{
background: rgba(65, 131, 196, 0.01);
}

.hidden0 {
    display: none;
}

/* see http://stackoverflow.com/questions/4997493/set-opacity-of-background-image-without-affecting-child-elements 
and
http://localhost/opacity.php and http://www.impressivewebs.com/css-opacity-that-doesnt-affect-child-elements/
*/
.whiteWash{
margin:1%;
	border-radius:5px;	
	background: rgba(255,255,255,.5);
}
#main_content{
	min-height:600px;
	position:relative;
	top:0px;	
}
.listSort{
	background:#F7F9FC;
}
.highlighted{
	background:#FFF4D2;
	background:rgba(255,244,210, 0.5);
}
.disabledLink:hover{
    text-decoration:none;
}
/*beging iosslider*/
/* slider container */
.iosSlider {
	/* required */
	position: relative;
	top: 0;
	left: 0;
	overflow: hidden;	
	width: 300px;
	height: 200px;
}
/* slider */
.iosSlider .slider {
	/* required */
	width: 100%;
	height: 100%;
}
/* slide */
.iosSlider .slider .slide {
	/* required */
	float: left;
	width: 300px;
	height: 200px;
}
/*end iosslider*/

i.icon-repeat, .glyph-refresh {
  transform-origin: 47% 41%;
  -o-transform-origin:47% 41%;
  -ms-transform-origin:47% 41%;
  -moz-transform-origin:47% 41%;
  -webkit-transform-origin:47% 41%;

	/*-webkit-animation: Rotate 500ms infinite linear;
	-moz-animation: Rotate 500ms infinite linear;
	-ms-animation: Rotate 500ms infinite linear;
	-o-animation: Rotate 500ms infinite linear;
	animation: Rotate 500ms infinite linear;*/

	-webkit-animation: Rotate 250ms infinite linear;
	-moz-animation: Rotate 250ms infinite linear;
	-ms-animation: Rotate 250ms infinite linear;
	-o-animation: Rotate 250ms infinite linear;
	animation: Rotate 250ms infinite linear;

}
@-o-keyframes Rotate {
  from {-o-transform:rotate(0deg);}
  to {-o-transform:rotate(360deg);}
}
@-moz-keyframes Rotate {
  from {-moz-transform:rotate(0deg);}
  to {-moz-transform:rotate(360deg);}
}
@-ms-keyframes Rotate {
  from {-ms-transform:rotate(0deg);}
  to {-ms-transform:rotate(360deg);}
}
@-webkit-keyframes Rotate {
  from {-webkit-transform:rotate(0deg);}
  to {-webkit-transform:rotate(360deg);}
}
@-keyframes Rotate {
  from { transform:rotate(0deg);}
  to { transform:rotate(360deg);}
}
#iconRepeatMain{
	display: inline-block;	
    position: absolute;
	z-index:10007;
    /*left: 50%;
    top: 50%;
    margin-left: -18px; /* -1 * image width / 2 */
   /* margin-top: -18px;  /* -1 * image height / 2 */
    /*width: 26px;
    height: 19px;*/
    padding:0px;
}
.boxShadow{
	-moz-box-shadow: 0 0 3px gray;
	-webkit-box-shadow: 0 0 3px gray;
	box-shadow: 0 0 3px gray;
}

.boxShadow5{
	-moz-box-shadow: 0 0 5px gray;
	-webkit-box-shadow: 0 0 5px gray;
	box-shadow: 0 0 5px gray;
}

.boxShadowInset5{
	-moz-box-shadow: inset 0 0 5px gray;
	-webkit-box-shadow: inset 0 0 5px gray;
	box-shadow: inset 0 0 5px gray;
}
.boxShadow6{
		-webkit-box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 6px rgba(0, 0, 0, 0.31);
	-moz-box-shadow: 0 0 1px rgba(0,0,0,0.1),0 2px 6px rgba(0,0,0,0.31);
	box-shadow: 0 0 1px rgba(0, 0, 0, 0.1),0 2px 6px rgba(0, 0, 0, 0.31);
}
.nicetime{
	font-size:14px;
	color:gray; 
	font-weight:normal;
	/*#808080*/
}
.char-count {
	height:28px;
	font-size: 14px;
	font-weight: 600;
	line-height: 28px;
	color: rgba(0, 0, 0, 0.2);
	text-shadow: 0 1px 0 white;
	/*taken from alpha.app.net character count*/
}
.500characterInput:focus{
	border:0px;

}
.screenShot{
	width:90%;
	height:90%;
}
.clear{
	clear:both;
}
.rad3{
	border-radius:3px;
	-moz-border-radius:3px;
}
.rad5{
	border-radius:5px;
	-moz-border-radius:5px;
}
.rad6{
	border-radius:6px;
	-moz-border-radius:6px;
}
.noUnderline:hover{
	text-decoration:none;
}
.textMiddle{
	vertical-align:middle;
}
.pointer{
	cursor:pointer;
}
.stackingElements, .positionAbsolute{
	position:absolute;
}
.pad0{
	padding:0px;
}
.pad5{
padding:5px;
}
.pad10{
padding:10px;
}
.pad20{
padding:20px;
}
.pad5Top{
	padding-top:5px;
}
.pad10Top{
	padding-top:10px;
}
.pad20Top{
	padding-top:20px;
}
.marg0{
	margin:0px;
}
.marg5{
	margin:5px;
}
.marg10{
	margin:10px;
}
.marg20{
	margin:20px;
}
.marg10Left{
	width:92.5%;
	margin-left:auto;
}
.marg5Top{
	margin-top:5px;
}
.marg10Top{
	margin-top:10px;
}
.marg10Bottom{
	margin-bottom:10px;
}
.marg20Top{
	
	margin-top:20px;
}

.mini{
	background-size:16px 16px;height:16px; width:16px;
 }
.small{
	background-size:40px 40px; height:40px; width:40px;
}
.medium{ 
	background-size:125px 125px; height:125px; width:125px;
}
.large{
	background-size:260px 180px; height:180px; width:260px
}
.xlarge{
	background-size:360px 268px; height:268px; width:360px
}

.rect4020{
	width:40px;
	height:20px;
}
.sq40{
	width:40px;
	height:40px;
}
.touchable{
	/*border:1px solid red;*/
	width:70px;
	height:70px;
	position:absolute;
	z-index:10011 !important;

}
/*.touchable * {
	margin:auto;
	color:green;
}*/
.rect5010{
	width:50px;
	height:10px;
}

 .noHover, .noHover:hover, .noHover:active{
	font-size:18px;	
	color:#666;	
	font-family: 'ProximaNova-Regular', "Helvetica Neue", Arial, Helvetica, sans-serif;
}
.alignBottom{
	vertical-align:bottom;
}
.lucida {
 	font-family:"lucida grande",tahoma,verdana,arial,sans-serif;
 }
.overflowScroll{
	overflow:scroll;
}
.inline{
	display:inline;
}
.inlineBlock{
	display:inline-block;	
}
.block{
	display:block;
}
ul, .listNone{
	list-style:none;
	margin: 0px 0px 3px 0px;
}
ul .hashable{
	list-style:disc;
}
.textCenter{
	text-align:center;
}
.centerIt{
	margin:auto;
	width:50%;
}
.opacity05{
	opacity:0.5;
}
img.alignTop {
	vertical-align:text-top;
}
.alignMiddle {
	vertical-align:middle;
}
.visi{
	visibility:hidden;
}
.disi{
	display:none;
}
.icon-purple, .icon-purple:hover, .icon-purple:hover {
	color:#455093;
}
.icon-gray, .icon-noHover{	
	color:#cccccc;	
	/*text-shadow: 0 1px 0 white;*/
}
.icon-lightestBlue{
	color:#F7F9FC;	
}
.icon-lightGrey{
	color:lightGrey;
}
.icon-darkGray{
	color:gray;
}
.icon-red, .error{
	color:#DC143C;
	color:#B94A48; /*orangey-red*/
}
.icon-fbGray {
	color:#D2D9E7;
}
i.icon-fbGray{
	
}
.textShadow{
	text-shadow:0px 1px 1px gray;
	-webkit-transition: color 0.2s ease-in-out;
text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.35);
}
.text-shadow2{
	text-shadow: 0px 1px 0px rgba(255, 255, 255, .3), 0px -1px 0px rgba(0, 0, 0, .7);
} 
.icon-green{
	color:#58A279;	
  color:#389ba4; /*dwolla*/
}
i.icon-green.icon-thumbs-up:hover, i.icon-green.icon-thumbs-up:active {
	color:#58A279;
	position: relative;
	top: 0px;	
}
.icon-blue{
	color:#315B7E;
}
.icon-buttonBlue{
	color:#006DCC;
	color:#4183C4;
}
.icon-lighterYellow{
	color:#FFF4D2; /*rgba(255,244,210, 1);*/
}
.bg-lighterYellow{
  background:#FFF4D2; /*rgba(255,244,210, 1);*/
}
.icon-yellow{	
	color:#FFC14D;
}
i.icon-yellow:hover, i.icon-yellow:active{
	color:#FFC14D;
}
.icon-white{
	color:white;	
}
i.icon-white:hover, i.icon-white:active{
	color:#F7F9FC;	
}
.icon-voted{
color: #4183C4;
opacity: 0.7;
}
/*i{
	font-size:24px;
}
*/
i.i{
	font-size:14px;color:gray;
}
.txtBold{
	font-weight:bold;

}
.txtUnderline{
	text-decoration:underline;
}
.txt8{
	font-size:8px;
}
.txt10{
	font-size:10px;
}
.txt12{
	font-size:12px;
}
.txt14{
	font-size:14px;
}
.txt15{
	font-size:15px;
}

.txt16{
	font-size:16px;
}

.txt18{
	font-size:18px;
}
.txt19{
	font-size:19px;
}
.txt20{
	font-size:20px;
}
.txt24{
	font-size:24px;
}
.txt26{
	font-size:26px;
}
.txt28{
	font-size:28px;
}
.txt30{
	font-size:30px;
}
.txt32{
	font-size:32px;
}
.txt36{
	font-size:36px;
}
.txt48{
	font-size:48px;
}
.txt60{
	font-size:60px;
}
.txt72{
	font-size:72px;
}
.txt90{
	font-size:90px;
}
.txt180{
	font-size:180px;
}
img.miniThumb{
	width:25px;
	height:25px;
}
img.smallThumb{
	width:40px;
	height:40px;
}
img.largeImg{
	height:120px;
	width:120px;	
}
.roundedSpanMini {
	float: left;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	background-size:25px 25px;
}
.roundedSpanSm {
	float: left;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	background-size:40px 40px;
}
.roundedSpanMed {
	float: left;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	background-size:40px 40px;	
	-webkit-box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);
	-moz-box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);
	box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);	
}
.roundedSpanLrg {
	float: left;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	background-size:125px 125px;
}
.roundedSpanXLrg {
	float: left;
	-webkit-border-radius: 5px;
	-moz-border-radius: 5px;
	border-radius: 5px;
	background-size:260px 180px;
	background-repeat:no-repeat;
}
.roundedImgMini{
	vertical-align: bottom;
	opacity: 0;
	width:25px;
	height:25px;
}
.roundedImgSm{
	vertical-align: bottom;
	opacity: 0;
	width:40px;
	height:40px;
}
.roundedImgMed {
	vertical-align: bottom;
	opacity: 0;
	width:40px;
	height:40px;
}
.roundedImgLrg {
	vertical-align: bottom;
	opacity: 0;
	width:125px;
	height:125px;
}
.roundedImgXLrg {
	vertical-align: bottom;
	opacity: 0;
	width:260px;
	height:180px;
}
/*CIRCLE*/
.circle .image-wrap {
	-webkit-border-radius: 50em;
	-moz-border-radius: 50em;
	border-radius: 50em;
}
/**PHOTO*/
.card .image-wrap {
	-webkit-box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);
	-moz-box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);
	box-shadow: inset 0 0 1px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -1px 0 rgba(0,0,0,.4);
	-webkit-border-radius: 10px;
	-moz-border-radius: 10px;
	border-radius: 10px;
}

/**EMBOSSED*/
.light-embossed .image-wrap {
  -webkit-box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -2px 0 rgba(0,0,0,.6), inset 0 -3px 0 rgba(255,255,255,.3);
  -moz-box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -2px 0 rgba(0,0,0,.6), inset 0 -3px 0 rgba(255,255,255,.3);
  box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -2px 0 rgba(0,0,0,.6), inset 0 -3px 0 rgba(255,255,255,.3);
  -webkit-border-radius: 10px;
  -moz-border-radius: 10px;
  border-radius: 10px;
}

/**EMBOSSED*/
.embossed .image-wrap {
	-webkit-box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -7px 0 rgba(0,0,0,.6), inset 0 -9px 0 rgba(255,255,255,.3);
	-moz-box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -7px 0 rgba(0,0,0,.6), inset 0 -9px 0 rgba(255,255,255,.3);
	box-shadow: inset 0 0 2px rgba(0,0,0,.8), inset 0 2px 0 rgba(255,255,255,.5), inset 0 -7px 0 rgba(0,0,0,.6), inset 0 -9px 0 rgba(255,255,255,.3);
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
}
/**SOFT EMBOSSED*/
.soft-embossed .image-wrap {
	-webkit-box-shadow: inset 0 0 4px rgba(0,0,0,1), inset 0 2px 1px rgba(255,255,255,.5), inset 0 -9px 2px rgba(0,0,0,.6), inset 0 -12px 2px rgba(255,255,255,.3);
	-moz-box-shadow: inset 0 0 4px rgba(0,0,0,1), inset 0 2px 1px rgba(255,255,255,.5), inset 0 -9px 2px rgba(0,0,0,.6), inset 0 -12px 2px rgba(255,255,255,.3);
	box-shadow: inset 0 0 4px rgba(0,0,0,1), inset 0 2px 1px rgba(255,255,255,.5), inset 0 -9px 2px rgba(0,0,0,.6), inset 0 -12px 2px rgba(255,255,255,.3);
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
}
/**CUTOUT*/
.cutout {
	color: white;
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
}
.cutout .image-wrap {
	-webkit-box-shadow: 0 1px 0 rgba(255,255,255,.2), inset 0 4px 5px rgba(0,0,0,.6), inset 0 1px 0 rgba(0,0,0,.6);
	-moz-box-shadow: 0 1px 0 rgba(255,255,255,.2), inset 0 4px 5px rgba(0,0,0,.6), inset 0 1px 0 rgba(0,0,0,.6);
	box-shadow: 0 1px 0 rgba(255,255,255,.2), inset 0 4px 5px rgba(0,0,0,.6), inset 0 1px 0 rgba(0,0,0,.6);
	-webkit-border-radius: 30em;
	-moz-border-radius: 30em;
	border-radius: 30em;
}
/**MORPHING + GLOWING*/
.glowing {
	color: #fff;
	-webkit-border-radius: 50em;
	-moz-border-radius: 50em;
	border-radius: 50em;
}
.glowing .image-wrap {
	-webkit-transition: 1s;
	-moz-transition: 1s;
	transition: 1s;
	-webkit-border-radius: 50em;
	-moz-border-radius: 50em;
	border-radius: 50em;
}
.glowing .image-wrap:hover {
	-webkit-box-shadow: 0 0 20px rgba(255,255,255,.6), inset 0 0 20px rgba(255,255,255,1);
	-moz-box-shadow: 0 0 20px rgba(255,255,255,.6), inset 0 0 20px rgba(255,255,255,1);
	box-shadow: 0 0 20px rgba(255,255,255,.6), inset 0 0 20px rgba(255,255,255,1);
	-webkit-border-radius: 50em;
	-moz-border-radius: 50em;
	border-radius: 50em;
}
/**GLOSSY*/
.glossy {
	color: white;
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
}
.glossy .image-wrap {
	-webkit-box-shadow: inset 0 -1px 0 rgba(0,0,0,.5);
	-moz-box-shadow: inset 0 -1px 0 rgba(0,0,0,.5);
	box-shadow: inset 0 -1px 0 rgba(0,0,0,.5);
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
}
.glossy .image-wrap::after {
	position: absolute;
	content: ' ';
	width: 100%;
	height: 50%;
	top: 0;
	left: 0;
	-webkit-border-radius: 20px;
	-moz-border-radius: 20px;
	border-radius: 20px;
	background: -moz-linear-gradient(top, rgba(255, 255, 255, 0.7) 0%, rgba(255, 255, 255, .1) 100%);
	background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,rgba(255, 255, 255, 0.7)), color-stop(100%,rgba(255, 255, 255, .1)));
	background: linear-gradient(top, rgba(255, 255, 255, 0.7) 0%,rgba(255, 255, 255, .1) 100%);
}
/**END of image styles**/

.txtButtonBlue {
	color:#006DCC;
	/*color: #4183C4;*/ /*ajaxLink txt*/
}
.txtGray, .resetFormBtn{
	color:#93989E;
	color:gray;
}
.txtBlue {
	color: #315B7E;	
}
.txtGreen {
	color: #58A279;	
  color:#389ba4;
}
.txtRed {
	color: #DC143C;
}
.txtLightestBlue{
	color:#F7F9FC;
}


.oRoyalBlue {
	background: #0283E0;
	color: #F7F9FC;
}
.oGreen {
	color: #58A279;
	background: #58A279;
}
.oRed {
	color: #DC143C;
	background: #DC143C;
}
.oBlue {
	color: #315B7E;
	background: #315B7E;
}
.oYellow {
	color: #FFC14D;
	background: #FFC14D;
}
.oBrown {
	color: #8B4513;
	background: #8B4513;
}	

/* stripe  ".well"-like 
	div.box {

	-moz-border-radius: 4px;
	-webkit-border-radius: 4px;
	border-radius: 4px;
	-webkit-box-shadow: 0 0 1px 
	rgba(0, 0, 0, 0.1),0 2px 6px 
	rgba(0, 0, 0, 0.31);
	-moz-box-shadow: 0 0 1px rgba(0,0,0,0.1),0 2px 6px rgba(0,0,0,0.31);
	box-shadow: 0 0 1px 
	rgba(0, 0, 0, 0.1),0 2px 6px 
	rgba(0, 0, 0, 0.31);
}
*/
/* end general UX/UI---------------------------------------------------------------------------------*/

/* begin mediaQueries ---------------------------------------------------------------------*/
@media (min-width: 979px){	
	.hidden-phone{
		display:inline;
	}
	/*input{
		width:350px;		
	}
	textarea{
		width:450px;
		height:100px;
	}*/
}
@media (min-width: 768px){
	#libraryNavbar{
		/*width:67.5%;*/
	}
	.marg10Top10Bottom{
	margin-top:5px ! important;
	}
}
@media (max-width: 767px) {	
	#libraryNavbar{
		/*width:91.0%;
		width:82.5%;*/	
	}			
	body {
		padding-left:5px;	
		padding-right:5px;
	}		
	ul > li.span4{ 
		width:404px;
	}
	/*input, textarea {
		width:240px;
	}
	textarea{
		height:100px;
	}*/
} 
@media (min-width: 768px) and (max-width: 979px) {	
	.navbar-fixed-top {
		position: fixed;
		margin-bottom: 18px;
	}	
	/*input{
		width:300px;
	}
	textarea{
		width:400px;
		height:100px;
	}*/
}
@media (max-width: 480px) {
	.media-bar ul li a i, .media-bar ul li form, .media-bar ul li i {
		margin-top: 0px;
	}
	.media-bar ul li a{
		padding: 5px 0px 5px 0px;
		padding:0px;
	}
	/*#settingsDropdown ul li i{
		margin: 6px;
	}*/
	.ajaxLink{ 
		-webkit-tap-highlight-color:rgba(0,0,0,0);
		-webkit-tap-highlight-color:#F7F9FC;
		-webkit-tap-highlight-color:#8EC8FF;
		-webkit-tap-highlight-color: rgba(65,131,196, 0.3);
		/*-webkit-touch-callout: none;*/
		/* eliminate translucent gray highlight mobile Safari puts on links and buttons when they are clicked
		https://github.com/twitter/bootstrap/issues/3772 */
	}
	.container-fluid {
		padding-left: 0px;
		padding-right: 0px;
	}
	#searchCaretBtn{
		height: 30.47618865966797px;
	}
	#galleria{
		width:100%;margin:0 0%; 
	}
	#main_content{
		padding:0px;
	}
	.media-bar ul li a i, .media-bar ul li form, .media-bar ul li i {
		margin-top: 0px;
	}
	.marg10Top10Bottom{
		margin:10px 5px 10px 5px ! important;
	}	

}
/* end mediaQueries-------------------------------------------------------------- */

.glyphicons i::before {

color: #315B7E;
font-size: 36px;
}

.glyphicons i::after{

}


/*  Font Awesome styles
    ------------------------------------------------------- */


[class^="glyph-"], [class*=" glyph-"] {
  display: inline;
  width: auto;
  height: auto;
  line-height: inherit;
  vertical-align: baseline;
  background-image: none;
  background-position: 0% 0%;
  background-repeat: repeat;
}
[class^="glyph-"]:before, [class*=" glyph-"]:before  {
  font-family: Glyphicons;
  font-weight: normal;
  font-style: normal;
  display: inline-block;
  text-decoration: none;
  /*font-size:36px;*/
}
a [class^="glyph-"], a [class*=" gylph-"] {
  display: inline-block;
  text-decoration: none;
}
/*.glyphicons i::before:hover, .glyphicons i::before:active {
position: absolute;

}*/

/*[class^="icon1-"], [class*=" icon1-"] {
  display: inline-block;
  width: 28px;
  height: 28px;
  line-height: 28px;
  vertical-align: text-top;
  background-image: url("/assets/img/glyphiconsblue.png");
  background-position: 28px 28px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}
[class^="icon1-"]:last-child, [class*=" icon1-"]:last-child {
  *margin-left: 0;
}

[class^="icont-"], [class*=" icont-"] {
  display: inline-block;
  width: 14px;
  height: 14px;
  line-height: 14px;
  vertical-align: text-top;
  background-image: url("/assets/img/glyphicons2xteallight.png");
  background-position: 14px 14px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}
[class^="icont-"]:last-child, [class*=" icont-"]:last-child {
  *margin-left: 0;
}

.icont-unlock {
  background-position: 0 -2px;
}

[class^="iconv-"], [class*=" iconv-"] {
  display: inline-block;
  width: 28px;
  height: 28px;
  line-height: 28px;
  vertical-align: text-top;
  background-image: url("/assets/img/glyphiconsinvert.png");
  background-position: 28px 28px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}
[class^="iconv-"]:last-child, [class*=" iconv-"]:last-child {
  *margin-left: 0;
}
.iconv-play{
	background-position: -144px   -816px;
}
.iconv-pause{
 	background-position: -192px   -816px;
}
.iconv-resize-full{
	background-image: url("/assets/img/glyphiconsinvert.png");
 	background-position: -240px   -1004px;
}
.iconv-resize-small{
	background-image: url("/assets/img/glyphiconsinvert.png");
 	background-position: -192px   -1004px;
}
.iconv-volume-off {	
	background-image: url("/assets/img/glyphiconsinvert.png");
 	background-position: -96px   -864px;
}
.iconv-volume-down {	
	background-image: url("/assets/img/glyphiconsinvert.png");
 	background-position: -144px   -864px;
}
.iconv-volume-up {	
 	background-image: url("/assets/img/glyphiconsinvert.png");
 	background-position: -192px   -864px;
}
.iconv-horz{
	  background-position: -240px   -720px;
}
.iconv-vert{
	 background-position: -192px    -528px;
}
.iconv-counts{
	 background-position: 0px    -1056px;
}
.iconv-trophy{
	 background-position: -192px    -336px;
}
.iconv-coins{
	 background-position: -336px    -144px;
}
.iconv-chart{
	 background-position: -48px    -192px;
}
.iconv-list{
	 background-position: -192px    -528px;
}

[class^="iconv2-"], [class*=" iconv2-"] {
  display: inline-block;
  width: 14px;
  height: 14px;
  line-height: 14px;
  vertical-align: text-top;
  background-image: url("/assets/img/glyphiconsinvert2.png");
  background-position: 14px 14px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}
.iconv2-trophy{
	background-position: -96px    -168px;
}
.icon1-glass {
  background-position: 0    0;
}
.icon1-leaf {
  background-position: -48px 0;
}
.icon1-dog {
  background-position: -96px 0;
}
.icon1-user {
  background-position: -144px 0;
}
.icon1-girl {
  background-position: -192px 0;
}
.icon1-car {
  background-position: -240px 0;
}
.icon1-plus-user {
  background-position: -288px 0;
}
.icon1-minus-user {
  background-position: -336px 0;
}
.icon1-film {
  background-position: -384px 0;
}
.icon1-magic {
  background-position: -432px 0;
}
.icon1-envelope {
  background-position: 0px -48px;
}
.icon1-camera {
  background-position: -48px -48px;
}
.icon1-heart {
  background-position: -96px -48px;
}
.icon1-umbrella {
  background-position: -144px -48px;
}
.icon1-train {
  background-position: -192px -48px;
}
.icon1-print {
  background-position: -240px -48px;
}
.icon1-trash {
  background-position: -288px -48px;
}
.icon1-notes {
  background-position: -336px -48px;
}
.icon1-note {
  background-position: -384px -48px;
}
.icon1-settings {
  background-position: -432px -48px;
}
.icon1-home {
  background-position: 0px -96px;
}
.icon1-snowflake {
  background-position: -48px -96px;
}
.icon1-fire {
  background-position: -96px -96px;
}
.icon1-gears {
  background-position: -144px -96px;
}
.icon1-users {
  background-position: -192px -96px;
}
.icon1-binoculars {
  background-position: -240px -96px;
}
.icon1-road {
  background-position: -288px -96px;
}
.icon1-search {
  background-position: -336px -96px;
}
.icon1-cars {
  background-position: -384px -96px;
}
.icon1-clipboard {
  background-position: -432px -96px;
}
.icon1-notify {
  background-position: -432px -1536px;
}
.icon1-pencil {
  background-position: 0px -144px;
}
.icon1-bus {
  background-position: -48px -144px;
}
.icon1-radio {
  background-position: -144px -144px;
}
.icon1-luggage {
  background-position: -144px -144px;
}
.icon1-man {
  background-position: -192px -144px;
}
.icon1-woman {
  background-position: -240px -144px;
}
.icon1-paper {
  background-position: -288px -144px;
}
.icon1-coins {
  background-position: -336px -144px;
}
.icon1-plane {
  background-position: -384px -144px;
}
.icon1-flipboard {
  background-position: -432px -144px;
}
.icon1-upload{
	background-position: -48px -959px;
}
.icon1-shopping-cart{
	background-position: -96px -960px;
}
.icon1-gift{
	background-position: -432px -288px;
}
.icon1-gift{
	background-position: -144px -143px;
}
.icon1-line-chart{
	background-position: 0px -192px;
}
.icon1-magic-wand{
	background-position: -432px 0px;
}
.icon1-counts{	
	 background-position: 0px    -1056px;
}
.icon1-trophy{	
	 background-position: -192px    -336px;
}
.icon1-coins{
	 background-position: -336px    -144px;
}
.icon1-chart{	
	 background-position: -48px    -192px;
}
.icon1-list{
	 background-position: -192px    -528px;
}
*/


